@@ -1,10 +1,16 @@
+from __future__ import annotations
+
+import base64
 import logging
 import math
 import os
+import tempfile
+from pathlib import Path
 from tempfile import SpooledTemporaryFile
 from unittest import mock
 
 import pytest
+from pdf2image.exceptions import PDFPageCountError
 from PIL import Image
 from unstructured_inference.inference import layout
 
@@ -13,14 +19,19 @@ from unstructured.chunking.title import chunk_by_title
 from unstructured.documents.coordinates import PixelSpace
 from unstructured.documents.elements import (
     CoordinatesMetadata,
+    Element,
     ElementMetadata,
+    ElementType,
+    Footer,
+    Header,
     ListItem,
     NarrativeText,
     Text,
     Title,
 )
-from unstructured.partition import ocr, pdf, strategies
+from unstructured.partition import pdf, strategies
 from unstructured.partition.pdf import get_uris_from_annots
+from unstructured.partition.pdf_image import ocr, pdfminer_processing
 from unstructured.partition.utils.constants import (
     UNSTRUCTURED_INCLUDE_DEBUG_METADATA,
     PartitionStrategy,
@@ -109,6 +120,16 @@ def test_partition_pdf_local(monkeypatch, filename, file):
         "process_file_with_model",
         lambda *args, **kwargs: MockDocumentLayout(),
     )
+    monkeypatch.setattr(
+        pdfminer_processing,
+        "process_data_with_pdfminer",
+        lambda *args, **kwargs: MockDocumentLayout(),
+    )
+    monkeypatch.setattr(
+        pdfminer_processing,
+        "process_file_with_pdfminer",
+        lambda *args, **kwargs: MockDocumentLayout(),
+    )
     monkeypatch.setattr(
         ocr,
         "process_data_with_ocr",
@@ -116,7 +137,7 @@ def test_partition_pdf_local(monkeypatch, filename, file):
     )
     monkeypatch.setattr(
         ocr,
-        "process_data_with_ocr",
+        "process_file_with_ocr",
         lambda *args, **kwargs: MockDocumentLayout(),
     )
 
@@ -125,25 +146,27 @@ def test_partition_pdf_local(monkeypatch, filename, file):
 
 
 def test_partition_pdf_local_raises_with_no_filename():
-    with pytest.raises(FileNotFoundError):
+    with pytest.raises((FileNotFoundError, PDFPageCountError)):
         pdf._partition_pdf_or_image_local(filename="", file=None, is_image=False)
 
 
 @pytest.mark.parametrize("file_mode", ["filename", "rb", "spool"])
 @pytest.mark.parametrize(
-    ("strategy", "expected", "origin"),
+    ("strategy", "starting_page_number", "expected_page_numbers", "origin"),
     # fast: can't capture the "intentionally left blank page" page
     # others: will ignore the actual blank page
     [
-        (PartitionStrategy.FAST, {1, 4}, {"pdfminer"}),
-        (PartitionStrategy.HI_RES, {1, 3, 4}, {"yolox", "pdfminer"}),
-        (PartitionStrategy.OCR_ONLY, {1, 3, 4}, {"ocr_tesseract"}),
+        (PartitionStrategy.FAST, 1, {1, 4}, {"pdfminer"}),
+        (PartitionStrategy.FAST, 3, {3, 6}, {"pdfminer"}),
+        (PartitionStrategy.HI_RES, 4, {4, 6, 7}, {"yolox", "pdfminer"}),
+        (PartitionStrategy.OCR_ONLY, 1, {1, 3, 4}, {"ocr_tesseract"}),
     ],
 )
-def test_partition_pdf(
+def test_partition_pdf_outputs_valid_amount_of_elements_and_metadata_values(
     file_mode,
     strategy,
-    expected,
+    starting_page_number,
+    expected_page_numbers,
     origin,
     filename=example_doc_path("layout-parser-paper-with-empty-pages.pdf"),
 ):
@@ -152,23 +175,29 @@ def test_partition_pdf(
         # validate that the result is a non-empty list of dicts
         assert len(result) > 10
         # check that the pdf has multiple different page numbers
-        assert {element.metadata.page_number for element in result} == expected
+        assert {element.metadata.page_number for element in result} == expected_page_numbers
         if UNSTRUCTURED_INCLUDE_DEBUG_METADATA:
             assert {element.metadata.detection_origin for element in result} == origin
 
     if file_mode == "filename":
-        result = pdf.partition_pdf(filename=filename, strategy=strategy)
+        result = pdf.partition_pdf(
+            filename=filename, strategy=strategy, starting_page_number=starting_page_number
+        )
         _test(result)
     elif file_mode == "rb":
         with open(filename, "rb") as f:
-            result = pdf.partition_pdf(file=f, strategy=strategy)
+            result = pdf.partition_pdf(
+                file=f, strategy=strategy, starting_page_number=starting_page_number
+            )
             _test(result)
     else:
         with open(filename, "rb") as test_file:
             spooled_temp_file = SpooledTemporaryFile()
             spooled_temp_file.write(test_file.read())
             spooled_temp_file.seek(0)
-            result = pdf.partition_pdf(file=spooled_temp_file, strategy=strategy)
+            result = pdf.partition_pdf(
+                file=spooled_temp_file, strategy=strategy, starting_page_number=starting_page_number
+            )
             _test(result)
 
 
@@ -187,7 +216,40 @@ def test_partition_pdf_with_model_name_env_var(
         assert mock_process.call_args[1]["model_name"] == "checkbox"
 
 
+@pytest.mark.parametrize("model_name", ["checkbox", "yolox", "chipper"])
 def test_partition_pdf_with_model_name(
+    monkeypatch,
+    model_name,
+    filename=example_doc_path("layout-parser-paper-fast.pdf"),
+):
+    monkeypatch.setattr(pdf, "extractable_elements", lambda *args, **kwargs: [])
+    with mock.patch.object(
+        layout,
+        "process_file_with_model",
+        mock.MagicMock(),
+    ) as mock_process:
+        pdf.partition_pdf(
+            filename=filename,
+            strategy=PartitionStrategy.HI_RES,
+            model_name=model_name,
+        )
+        assert mock_process.call_args[1]["model_name"] == model_name
+
+    with mock.patch.object(
+        layout,
+        "process_data_with_model",
+        mock.MagicMock(),
+    ) as mock_process:
+        with open(filename, "rb") as f:
+            pdf.partition_pdf(
+                file=f,
+                strategy=PartitionStrategy.HI_RES,
+                model_name=model_name,
+            )
+            assert mock_process.call_args[1]["model_name"] == model_name
+
+
+def test_partition_pdf_with_hi_res_model_name(
     monkeypatch,
     filename=example_doc_path("layout-parser-paper-fast.pdf"),
 ):
@@ -198,8 +260,26 @@ def test_partition_pdf_with_model_name(
         mock.MagicMock(),
     ) as mock_process:
         pdf.partition_pdf(
-            filename=filename, strategy=PartitionStrategy.HI_RES, model_name="checkbox"
+            filename=filename, strategy=PartitionStrategy.HI_RES, hi_res_model_name="checkbox"
         )
+        # unstructured-ingest uses `model_name` instead of `hi_res_model_name`
+        assert mock_process.call_args[1]["model_name"] == "checkbox"
+
+
+def test_partition_pdf_or_image_with_hi_res_model_name(
+    monkeypatch,
+    filename=example_doc_path("layout-parser-paper-fast.pdf"),
+):
+    monkeypatch.setattr(pdf, "extractable_elements", lambda *args, **kwargs: [])
+    with mock.patch.object(
+        layout,
+        "process_file_with_model",
+        mock.MagicMock(),
+    ) as mock_process:
+        pdf.partition_pdf_or_image(
+            filename=filename, strategy=PartitionStrategy.HI_RES, hi_res_model_name="checkbox"
+        )
+        # unstructured-ingest uses `model_name` instead of `hi_res_model_name`
         assert mock_process.call_args[1]["model_name"] == "checkbox"
 
 
@@ -230,10 +310,12 @@ def test_partition_pdf_with_no_page_breaks(
 def test_partition_pdf_with_fast_strategy(
     filename=example_doc_path("layout-parser-paper-fast.pdf"),
 ):
-    elements = pdf.partition_pdf(filename=filename, url=None, strategy=PartitionStrategy.FAST)
+    elements = pdf.partition_pdf(
+        filename=filename, url=None, strategy=PartitionStrategy.FAST, starting_page_number=3
+    )
     assert len(elements) > 10
     # check that the pdf has multiple different page numbers
-    assert {element.metadata.page_number for element in elements} == {1, 2}
+    assert {element.metadata.page_number for element in elements} == {3, 4}
     for element in elements:
         assert element.metadata.filename == "layout-parser-paper-fast.pdf"
 
@@ -305,7 +387,7 @@ def test_partition_pdf_falls_back_to_fast(
 
     monkeypatch.setattr(strategies, "dependency_exists", mock_exists)
 
-    mock_return = [Text("Hello there!")]
+    mock_return = [[Text("Hello there!")], []]
     with mock.patch.object(
         pdf,
         "extractable_elements",
@@ -327,7 +409,7 @@ def test_partition_pdf_falls_back_to_fast_from_ocr_only(
 
     monkeypatch.setattr(strategies, "dependency_exists", mock_exists)
 
-    mock_return = [Text("Hello there!")]
+    mock_return = [[Text("Hello there!")], []]
     with mock.patch.object(
         pdf,
         "extractable_elements",
@@ -391,7 +473,7 @@ def test_partition_pdf_falls_back_to_ocr_only(
 def test_partition_pdf_uses_table_extraction():
     filename = example_doc_path("layout-parser-paper-fast.pdf")
     with mock.patch(
-        "unstructured.partition.ocr.process_file_with_ocr",
+        "unstructured.partition.pdf_image.ocr.process_file_with_ocr",
     ) as mock_process_file_with_model:
         pdf.partition_pdf(filename, infer_table_structure=True)
         assert mock_process_file_with_model.call_args[1]["infer_table_structure"]
@@ -416,7 +498,8 @@ def test_partition_pdf_hi_table_extraction_with_languages(ocr_mode):
     table = [el.metadata.text_as_html for el in elements if el.metadata.text_as_html]
     assert elements[0].metadata.languages == ["kor"]
     assert len(table) == 2
-    assert "<table><thead><th>" in table[0]
+    assert "<table><thead><tr>" in table[0]
+    assert "</thead><tbody><tr>" in table[0]
     # FIXME(yuming): didn't test full sentence here since unit test and docker test have
     # some differences on spaces between characters
     assert "업" in table[0]
@@ -457,7 +540,8 @@ def test_partition_pdf_hi_res_ocr_mode_with_table_extraction(ocr_mode):
     )
     table = [el.metadata.text_as_html for el in elements if el.metadata.text_as_html]
     assert len(table) == 2
-    assert "<table><thead><th>" in table[0]
+    assert "<table><thead><tr>" in table[0]
+    assert "</thead><tbody><tr>" in table[0]
     assert "Layouts of history Japanese documents" in table[0]
     assert "Layouts of scanned modern magazines and scientific report" in table[0]
     assert "Layouts of scanned US newspapers from the 20th century" in table[0]
@@ -467,7 +551,7 @@ def test_partition_pdf_with_copy_protection():
     filename = os.path.join("example-docs", "copy-protected.pdf")
     elements = pdf.partition_pdf(filename=filename, strategy=PartitionStrategy.HI_RES)
     title = "LayoutParser: A Uniﬁed Toolkit for Deep Learning Based Document Image Analysis"
-    idx = 3
+    idx = 2
     assert elements[idx].text == title
     assert {element.metadata.page_number for element in elements} == {1, 2}
     assert elements[idx].metadata.detection_class_prob is not None
@@ -620,20 +704,28 @@ def test_partition_pdf_exclude_metadata(
     ],
 )
 @pytest.mark.parametrize("last_modification_date", [None, "2020-07-05T09:24:28"])
+@pytest.mark.parametrize("date_from_file_object", [True, False])
 def test_partition_pdf_metadata_date(
     mocker,
     file_mode,
     strategy,
     last_modification_date,
+    date_from_file_object,
     filename=example_doc_path("copy-protected.pdf"),
 ):
     mocked_last_modification_date = "2029-07-05T09:24:28"
     expected_last_modification_date = (
         last_modification_date if last_modification_date else mocked_last_modification_date
     )
+    if not date_from_file_object and not last_modification_date and file_mode != "filename":
+        expected_last_modification_date = None
 
     mocker.patch(
-        "unstructured.partition.pdf.get_the_last_modification_date_pdf_or_img",
+        "unstructured.partition.pdf_image.pdf_image_utils.get_last_modified_date_from_file",
+        return_value=mocked_last_modification_date,
+    )
+    mocker.patch(
+        "unstructured.partition.pdf_image.pdf_image_utils.get_last_modified_date",
         return_value=mocked_last_modification_date,
     )
 
@@ -642,6 +734,7 @@ def test_partition_pdf_metadata_date(
             filename=filename,
             strategy=strategy,
             metadata_last_modified=last_modification_date,
+            date_from_file_object=date_from_file_object,
         )
     elif file_mode == "rb":
         with open(filename, "rb") as f:
@@ -649,6 +742,7 @@ def test_partition_pdf_metadata_date(
                 file=f,
                 strategy=strategy,
                 metadata_last_modified=last_modification_date,
+                date_from_file_object=date_from_file_object,
             )
     else:
         with open(filename, "rb") as test_file:
@@ -659,6 +753,7 @@ def test_partition_pdf_metadata_date(
                 file=spooled_temp_file,
                 strategy=strategy,
                 metadata_last_modified=last_modification_date,
+                date_from_file_object=date_from_file_object,
             )
 
     assert {el.metadata.last_modified for el in elements} == {expected_last_modification_date}
@@ -696,14 +791,6 @@ def test_partition_pdf_warns_with_ocr_languages(caplog):
     assert "The ocr_languages kwarg will be deprecated" in caplog.text
 
 
-def test_partition_pdf_or_image_warns_with_ocr_languages(caplog):
-    filename = example_doc_path("DA-1p.pdf")
-    pdf.partition_pdf_or_image(
-        filename=filename, strategy=PartitionStrategy.HI_RES, ocr_languages="eng"
-    )
-    assert "The ocr_languages kwarg will be deprecated" in caplog.text
-
-
 def test_partition_categorization_backup():
     text = "This is Clearly a Title"
     with mock.patch.object(pdf, "_partition_pdf_or_image_local", return_value=[Text(text)]):
@@ -786,16 +873,40 @@ def test_partition_pdf_uses_model_name():
         assert mockpartition.call_args.kwargs["model_name"]
 
 
+def test_partition_pdf_uses_hi_res_model_name():
+    with mock.patch.object(
+        pdf,
+        "_partition_pdf_or_image_local",
+    ) as mockpartition:
+        pdf.partition_pdf(
+            example_doc_path("layout-parser-paper-fast.pdf"),
+            hi_res_model_name="test",
+            strategy=PartitionStrategy.HI_RES,
+        )
+
+        mockpartition.assert_called_once()
+        assert "hi_res_model_name" in mockpartition.call_args.kwargs
+        assert mockpartition.call_args.kwargs["hi_res_model_name"]
+
+
 def test_partition_pdf_word_bbox_not_char(
     filename=example_doc_path("interface-config-guide-p93.pdf"),
 ):
     try:
-        elements = pdf.partition_pdf(filename=filename)
+        elements = pdf.partition_pdf(filename=filename, strategy="fast")
     except Exception as e:
         raise ("Partitioning fail: %s" % e)
     assert len(elements) == 17
 
 
+def test_partition_pdf_fast_no_mapping_errors(
+    filename=example_doc_path("a1977-backus-p21.pdf"),
+):
+    """Verify there is no regression for https://github.com/Unstructured-IO/unstructured/pull/2940,
+    failing to map old parent_id's to new"""
+    pdf.partition_pdf(filename=filename, strategy="fast")
+
+
 def test_partition_pdf_raises_TypeError_for_invalid_languages():
     filename = example_doc_path("chevron-page.pdf")
     with pytest.raises(TypeError):
@@ -824,18 +935,16 @@ def test_check_annotations_within_element(threshold, expected):
 
 
 @pytest.mark.parametrize(
-    ("infer_table_structure", "env", "expected"),
+    ("env", "expected"),
     [
-        (False, None, "yolox_quantized"),
-        (True, None, "yolox"),
-        (False, "test", "test"),
-        (True, "test", "test"),
+        (None, "yolox"),
+        ("test", "test"),
     ],
 )
-def test_default_hi_res_model(infer_table_structure, env, expected, monkeypatch):
+def test_default_hi_res_model(env, expected, monkeypatch):
     if env is not None:
         monkeypatch.setenv("UNSTRUCTURED_HI_RES_MODEL_NAME", env)
-    assert pdf.default_hi_res_model(infer_table_structure) == expected
+    assert pdf.default_hi_res_model() == expected
 
 
 def test_partition_model_name_default_to_None():
@@ -851,6 +960,18 @@ def test_partition_model_name_default_to_None():
         pytest.fail("partition_pdf() raised AttributeError unexpectedly!")
 
 
+def test_partition_hi_res_model_name_default_to_None():
+    filename = example_doc_path("DA-1p.pdf")
+    try:
+        pdf.partition_pdf(
+            filename=filename,
+            strategy=PartitionStrategy.HI_RES,
+            hi_res_model_name=None,
+        )
+    except AttributeError:
+        pytest.fail("partition_pdf() raised AttributeError unexpectedly!")
+
+
 @pytest.mark.parametrize(
     ("strategy", "ocr_func"),
     [
@@ -1038,15 +1159,202 @@ def test_partition_pdf_with_bad_color_profile():
     assert pdf.partition_pdf(filename, strategy="fast")
 
 
+def test_partition_pdf_with_fast_finds_headers_footers(filename="example-docs/header-test-doc.pdf"):
+    elements = pdf.partition_pdf(filename, strategy="fast")
+    assert isinstance(elements[0], Header)
+    assert isinstance(elements[-1], Footer)
+    assert [element.text for element in elements] == [
+        "I Am A Header",
+        "Title",
+        "Here is a lovely sentences.",
+        "I Am A Footer",
+    ]
+
+
 @pytest.mark.parametrize(
     ("filename", "expected_log"),
     [
         ("invalid-pdf-structure-pdfminer-entire-doc.pdf", "Repairing the PDF document ..."),
         ("invalid-pdf-structure-pdfminer-one-page.pdf", "Repairing the PDF page 2 ..."),
-        ("failure-after-repair.pdf", "PDFMiner failed to process PDF page 26 after repairing it."),
     ],
 )
 def test_extractable_elements_repair_invalid_pdf_structure(filename, expected_log, caplog):
     caplog.set_level(logging.INFO)
     assert pdf.extractable_elements(filename=example_doc_path(filename))
     assert expected_log in caplog.text
+
+
+def assert_element_extraction(
+    elements: list[Element],
+    extract_image_block_types: list[str],
+    extract_image_block_to_payload: bool,
+    tmpdir: str,
+):
+    extracted_elements: list[list[Element]] = []
+    for el_type in extract_image_block_types:
+        extracted_elements_by_type: list[Element] = []
+        for el in elements:
+            if el.category == el_type:
+                extracted_elements_by_type.append(el)
+        extracted_elements.append(extracted_elements_by_type)
+
+    for extracted_elements_by_type in extracted_elements:
+        for i, el in enumerate(extracted_elements_by_type):
+            if extract_image_block_to_payload:
+                assert el.metadata.image_base64 is not None
+                assert el.metadata.image_mime_type == "image/jpeg"
+                image_data = base64.b64decode(el.metadata.image_base64)
+                assert isinstance(image_data, bytes)
+                assert el.metadata.image_path is None
+            else:
+                basename = "table" if el.category == ElementType.TABLE else "figure"
+                expected_image_path = os.path.join(
+                    str(tmpdir), f"{basename}-{el.metadata.page_number}-{i + 1}.jpg"
+                )
+                assert el.metadata.image_path == expected_image_path
+                assert os.path.isfile(expected_image_path)
+                assert el.metadata.image_base64 is None
+                assert el.metadata.image_mime_type is None
+
+
+@pytest.mark.parametrize("file_mode", ["filename", "rb"])
+@pytest.mark.parametrize("extract_image_block_to_payload", [False, True])
+def test_partition_pdf_element_extraction(
+    file_mode,
+    extract_image_block_to_payload,
+    filename=example_doc_path("embedded-images-tables.pdf"),
+):
+    extract_image_block_types = ["Image", "Table"]
+
+    with tempfile.TemporaryDirectory() as tmpdir:
+        if file_mode == "filename":
+            elements = pdf.partition_pdf(
+                filename=filename,
+                # Image extraction shouldn't break by setting this
+                starting_page_number=20,
+                extract_image_block_types=extract_image_block_types,
+                extract_image_block_to_payload=extract_image_block_to_payload,
+                extract_image_block_output_dir=tmpdir,
+            )
+        else:
+            with open(filename, "rb") as f:
+                elements = pdf.partition_pdf(
+                    file=f,
+                    # Image extraction shouldn't break by setting this
+                    starting_page_number=20,
+                    extract_image_block_types=extract_image_block_types,
+                    extract_image_block_to_payload=extract_image_block_to_payload,
+                    extract_image_block_output_dir=tmpdir,
+                )
+        assert elements[0].metadata.page_number == 20
+        assert_element_extraction(
+            elements, extract_image_block_types, extract_image_block_to_payload, tmpdir
+        )
+
+
+def test_partition_pdf_always_keep_all_image_elements(
+    filename=example_doc_path("embedded-images.pdf"),
+):
+    elements = pdf.partition_pdf(
+        filename=filename,
+        strategy="hi_res",
+    )
+    image_elements = [el for el in elements if el.category == ElementType.IMAGE]
+    assert len(image_elements) == 3
+
+
+@pytest.fixture()
+def expected_element_ids_for_fast_strategy():
+    return [
+        "27a6cb3e5a4ad399b2f865729bbd3840",
+        "a90a54baba0093296a013d26b7acbc17",
+        "9be424e2d151dac4b5f36a85e9bbfe65",
+        "4631da875fb4996c63b2d80cea6b588e",
+        "6264f4eda97a049f4710f9bea0c01cbd",
+        "abded7b2ff3a5542c88b4a831755ec24",
+        "b781ea5123cb31e0571391b7b42cac75",
+        "033f27d2618ba4cda9068b267b5a731e",
+        "8982a12fcced30dd12ccbf61d14f30bf",
+        "41af2fd5df0cf47aa7e8ecca200d3ac6",
+    ]
+
+
+@pytest.fixture()
+def expected_element_ids_for_hi_res_strategy():
+    return [
+        "27a6cb3e5a4ad399b2f865729bbd3840",
+        "a90a54baba0093296a013d26b7acbc17",
+        "9be424e2d151dac4b5f36a85e9bbfe65",
+        "4631da875fb4996c63b2d80cea6b588e",
+        "6264f4eda97a049f4710f9bea0c01cbd",
+        "abded7b2ff3a5542c88b4a831755ec24",
+        "b781ea5123cb31e0571391b7b42cac75",
+        "033f27d2618ba4cda9068b267b5a731e",
+        "8982a12fcced30dd12ccbf61d14f30bf",
+        "41af2fd5df0cf47aa7e8ecca200d3ac6",
+    ]
+
+
+@pytest.fixture()
+def expected_element_ids_for_ocr_strategy():
+    return [
+        "272ab65cbe81795161128aea59599d83",
+        "b38affd7bbbb3dddf5c85ba8b14d380d",
+        "65903214d456b8b3cba6faa6714bd9ba",
+        "5b41ceae05dcfaeeac32ff8e82dc2ff1",
+        "6582fc6c6c595225feeddcc3263f0ae3",
+        "64b610c8f4274f1ce2175bf30814409d",
+        "8edde8bf2d3a68370dc4bd142c408ca4",
+        "a052bc17696043efce2e4f4f28393a83",
+    ]
+
+
+@pytest.fixture()
+def expected_ids(request):
+    return request.getfixturevalue(request.param)
+
+
+@pytest.mark.parametrize(
+    ("strategy", "expected_ids"),
+    [
+        (PartitionStrategy.FAST, "expected_element_ids_for_fast_strategy"),
+        (PartitionStrategy.HI_RES, "expected_element_ids_for_hi_res_strategy"),
+        (PartitionStrategy.OCR_ONLY, "expected_element_ids_for_ocr_strategy"),
+    ],
+    indirect=["expected_ids"],
+)
+def test_unique_and_deterministic_element_ids(strategy, expected_ids):
+    elements = pdf.partition_pdf(
+        "example-docs/fake-memo-with-duplicate-page.pdf", strategy=strategy, starting_page_number=2
+    )
+    ids = [element.id for element in elements]
+    assert ids == expected_ids, "Element IDs do not match expected IDs"
+
+
+def test_analysis_artifacts_saved():
+    with tempfile.TemporaryDirectory() as temp_dir:
+        filename = example_doc_path("layout-parser-paper-fast.pdf")
+        pdf.partition_pdf(
+            filename=filename,
+            strategy=PartitionStrategy.HI_RES,
+            analysis=True,
+            analyzed_image_output_dir_path=temp_dir,
+        )
+
+        analysis_dir = Path(temp_dir)
+        layout_dump_dir = analysis_dir / "analysis" / "layout-parser-paper-fast" / "layout_dump"
+        assert layout_dump_dir.exists()
+        layout_dump_files = list(layout_dump_dir.iterdir())
+        assert len(layout_dump_files) == 1
+        assert (layout_dump_dir / "object_detection.json").exists()
+
+        bboxes_dir = analysis_dir / "analysis" / "layout-parser-paper-fast" / "bboxes"
+        assert bboxes_dir.exists()
+        bboxes_files = list(bboxes_dir.iterdir())
+        assert len(bboxes_files) == 2 * 4  # 2 pages * 4 different layouts per page
+
+        expected_layouts = ["od_model", "ocr", "pdfminer", "final"]
+        expected_pages = [1, 2]
+        for el in expected_layouts:
+            for page in expected_pages:
+                assert bboxes_dir / f"page{page}_layout_{el}.png" in bboxes_files
