@@ -1,14 +1,23 @@
+import datetime as dt
+import io
+import os
+import pathlib
 from dataclasses import dataclass
+from multiprocessing import Pool
 from unittest import mock
 
+import numpy as np
 import pytest
 from PIL import Image
 from unstructured_inference.inference import layout
 from unstructured_inference.inference.elements import TextRegion
-from unstructured_inference.inference.layout import DocumentLayout, LayoutElement, PageLayout
+from unstructured_inference.inference.layout import DocumentLayout, PageLayout
+from unstructured_inference.inference.layoutelement import LayoutElement
 
+from test_unstructured.unit_utils import example_doc_path
 from unstructured.documents.coordinates import PixelSpace
 from unstructured.documents.elements import (
+    TYPE_TO_TEXT_ELEMENT_MAP,
     CheckBox,
     CoordinatesMetadata,
     ElementMetadata,
@@ -28,7 +37,7 @@ from unstructured.partition.utils.constants import SORT_MODE_BASIC, SORT_MODE_DO
 
 
 class MockPageLayout(layout.PageLayout):
-    def __init__(self, number: int, image: Image):
+    def __init__(self, number: int, image: Image.Image):
         self.number = number
         self.image = image
 
@@ -202,30 +211,48 @@ def test_normalize_layout_element_layout_element_narrative_text():
     )
 
 
-def test_normalize_layout_element_checked_box():
+@pytest.mark.parametrize(
+    ("element_type", "expected_element_class"),
+    TYPE_TO_TEXT_ELEMENT_MAP.items(),
+)
+def test_normalize_layout_element_layout_element_maps_to_appropriate_text_element(
+    element_type: str,
+    expected_element_class: type[Text],
+):
     layout_element = LayoutElement.from_coords(
-        type="Checked",
+        type=element_type,
         x1=1,
         y1=2,
         x2=3,
         y2=4,
-        text="",
+        text="Some lovely text",
     )
     coordinate_system = PixelSpace(width=10, height=20)
     element = common.normalize_layout_element(
         layout_element,
         coordinate_system=coordinate_system,
     )
-    assert element == CheckBox(
-        checked=True,
+    assert element == expected_element_class(
+        text="Some lovely text",
         coordinates=((1, 2), (1, 4), (3, 4), (3, 2)),
         coordinate_system=coordinate_system,
     )
 
 
-def test_normalize_layout_element_unchecked_box():
+@pytest.mark.parametrize(
+    ("element_type", "expected_checked"),
+    [
+        (ElementType.CHECK_BOX_UNCHECKED, False),
+        (ElementType.CHECK_BOX_CHECKED, True),
+        (ElementType.RADIO_BUTTON_UNCHECKED, False),
+        (ElementType.RADIO_BUTTON_CHECKED, True),
+        (ElementType.CHECKED, True),
+        (ElementType.UNCHECKED, False),
+    ],
+)
+def test_normalize_layout_element_checkable(element_type: str, expected_checked: bool):
     layout_element = LayoutElement.from_coords(
-        type="Unchecked",
+        type=element_type,
         x1=1,
         y1=2,
         x2=3,
@@ -237,8 +264,9 @@ def test_normalize_layout_element_unchecked_box():
         layout_element,
         coordinate_system=coordinate_system,
     )
+    assert isinstance(element, CheckBox)
     assert element == CheckBox(
-        checked=False,
+        checked=expected_checked,
         coordinates=((1, 2), (1, 4), (3, 4), (3, 2)),
         coordinate_system=coordinate_system,
     )
@@ -310,20 +338,58 @@ def test_normalize_layout_element_bulleted_list():
     ]
 
 
-class MockPopenWithError:
-    def __init__(self, *args, **kwargs):
-        pass
+class MockRunOutput:
 
-    def communicate(self):
-        return b"", b"an error occurred"
+    def __init__(self, returncode, stdout, stderr):
+        self.returncode = returncode
+        self.stdout = stdout
+        self.stderr = stderr
 
 
 def test_convert_office_doc_captures_errors(monkeypatch, caplog):
-    import subprocess
+    from unstructured.partition.common import subprocess
 
-    monkeypatch.setattr(subprocess, "Popen", MockPopenWithError)
+    def mock_run(*args, **kwargs):
+        return MockRunOutput(1, "an error occurred".encode(), "error details".encode())
+
+    monkeypatch.setattr(subprocess, "run", mock_run)
     common.convert_office_doc("no-real.docx", "fake-directory", target_format="docx")
-    assert "an error occurred" in caplog.text
+    assert "soffice failed to convert to format docx with code 1" in caplog.text
+
+
+def test_convert_office_docs_avoids_concurrent_call_to_soffice():
+    paths_to_save = [pathlib.Path(path) for path in ("/tmp/proc1", "/tmp/proc2", "/tmp/proc3")]
+    for path in paths_to_save:
+        path.mkdir(exist_ok=True)
+        (path / "simple.docx").unlink(missing_ok=True)
+    file_to_convert = example_doc_path("simple.doc")
+
+    with Pool(3) as pool:
+        pool.starmap(common.convert_office_doc, [(file_to_convert, path) for path in paths_to_save])
+
+    assert np.sum([(path / "simple.docx").is_file() for path in paths_to_save]) == 3
+
+
+def test_convert_office_docs_respects_wait_timeout():
+    paths_to_save = [
+        pathlib.Path(path) for path in ("/tmp/wait/proc1", "/tmp/wait/proc2", "/tmp/wait/proc3")
+    ]
+    for path in paths_to_save:
+        path.mkdir(parents=True, exist_ok=True)
+        (path / "simple.docx").unlink(missing_ok=True)
+    file_to_convert = example_doc_path("simple.doc")
+
+    with Pool(3) as pool:
+        pool.starmap(
+            common.convert_office_doc,
+            # set timeout to wait for soffice to be available to 0 so only one process can convert
+            # the doc file on the first try; then the catch all
+            [(file_to_convert, path, "docx", None, 0) for path in paths_to_save],
+        )
+
+    # because this test file is very small we could have occasions where two files are converted
+    # when one of the processes spawned just a little
+    assert np.sum([(path / "simple.docx").is_file() for path in paths_to_save]) < 3
 
 
 class MockDocxEmptyTable:
@@ -390,9 +456,11 @@ def test_set_element_hierarchy():
     assert elements[4].metadata.parent_id == elements[3].id, "ListItem should be child of Title"
     assert elements[5].metadata.parent_id == elements[3].id, "ListItem should be child of Title"
     assert elements[6].metadata.parent_id == elements[0].id, "ListItem should be child of Title"
-    assert (
-        elements[7].metadata.parent_id is None
-    ), "CheckBox should be None, as it's not a Text based element"
+    # NOTE(Hubert): moving the category field to Element, caused this to fail.
+    # Checkboxes will soon be deprecated, then we can remove the test.
+    # assert (
+    #         elements[7].metadata.parent_id is None
+    # ), "CheckBox should be None, as it's not a Text based element"
     assert elements[8].metadata.parent_id is None, "Title 2 should be child of None"
     assert elements[9].metadata.parent_id == elements[8].id, "ListItem should be child of Title 2"
     assert elements[10].metadata.parent_id == elements[8].id, "ListItem should be child of Title 2"
@@ -451,7 +519,6 @@ def test_document_to_element_list_handles_parent():
     page = PageLayout(
         number=1,
         image=MockImage(),
-        layout=None,
     )
     page.elements = [block1, block2]
     doc = DocumentLayout.from_pages([page])
@@ -477,7 +544,6 @@ def test_document_to_element_list_doesnt_sort_on_sort_method(sort_mode, call_cou
     page = PageLayout(
         number=1,
         image=MockImage(),
-        layout=None,
     )
     page.elements = [block1, block2]
     doc = DocumentLayout.from_pages([page])
@@ -541,3 +607,99 @@ def test_ocr_data_to_elements(
             points=layout_el.bbox.coordinates,
             system=coordinate_system,
         )
+
+
+class Describe_get_last_modified:
+    """Isolated unit-tests for `unstructured.partition.common.get_last_modified()."""
+
+    def it_pulls_last_modified_from_the_filesystem_when_a_path_is_provided(
+        self, file_and_last_modified: tuple[str, str]
+    ):
+        file_path, last_modified = file_and_last_modified
+        last_modified_date = common.get_last_modified(str(file_path), None, False)
+        assert last_modified_date == last_modified
+
+    def and_it_pulls_last_modified_from_the_file_like_object_when_one_is_provided(
+        self, file_and_last_modified: tuple[str, str]
+    ):
+        file_path, last_modified = file_and_last_modified
+        with open(file_path, "rb") as f:
+            last_modified_date = common.get_last_modified(None, f, True)
+        assert last_modified_date == last_modified
+
+    def but_not_when_date_from_file_object_is_False(self, file_and_last_modified: tuple[str, str]):
+        file_path, _ = file_and_last_modified
+        with open(file_path, "rb") as f:
+            last_modified_date = common.get_last_modified(None, f, False)
+        assert last_modified_date is None
+
+    # -- fixtures --------------------------------------------------------------------------------
+
+    @pytest.fixture()
+    def file_and_last_modified(self, tmp_path: pathlib.Path) -> tuple[str, str]:
+        modified_timestamp = dt.datetime(
+            year=2024, month=6, day=14, hour=15, minute=39, second=25
+        ).timestamp()
+        file_path = tmp_path / "some_file.txt"
+        file_path.write_text("abcdefg")
+        os.utime(file_path, (modified_timestamp, modified_timestamp))
+        return str(file_path), "2024-06-14T15:39:25"
+
+
+class Describe_get_last_modified_date:
+    def it_gets_the_modified_time_of_a_file_identified_by_a_path(self, tmp_path: pathlib.Path):
+        modified_timestamp = dt.datetime(
+            year=2024, month=3, day=5, hour=17, minute=43, second=40
+        ).timestamp()
+        file_path = tmp_path / "some_file.txt"
+        file_path.write_text("abcdefg")
+        os.utime(file_path, (modified_timestamp, modified_timestamp))
+
+        last_modified_date = common.get_last_modified_date(str(file_path))
+
+        assert last_modified_date == "2024-03-05T17:43:40"
+
+    def but_it_returns_None_when_there_is_no_file_at_that_path(self, tmp_path: pathlib.Path):
+        file_path = tmp_path / "some_file_that_does_not_exist.txt"
+
+        last_modified_date = common.get_last_modified_date(str(file_path))
+
+        assert last_modified_date is None
+
+
+class Describe_get_last_modified_date_from_file:
+    def it_gets_the_modified_time_of_a_file_like_object_corresponding_to_a_filesystem_file(
+        self, tmp_path: pathlib.Path
+    ):
+        modified_timestamp = dt.datetime(
+            year=2024, month=3, day=5, hour=20, minute=48, second=26
+        ).timestamp()
+        file_path = tmp_path / "some_file_2.txt"
+        file_path.write_text("abcdefg")
+        os.utime(file_path, (modified_timestamp, modified_timestamp))
+
+        with open(file_path, "rb") as f:
+            last_modified_date = common.get_last_modified_date_from_file(f)
+
+        assert last_modified_date == "2024-03-05T20:48:26"
+
+    def but_it_returns_None_when_the_argument_is_a_bytes_object(self):
+        assert common.get_last_modified_date_from_file(b"abcdefg") is None
+
+    def and_it_returns_None_when_the_file_like_object_has_no_name_attribute(self):
+        file = io.BytesIO(b"abcdefg")
+        assert hasattr(file, "name") is False
+
+        last_modified_date = common.get_last_modified_date_from_file(file)
+
+        assert last_modified_date is None
+
+    def and_it_returns_None_when_the_file_like_object_name_is_not_a_path_to_a_file(
+        self, tmp_path: pathlib.Path
+    ):
+        file = io.BytesIO(b"abcdefg")
+        file.name = str(tmp_path / "a_file_that_isn't_here.txt")
+
+        last_modified_date = common.get_last_modified_date_from_file(file)
+
+        assert last_modified_date is None
