@@ -5,7 +5,9 @@ from __future__ import annotations
 import json
 import os
 import pathlib
-from typing import Optional, Sequence, Type, cast
+import uuid
+from tempfile import SpooledTemporaryFile
+from typing import Optional, Type
 
 import pytest
 from pytest_mock import MockerFixture
@@ -13,7 +15,7 @@ from pytest_mock import MockerFixture
 from test_unstructured.unit_utils import assert_round_trips_through_JSON, example_doc_path
 from unstructured.chunking.title import chunk_by_title
 from unstructured.cleaners.core import group_broken_paragraphs
-from unstructured.documents.elements import Address, ListItem, NarrativeText, Text, Title
+from unstructured.documents.elements import Address, ListItem, NarrativeText, Title
 from unstructured.partition.text import (
     _combine_paragraphs_less_than_min,
     _split_content_to_fit_max,
@@ -256,7 +258,7 @@ def test_partition_text_extract_regex_metadata():
 
 def test_partition_text_splits_long_text():
     filename = os.path.join(EXAMPLE_DOCS_DIRECTORY, "norwich-city.txt")
-    elements = cast(Sequence[Text], partition_text(filename=filename))
+    elements = partition_text(filename=filename)
     assert len(elements) > 0
     assert elements[0].text.startswith("Iwan Roberts")
     assert elements[-1].text.endswith("External links")
@@ -264,8 +266,8 @@ def test_partition_text_splits_long_text():
 
 def test_partition_text_splits_long_text_max_partition():
     filename = os.path.join(EXAMPLE_DOCS_DIRECTORY, "norwich-city.txt")
-    elements = cast(Sequence[Text], partition_text(filename=filename))
-    elements_max_part = cast(Sequence[Text], partition_text(filename=filename, max_partition=500))
+    elements = partition_text(filename=filename)
+    elements_max_part = partition_text(filename=filename, max_partition=500)
     # NOTE(klaijan) - I edited the operation here from < to <=
     # Please revert back if this does not make sense
     assert len(elements) <= len(elements_max_part)
@@ -278,11 +280,8 @@ def test_partition_text_splits_long_text_max_partition():
 
 def test_partition_text_splits_max_min_partition():
     filename = os.path.join(EXAMPLE_DOCS_DIRECTORY, "norwich-city.txt")
-    elements = cast(Sequence[Text], partition_text(filename=filename))
-    elements_max_part = cast(
-        Sequence[Text],
-        partition_text(filename=filename, min_partition=1000, max_partition=1500),
-    )
+    elements = partition_text(filename=filename)
+    elements_max_part = partition_text(filename=filename, min_partition=1000, max_partition=1500)
     for i, element in enumerate(elements_max_part):
         # NOTE(robinson) - the last element does not have a next element to merge with,
         # so it can be short
@@ -314,27 +313,14 @@ def test_partition_text_splits_max_min_partition():
 
 
 def test_partition_text_min_max():
-    segments = cast(
-        Sequence[Text],
-        partition_text(
-            text=SHORT_PARAGRAPHS,
-            min_partition=6,
-        ),
-    )
+    segments = partition_text(text=SHORT_PARAGRAPHS, min_partition=6)
     for i, segment in enumerate(segments):
         # NOTE(robinson) - the last element does not have a next element to merge with,
         # so it can be short
         if i < len(segments) - 1:
             assert len(segment.text) >= 6
 
-    segments = cast(
-        Sequence[Text],
-        partition_text(
-            text=SHORT_PARAGRAPHS,
-            max_partition=20,
-            min_partition=7,
-        ),
-    )
+    segments = partition_text(text=SHORT_PARAGRAPHS, max_partition=20, min_partition=7)
     for i, segment in enumerate(segments):
         # NOTE(robinson) - the last element does not have a next element to merge with,
         # so it can be short
@@ -368,7 +354,7 @@ def test_combine_paragraphs_less_than_min():
 
 def test_partition_text_doesnt_get_page_breaks():
     text = "--------------------"
-    elements = cast(Sequence[Text], partition_text(text=text))
+    elements = partition_text(text=text)
     assert len(elements) == 1
     assert elements[0].text == text
     assert not isinstance(elements[0], ListItem)
@@ -449,6 +435,24 @@ def test_partition_text_from_file_metadata_date(mocker: MockerFixture):
             file=f,
         )
 
+    assert elements[0].metadata.last_modified is None
+
+
+def test_partition_text_from_file_explicit_get_metadata_date(mocker: MockerFixture):
+    filename = os.path.join(EXAMPLE_DOCS_DIRECTORY, "fake-text.txt")
+    mocked_last_modification_date = "2029-07-05T09:24:28"
+
+    mocker.patch(
+        "unstructured.partition.text.get_last_modified_date_from_file",
+        return_value=mocked_last_modification_date,
+    )
+
+    with open(filename, "rb") as f:
+        elements = partition_text(
+            file=f,
+            date_from_file_object=True,
+        )
+
     assert elements[0].metadata.last_modified == mocked_last_modification_date
 
 
@@ -491,19 +495,35 @@ def test_partition_text_from_text_with_custom_metadata_date():
     assert elements[0].metadata.last_modified == expected_last_modification_date
 
 
-def test_partition_text_with_unique_ids():
-    elements = partition_text(text="hello there!")
-    assert elements[0].id == "c69509590d81db2f37f9d75480c8efed"
-    # Test that the element is JSON serializable. This should run without an error
-    json.dumps(elements[0].to_dict())
+def test_partition_text_from_file_without_metadata_date():
+    """Test partition_text() with file that are not possible to get last modified date"""
+    filename = os.path.join(EXAMPLE_DOCS_DIRECTORY, "fake-text.txt")
+    with open(filename, "rb") as f:
+        sf = SpooledTemporaryFile()
+        sf.write(f.read())
+        sf.seek(0)
+        elements = partition_text(file=sf, date_from_file_object=True)
 
-    elements = partition_text(text="hello there!", unique_element_ids=True)
-    id = elements[0].id
-    assert isinstance(id, str)  # included for type-narrowing
-    assert len(id) == 36
-    assert id.count("-") == 4
-    # Test that the element is JSON serializable. This should run without an error
-    json.dumps(elements[0].to_dict())
+    assert elements[0].metadata.last_modified is None
+
+
+def test_Text_element_assigns_id_hashes_that_are_unique_and_deterministic():
+    ids = [element.id for element in partition_text(text="hello\nhello\nhello")]
+    assert ids == [
+        "8657c0ec31a4cfc822f6cd4a5684cafd",
+        "72aefb4a12be063ad160931fdb380163",
+        "ba8c1a216ca585aecdd365a72e6124f1",
+    ]
+
+
+def test_Text_element_assings_UUID_when_unique_element_ids_is_True():
+    elements = partition_text(text="hello\nhello\nhello", unique_element_ids=True)
+
+    for element in elements:
+        assert uuid.UUID(element.id, version=4)
+
+        # Test that the element is JSON serializable. This should run without an error
+        json.dumps(element.to_dict())
 
 
 @pytest.mark.parametrize(
