@@ -1,7 +1,12 @@
+from __future__ import annotations
+
 import quopri
 import re
 import sys
 import unicodedata
+from typing import Optional, Tuple
+
+import numpy as np
 
 from unstructured.file_utils.encoding import (
     format_encoding_str,
@@ -29,7 +34,7 @@ def clean_non_ascii_chars(text) -> str:
     return en.decode()
 
 
-def clean_bullets(text) -> str:
+def clean_bullets(text: str) -> str:
     """Cleans unicode bullets from a section of text.
 
     Example
@@ -68,6 +73,37 @@ def clean_ordered_bullets(text) -> str:
     return text_cl
 
 
+def clean_ligatures(text) -> str:
+    """Replaces ligatures with their most likely equivalent characters.
+
+    Example
+    -------
+    The beneﬁts -> The benefits
+    High quality ﬁnancial -> High quality financial
+    """
+    ligatures_map = {
+        "æ": "ae",
+        "Æ": "AE",
+        "ﬀ": "ff",
+        "ﬁ": "fi",
+        "ﬂ": "fl",
+        "ﬃ": "ffi",
+        "ﬄ": "ffl",
+        "ﬅ": "ft",
+        "ʪ": "ls",
+        "œ": "oe",
+        "Œ": "OE",
+        "ȹ": "qp",
+        "ﬆ": "st",
+        "ʦ": "ts",
+    }
+    cleaned_text: str = text
+    for k, v in ligatures_map.items():
+        cleaned_text = cleaned_text.replace(k, v)
+
+    return cleaned_text
+
+
 def group_bullet_paragraph(paragraph: str) -> list:
     """Groups paragraphs with bullets that have line breaks for visual/formatting purposes.
     For example:
@@ -98,8 +134,8 @@ def group_bullet_paragraph(paragraph: str) -> list:
 
 def group_broken_paragraphs(
     text: str,
-    line_split: re.Pattern = PARAGRAPH_PATTERN_RE,
-    paragraph_split: re.Pattern = DOUBLE_PARAGRAPH_PATTERN_RE,
+    line_split: re.Pattern[str] = PARAGRAPH_PATTERN_RE,
+    paragraph_split: re.Pattern[str] = DOUBLE_PARAGRAPH_PATTERN_RE,
 ) -> str:
     """Groups paragraphs that have line breaks for visual/formatting purposes.
     For example:
@@ -140,7 +176,7 @@ def group_broken_paragraphs(
 
 def new_line_grouper(
     text: str,
-    paragraph_split: re.Pattern = LINE_BREAK_RE,
+    paragraph_split: re.Pattern[str] = LINE_BREAK_RE,
 ) -> str:
     """
     Concatenates text document that has one-line paragraph break pattern
@@ -187,7 +223,7 @@ def blank_line_grouper(
 
 def auto_paragraph_grouper(
     text: str,
-    line_split: re.Pattern = LINE_BREAK_RE,
+    line_split: re.Pattern[str] = LINE_BREAK_RE,
     max_line_count: int = 2000,
     threshold: float = 0.1,
 ) -> str:
@@ -222,7 +258,7 @@ def auto_paragraph_grouper(
 # TODO(robinson) - There's likely a cleaner was to accomplish this and get all of the
 # unicode characters instead of just the quotes. Doing this for now since quotes are
 # an issue that are popping up in the SEC filings tests
-def replace_unicode_quotes(text) -> str:
+def replace_unicode_quotes(text: str) -> str:
     """Replaces unicode bullets in text with the expected character
 
     Example
@@ -268,6 +304,15 @@ def remove_punctuation(s: str) -> str:
     return s
 
 
+def remove_sentence_punctuation(s: str, exclude_punctuation: Optional[list]) -> str:
+    tbl_new = tbl.copy()
+    if exclude_punctuation:
+        for punct in exclude_punctuation:
+            del tbl_new[ord(punct)]
+    s = s.translate(tbl_new)
+    return s
+
+
 def clean_extra_whitespace(text: str) -> str:
     """Cleans extra whitespace characters that appear between words.
 
@@ -381,3 +426,46 @@ def bytes_string_to_string(text: str, encoding: str = "utf-8"):
     text_bytes = bytes([ord(char) for char in text])
     formatted_encoding = format_encoding_str(encoding)
     return text_bytes.decode(formatted_encoding)
+
+
+def clean_extra_whitespace_with_index_run(text: str) -> Tuple[str, np.ndarray]:
+    """Cleans extra whitespace characters that appear between words.
+    Calculate distance between characters of original text and cleaned text.
+
+    Returns cleaned text along with array of indices it has moved from original.
+
+    Example
+    -------
+    ITEM 1.     BUSINESS -> ITEM 1. BUSINESS
+    array([0., 0., 0., 0., 0., 0., 0., 0., 4., 4., 4., 4., 4., 4., 4., 4., 4., 4., 4., 4.]))
+    """
+
+    cleaned_text = re.sub(r"[\xa0\n]", " ", text)
+    cleaned_text = re.sub(r"([ ]{2,})", " ", cleaned_text)
+
+    cleaned_text = cleaned_text.strip()
+
+    moved_indices = np.zeros(len(text))
+
+    distance, original_index, cleaned_index = 0, 0, 0
+    while cleaned_index < len(cleaned_text):
+        if text[original_index] == cleaned_text[cleaned_index] or (
+            bool(re.match("[\xa0\n]", text[original_index]))
+            and bool(re.match(" ", cleaned_text[cleaned_index]))
+        ):
+            moved_indices[cleaned_index] = distance
+            original_index += 1
+            cleaned_index += 1
+            continue
+
+        distance += 1
+        moved_indices[cleaned_index] = distance
+        original_index += 1
+
+    moved_indices[cleaned_index:] = distance
+
+    return cleaned_text, moved_indices
+
+
+def index_adjustment_after_clean_extra_whitespace(index, moved_indices) -> int:
+    return int(index - moved_indices[index])
