@@ -6,15 +6,15 @@ import dataclasses as dc
 import enum
 import functools
 import hashlib
-import inspect
 import os
 import pathlib
 import re
 import uuid
+from itertools import groupby
 from types import MappingProxyType
-from typing import Any, Callable, Dict, FrozenSet, List, Optional, Tuple, Union
+from typing import Any, Callable, FrozenSet, Optional, Sequence, cast
 
-from typing_extensions import ParamSpec, TypedDict
+from typing_extensions import ParamSpec, TypeAlias, TypedDict
 
 from unstructured.documents.coordinates import (
     TYPE_TO_COORDINATE_SYSTEM_MAP,
@@ -22,15 +22,10 @@ from unstructured.documents.coordinates import (
     RelativeCoordinateSystem,
 )
 from unstructured.partition.utils.constants import UNSTRUCTURED_INCLUDE_DEBUG_METADATA
-from unstructured.utils import lazyproperty
+from unstructured.utils import get_call_args_applying_defaults, lazyproperty
 
-
-class NoID(abc.ABC):
-    """Class to indicate that an element do not have an ID."""
-
-
-class UUID(abc.ABC):
-    """Class to indicate that an element should have a UUID."""
+Point: TypeAlias = "tuple[float, float]"
+Points: TypeAlias = "tuple[Point, ...]"
 
 
 @dc.dataclass
@@ -39,17 +34,17 @@ class DataSourceMetadata:
 
     url: Optional[str] = None
     version: Optional[str] = None
-    record_locator: Optional[Dict[str, Any]] = None  # Values must be JSON-serializable
+    record_locator: Optional[dict[str, Any]] = None  # Values must be JSON-serializable
     date_created: Optional[str] = None
     date_modified: Optional[str] = None
     date_processed: Optional[str] = None
-    permissions_data: Optional[List[Dict[str, Any]]] = None
+    permissions_data: Optional[list[dict[str, Any]]] = None
 
     def to_dict(self):
         return {key: value for key, value in self.__dict__.items() if value is not None}
 
     @classmethod
-    def from_dict(cls, input_dict: Dict[str, Any]):
+    def from_dict(cls, input_dict: dict[str, Any]):
         # Only use existing fields when constructing
         supported_fields = [f.name for f in dc.fields(cls)]
         args = {k: v for k, v in input_dict.items() if k in supported_fields}
@@ -61,10 +56,10 @@ class DataSourceMetadata:
 class CoordinatesMetadata:
     """Metadata fields that pertain to the coordinates of the element."""
 
-    points: Tuple[Tuple[float, float], ...]
-    system: CoordinateSystem
+    points: Optional[Points]
+    system: Optional[CoordinateSystem]
 
-    def __init__(self, points, system):
+    def __init__(self, points: Optional[Points], system: Optional[CoordinateSystem]):
         # Both `points` and `system` must be present; one is not meaningful without the other.
         if (points is None and system is not None) or (points is not None and system is None):
             raise ValueError(
@@ -92,32 +87,44 @@ class CoordinatesMetadata:
         }
 
     @classmethod
-    def from_dict(cls, input_dict: Dict[str, Any]):
+    def from_dict(cls, input_dict: dict[str, Any]):
         # `input_dict` may contain a tuple of tuples or a list of lists
-        def convert_to_tuple_of_tuples(sequence_of_sequences):
-            subsequences = []
+        def convert_to_points(sequence_of_sequences: Sequence[Sequence[float]]) -> Points:
+            points: list[Point] = []
             for seq in sequence_of_sequences:
                 if isinstance(seq, list):
-                    subsequences.append(tuple(seq))
+                    points.append(cast(Point, tuple(seq)))
                 elif isinstance(seq, tuple):
-                    subsequences.append(seq)
-            return tuple(subsequences)
+                    points.append(cast(Point, seq))
+            return tuple(points)
 
-        input_points = input_dict.get("points", None)
-        points = convert_to_tuple_of_tuples(input_points) if input_points is not None else None
-        width = input_dict.get("layout_width", None)
-        height = input_dict.get("layout_height", None)
-        system = None
-        if input_dict.get("system", None) == "RelativeCoordinateSystem":
-            system = RelativeCoordinateSystem()
-        elif (
-            width is not None
-            and height is not None
-            and input_dict.get("system", None) in TYPE_TO_COORDINATE_SYSTEM_MAP
-        ):
-            system = TYPE_TO_COORDINATE_SYSTEM_MAP[input_dict["system"]](width, height)
-        constructor_args = {"points": points, "system": system}
-        return cls(**constructor_args)
+        # -- parse points --
+        input_points = input_dict.get("points")
+        points = convert_to_points(input_points) if input_points is not None else None
+
+        # -- parse system --
+        system_name = input_dict.get("system")
+        width = input_dict.get("layout_width")
+        height = input_dict.get("layout_height")
+        system = (
+            None
+            if system_name is None
+            else (
+                RelativeCoordinateSystem()
+                if system_name == "RelativeCoordinateSystem"
+                else (
+                    TYPE_TO_COORDINATE_SYSTEM_MAP[system_name](width, height)
+                    if (
+                        width is not None
+                        and height is not None
+                        and system_name in TYPE_TO_COORDINATE_SYSTEM_MAP
+                    )
+                    else None
+                )
+            )
+        )
+
+        return cls(points=points, system=system)
 
 
 class RegexMetadata(TypedDict):
@@ -136,6 +143,18 @@ class Link(TypedDict):
     start_index: int
 
 
+class FormKeyOrValue(TypedDict):
+    text: str
+    layout_element_id: Optional[str]
+    custom_element: Optional[Text]
+
+
+class FormKeyValuePair(TypedDict):
+    key: FormKeyOrValue
+    value: Optional[FormKeyOrValue]
+    confidence: float
+
+
 class ElementMetadata:
     """Fully-dynamic replacement for dataclass-based ElementMetadata."""
 
@@ -143,9 +162,9 @@ class ElementMetadata:
     # - Add the field declaration with type here at the top. This makes it a "known" field and
     #   enables type-checking and completion.
     # - Add a parameter with default for field in __init__() and assign it in __init__() body.
-    # - Add a consolidation strategy for the field below in `ConsolidationStrategy`
-    #   `.field_consolidation_strategies()` to be used when consolidating metadata fields of a
-    #   section's elements during chunking.
+    # - Add a consolidation strategy for the new field in
+    #   `ConsolidationStrategy.field_consolidation_strategies()` below. This strategy will be used
+    #   to consolidate this new metadata field from each pre-chunk element during chunking.
     # - Add field-name to DEBUG_FIELD_NAMES if it shouldn't appear in dict/JSON or participate in
     #   equality comparison.
 
@@ -157,38 +176,44 @@ class ElementMetadata:
     detection_class_prob: Optional[float]
     # -- DEBUG field, the detection mechanism that emitted this element --
     detection_origin: Optional[str]
-    emphasized_text_contents: Optional[List[str]]
-    emphasized_text_tags: Optional[List[str]]
+    emphasized_text_contents: Optional[list[str]]
+    emphasized_text_tags: Optional[list[str]]
     file_directory: Optional[str]
     filename: Optional[str]
     filetype: Optional[str]
+    image_path: Optional[str]
+    image_base64: Optional[str]
+    image_mime_type: Optional[str]
     # -- specific to DOCX which has distinct primary, first-page, and even-page header/footers --
     header_footer_type: Optional[str]
-    image_path: Optional[str]
     # -- used in chunks only, when chunk must be split mid-text to fit window --
     is_continuation: Optional[bool]
-    languages: Optional[List[str]]
+    key_value_pairs: Optional[list[FormKeyValuePair]]
+    languages: Optional[list[str]]
     last_modified: Optional[str]
-    link_texts: Optional[List[str]]
-    link_urls: Optional[List[str]]
-    links: Optional[List[Link]]
+    link_texts: Optional[list[str]]
+    link_urls: Optional[list[str]]
+    link_start_indexes: Optional[list[int]]
+    links: Optional[list[Link]]
+    # -- used in chunks only, allowing access to element(s) chunk was formed from when enabled --
+    orig_elements: Optional[list[Element]]
     # -- the worksheet name in XLXS documents --
     page_name: Optional[str]
     # -- page numbers currently supported for DOCX, HTML, PDF, and PPTX documents --
     page_number: Optional[int]
-    parent_id: Optional[str | uuid.UUID | NoID | UUID]
+    parent_id: Optional[str]
     # -- "fields" e.g. status, dept.no, etc. extracted from text via regex --
-    regex_metadata: Optional[Dict[str, List[RegexMetadata]]]
-    # -- EPUB document section --
-    section: Optional[str]
+    regex_metadata: Optional[dict[str, list[RegexMetadata]]]
 
     # -- e-mail specific metadata fields --
-    sent_from: Optional[List[str]]
-    sent_to: Optional[List[str]]
+    sent_from: Optional[list[str]]
+    sent_to: Optional[list[str]]
     subject: Optional[str]
+    signature: Optional[str]
 
     # -- used for Table elements to capture rows/col structure --
     text_as_html: Optional[str]
+    table_as_cells: Optional[dict[str, str | int]]
     url: Optional[str]
 
     # -- debug fields can be assigned and referenced using dotted-notation but are not serialized
@@ -203,27 +228,32 @@ class ElementMetadata:
         coordinates: Optional[CoordinatesMetadata] = None,
         data_source: Optional[DataSourceMetadata] = None,
         detection_class_prob: Optional[float] = None,
-        emphasized_text_contents: Optional[List[str]] = None,
-        emphasized_text_tags: Optional[List[str]] = None,
+        emphasized_text_contents: Optional[list[str]] = None,
+        emphasized_text_tags: Optional[list[str]] = None,
         file_directory: Optional[str] = None,
         filename: Optional[str | pathlib.Path] = None,
         filetype: Optional[str] = None,
         header_footer_type: Optional[str] = None,
+        image_base64: Optional[str] = None,
+        image_mime_type: Optional[str] = None,
         image_path: Optional[str] = None,
         is_continuation: Optional[bool] = None,
-        languages: Optional[List[str]] = None,
+        languages: Optional[list[str]] = None,
         last_modified: Optional[str] = None,
-        link_texts: Optional[List[str]] = None,
-        link_urls: Optional[List[str]] = None,
-        links: Optional[List[Link]] = None,
+        link_start_indexes: Optional[list[int]] = None,
+        link_texts: Optional[list[str]] = None,
+        link_urls: Optional[list[str]] = None,
+        links: Optional[list[Link]] = None,
+        orig_elements: Optional[list[Element]] = None,
         page_name: Optional[str] = None,
         page_number: Optional[int] = None,
-        parent_id: Optional[str | uuid.UUID | NoID | UUID] = None,
-        regex_metadata: Optional[Dict[str, List[RegexMetadata]]] = None,
-        section: Optional[str] = None,
-        sent_from: Optional[List[str]] = None,
-        sent_to: Optional[List[str]] = None,
+        parent_id: Optional[str] = None,
+        regex_metadata: Optional[dict[str, list[RegexMetadata]]] = None,
+        sent_from: Optional[list[str]] = None,
+        sent_to: Optional[list[str]] = None,
+        signature: Optional[str] = None,
         subject: Optional[str] = None,
+        table_as_cells: Optional[dict[str, str | int]] = None,
         text_as_html: Optional[str] = None,
         url: Optional[str] = None,
     ) -> None:
@@ -246,22 +276,27 @@ class ElementMetadata:
 
         self.filetype = filetype
         self.header_footer_type = header_footer_type
+        self.image_base64 = image_base64
+        self.image_mime_type = image_mime_type
         self.image_path = image_path
         self.is_continuation = is_continuation
         self.languages = languages
         self.last_modified = last_modified
         self.link_texts = link_texts
         self.link_urls = link_urls
+        self.link_start_indexes = link_start_indexes
         self.links = links
+        self.orig_elements = orig_elements
         self.page_name = page_name
         self.page_number = page_number
         self.parent_id = parent_id
         self.regex_metadata = regex_metadata
-        self.section = section
         self.sent_from = sent_from
         self.sent_to = sent_to
+        self.signature = signature
         self.subject = subject
         self.text_as_html = text_as_html
+        self.table_as_cells = table_as_cells
         self.url = url
 
     def __eq__(self, other: object) -> bool:
@@ -291,12 +326,14 @@ class ElementMetadata:
         super().__setattr__(__name, __value)
 
     @classmethod
-    def from_dict(cls, meta_dict: Dict[str, Any]) -> ElementMetadata:
+    def from_dict(cls, meta_dict: dict[str, Any]) -> ElementMetadata:
         """Construct from a metadata-dict.
 
         This would generally be a dict formed using the `.to_dict()` method and stored as JSON
         before "rehydrating" it using this method.
         """
+        from unstructured.staging.base import elements_from_base64_gzipped_json
+
         # -- avoid unexpected mutation by working on a copy of provided dict --
         meta_dict = copy.deepcopy(meta_dict)
         self = ElementMetadata()
@@ -305,6 +342,10 @@ class ElementMetadata:
                 self.coordinates = CoordinatesMetadata.from_dict(field_value)
             elif field_name == "data_source":
                 self.data_source = DataSourceMetadata.from_dict(field_value)
+            elif field_name == "orig_elements":
+                self.orig_elements = elements_from_base64_gzipped_json(field_value)
+            elif field_name == "key_value_pairs":
+                self.key_value_pairs = _kvform_rehydrate_internal_elements(field_value)
             else:
                 setattr(self, field_name, field_value)
 
@@ -342,12 +383,14 @@ class ElementMetadata:
             }
         )
 
-    def to_dict(self) -> Dict[str, Any]:
+    def to_dict(self) -> dict[str, Any]:
         """Convert this metadata to dict form, suitable for JSON serialization.
 
         The returned dict is "sparse" in that no key-value pair appears for a field with value
         `None`.
         """
+        from unstructured.staging.base import elements_to_base64_gzipped_json
+
         meta_dict = copy.deepcopy(dict(self.fields))
 
         # -- remove fields that should not be serialized --
@@ -355,7 +398,7 @@ class ElementMetadata:
             meta_dict.pop(field_name, None)
 
         # -- don't serialize empty lists --
-        meta_dict: Dict[str, Any] = {
+        meta_dict: dict[str, Any] = {
             field_name: value
             for field_name, value in meta_dict.items()
             if value != [] and value != {}
@@ -366,6 +409,10 @@ class ElementMetadata:
             meta_dict["coordinates"] = self.coordinates.to_dict()
         if self.data_source is not None:
             meta_dict["data_source"] = self.data_source.to_dict()
+        if self.orig_elements is not None:
+            meta_dict["orig_elements"] = elements_to_base64_gzipped_json(self.orig_elements)
+        if self.key_value_pairs is not None:
+            meta_dict["key_value_pairs"] = _kvform_pairs_to_dict(self.key_value_pairs)
 
         return meta_dict
 
@@ -425,7 +472,7 @@ class ConsolidationStrategy(enum.Enum):
     """Combine regex-metadata of elements, adjust start and stop offsets for concatenated text."""
 
     @classmethod
-    def field_consolidation_strategies(cls) -> Dict[str, ConsolidationStrategy]:
+    def field_consolidation_strategies(cls) -> dict[str, ConsolidationStrategy]:
         """Mapping from ElementMetadata field-name to its consolidation strategy.
 
         Note that only _TextSection objects ("pre-chunks" containing only `Text` elements that are
@@ -446,39 +493,82 @@ class ConsolidationStrategy(enum.Enum):
             "filetype": cls.FIRST,
             "header_footer_type": cls.DROP,
             "image_path": cls.DROP,
+            "image_base64": cls.DROP,
+            "image_mime_type": cls.DROP,
             "is_continuation": cls.DROP,  # -- not expected, added by chunking, not before --
             "languages": cls.LIST_UNIQUE,
             "last_modified": cls.FIRST,
             "link_texts": cls.LIST_CONCATENATE,
             "link_urls": cls.LIST_CONCATENATE,
+            "link_start_indexes": cls.DROP,
             "links": cls.DROP,  # -- deprecated field --
             "max_characters": cls.DROP,  # -- unused, remove from ElementMetadata --
+            "orig_elements": cls.DROP,  # -- not expected, added by chunking, not before --
             "page_name": cls.FIRST,
             "page_number": cls.FIRST,
             "parent_id": cls.DROP,
             "regex_metadata": cls.REGEX,
-            "section": cls.FIRST,
             "sent_from": cls.FIRST,
             "sent_to": cls.FIRST,
+            "signature": cls.FIRST,
             "subject": cls.FIRST,
-            "text_as_html": cls.DROP,  # -- not expected, only occurs in _TableSection --
+            "text_as_html": cls.FIRST,  # -- only occurs in Table --
+            "table_as_cells": cls.FIRST,  # -- only occurs in Table --
             "url": cls.FIRST,
+            "key_value_pairs": cls.DROP,  # -- only occurs in FormKeysValues --
         }
 
 
 _P = ParamSpec("_P")
 
 
-def process_metadata() -> Callable[[Callable[_P, List[Element]]], Callable[_P, List[Element]]]:
+def assign_and_map_hash_ids(elements: list[Element]) -> list[Element]:
+    """Converts `id` and `parent_id` of elements from UUIDs to hashes.
+
+    This function ensures deterministic IDs by:
+    1. Converting each element's UUID into a hash.
+    2. Updating the `parent_id` to match the new hash ID of parent elements.
+
+    Args:
+        elements: A list of Element objects to update.
+
+    Returns:
+        List of updated Element objects with hashes for `id` and `parent_id`.
+    """
+    # -- generate sequence number for each element on a page --
+    page_numbers = [e.metadata.page_number for e in elements]
+    page_seq_pairs = [
+        seq_on_page for page, group in groupby(page_numbers) for seq_on_page, _ in enumerate(group)
+    ]
+
+    # -- assign hash IDs to elements --
+    old_to_new_mapping = {
+        element.id: element.id_to_hash(seq_on_page_counter)
+        for element, seq_on_page_counter in zip(elements, page_seq_pairs)
+    }
+
+    # -- map old parent IDs to new ones --
+    for e in elements:
+        parent_id = e.metadata.parent_id
+        if not parent_id:
+            continue
+        e.metadata.parent_id = old_to_new_mapping[parent_id]
+
+    return elements
+
+
+def process_metadata() -> Callable[[Callable[_P, list[Element]]], Callable[_P, list[Element]]]:
     """Post-process element-metadata for this document.
 
-    This decorator adds a post-processing step to a document partitioner. It adds documentation for
-    `metadata_filename` and `include_metadata` parameters if not present. Also adds regex-metadata
-    when `regex_metadata` keyword-argument is provided and changes the element-id to a UUID when
-    `unique_element_ids` argument is provided and True.
+    This decorator adds a post-processing step to a document partitioner.
+
+    - Adds `metadata_filename` and `include_metadata` parameters to docstring if not present.
+    - Adds `.metadata.regex-metadata` when `regex_metadata` keyword-argument is provided.
+    - Updates element.id to a UUID when `unique_element_ids` argument is provided and True.
+
     """
 
-    def decorator(func: Callable[_P, List[Element]]) -> Callable[_P, List[Element]]:
+    def decorator(func: Callable[_P, list[Element]]) -> Callable[_P, list[Element]]:
         if func.__doc__:
             if (
                 "metadata_filename" in func.__code__.co_varnames
@@ -499,24 +589,20 @@ def process_metadata() -> Callable[[Callable[_P, List[Element]]], Callable[_P, L
                 )
 
         @functools.wraps(func)
-        def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> List[Element]:
+        def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> list[Element]:
             elements = func(*args, **kwargs)
-            sig = inspect.signature(func)
-            params: Dict[str, Any] = dict(**dict(zip(sig.parameters, args)), **kwargs)
-            for param in sig.parameters.values():
-                if param.name not in params and param.default is not param.empty:
-                    params[param.name] = param.default
+            call_args = get_call_args_applying_defaults(func, *args, **kwargs)
 
-            regex_metadata: Dict["str", "str"] = params.get("regex_metadata", {})
+            regex_metadata: dict["str", "str"] = call_args.get("regex_metadata", {})
             # -- don't write an empty `{}` to metadata.regex_metadata when no regex-metadata was
             # -- requested, otherwise it will serialize (because it's not None) when it has no
             # -- meaning or is even misleading. Also it complicates tests that don't use regex-meta.
             if regex_metadata:
                 elements = _add_regex_metadata(elements, regex_metadata)
-            unique_element_ids: bool = params.get("unique_element_ids", False)
-            if unique_element_ids:
-                for element in elements:
-                    element.id_to_uuid()
+
+            unique_element_ids: bool = call_args.get("unique_element_ids", False)
+            if unique_element_ids is False:
+                elements = assign_and_map_hash_ids(elements)
 
             return elements
 
@@ -526,18 +612,18 @@ def process_metadata() -> Callable[[Callable[_P, List[Element]]], Callable[_P, L
 
 
 def _add_regex_metadata(
-    elements: List[Element],
-    regex_metadata: Dict[str, str] = {},
-) -> List[Element]:
+    elements: list[Element],
+    regex_metadata: dict[str, str] = {},
+) -> list[Element]:
     """Adds metadata based on a user provided regular expression.
 
     The additional metadata will be added to the regex_metadata attrbuted in the element metadata.
     """
     for element in elements:
         if isinstance(element, Text):
-            _regex_metadata: Dict["str", List[RegexMetadata]] = {}
+            _regex_metadata: dict["str", list[RegexMetadata]] = {}
             for field_name, pattern in regex_metadata.items():
-                results: List[RegexMetadata] = []
+                results: list[RegexMetadata] = []
                 for result in re.finditer(pattern, element.text):
                     start, end = result.span()
                     results.append(
@@ -555,44 +641,124 @@ def _add_regex_metadata(
     return elements
 
 
+class ElementType:
+    TITLE = "Title"
+    TEXT = "Text"
+    UNCATEGORIZED_TEXT = "UncategorizedText"
+    NARRATIVE_TEXT = "NarrativeText"
+    BULLETED_TEXT = "BulletedText"
+    PARAGRAPH = "Paragraph"
+    ABSTRACT = "Abstract"
+    THREADING = "Threading"
+    FORM = "Form"
+    FIELD_NAME = "Field-Name"
+    VALUE = "Value"
+    LINK = "Link"
+    COMPOSITE_ELEMENT = "CompositeElement"
+    IMAGE = "Image"
+    PICTURE = "Picture"
+    FIGURE_CAPTION = "FigureCaption"
+    FIGURE = "Figure"
+    CAPTION = "Caption"
+    LIST = "List"
+    LIST_ITEM = "ListItem"
+    LIST_ITEM_OTHER = "List-item"
+    CHECKED = "Checked"
+    UNCHECKED = "Unchecked"
+    CHECK_BOX_CHECKED = "CheckBoxChecked"
+    CHECK_BOX_UNCHECKED = "CheckBoxUnchecked"
+    RADIO_BUTTON_CHECKED = "RadioButtonChecked"
+    RADIO_BUTTON_UNCHECKED = "RadioButtonUnchecked"
+    ADDRESS = "Address"
+    EMAIL_ADDRESS = "EmailAddress"
+    PAGE_BREAK = "PageBreak"
+    FORMULA = "Formula"
+    TABLE = "Table"
+    HEADER = "Header"
+    HEADLINE = "Headline"
+    SUB_HEADLINE = "Subheadline"
+    PAGE_HEADER = "Page-header"  # Title?
+    SECTION_HEADER = "Section-header"
+    FOOTER = "Footer"
+    FOOTNOTE = "Footnote"
+    PAGE_FOOTER = "Page-footer"
+    PAGE_NUMBER = "PageNumber"
+    CODE_SNIPPET = "CodeSnippet"
+    FORM_KEYS_VALUES = "FormKeysValues"
+
+    @classmethod
+    def to_dict(cls):
+        """
+        Convert class attributes to a dictionary.
+
+        Returns:
+            dict: A dictionary where keys are attribute names and values are attribute values.
+        """
+        return {
+            attr: getattr(cls, attr)
+            for attr in dir(cls)
+            if not callable(getattr(cls, attr)) and not attr.startswith("__")
+        }
+
+
 class Element(abc.ABC):
-    """An element is a section of a page in the document."""
+    """An element is a semantically-coherent component of a document, often a paragraph.
+
+    There are a few design principles that are followed when creating an element:
+    1. It will always have an ID, which by default is a random UUID.
+    2. Asking for an ID should always return a string, it can never be None.
+    3. ID is lazy, meaning it will be generated when asked for the first time.
+    4. When deterministic behavior is needed, the ID can be converted.
+        to a hash based on its text `element.id_to_hash(position)`
+    4. Even if the `text` attribute is not defined in a subclass, it will default to a blank string.
+    6. Assigning a string ID manually is possible, but is meant to be used
+        only for deserialization purposes.
+    """
+
+    text: str
+    category = "UncategorizedText"
 
     def __init__(
         self,
-        element_id: Union[str, uuid.UUID, NoID, UUID] = NoID(),
-        coordinates: Optional[Tuple[Tuple[float, float], ...]] = None,
+        element_id: Optional[str] = None,
+        coordinates: Optional[tuple[tuple[float, float], ...]] = None,
         coordinate_system: Optional[CoordinateSystem] = None,
         metadata: Optional[ElementMetadata] = None,
         detection_origin: Optional[str] = None,
     ):
-        self.id: Union[str, uuid.UUID, NoID, UUID] = element_id
+        if element_id is not None and not isinstance(
+            element_id, str
+        ):  # pyright: ignore[reportUnnecessaryIsInstance]
+            raise ValueError("element_id must be of type str or None.")
+
+        self._element_id = element_id
         self.metadata = ElementMetadata() if metadata is None else metadata
         if coordinates is not None or coordinate_system is not None:
             self.metadata.coordinates = CoordinatesMetadata(
                 points=coordinates, system=coordinate_system
             )
         self.metadata.detection_origin = detection_origin
+        # -- all `Element` instances get a `text` attribute, defaults to the empty string if not
+        # -- defined in a subclass.
+        self.text = self.text if hasattr(self, "text") else ""
 
-    def id_to_uuid(self):
-        self.id = str(uuid.uuid4())
-
-    def to_dict(self) -> Dict[str, Any]:
-        return {
-            "type": None,
-            "element_id": self.id,
-            "metadata": self.metadata.to_dict(),
-        }
+    def __str__(self):
+        return self.text
 
     def convert_coordinates_to_new_system(
-        self,
-        new_system: CoordinateSystem,
-        in_place=True,
-    ) -> Optional[Tuple[Tuple[Union[int, float], Union[int, float]], ...]]:
-        """Converts the element location coordinates to a new coordinate system. If inplace is true,
-        changes the coordinates in place and updates the coordinate system."""
-        if self.metadata.coordinates is None:
+        self, new_system: CoordinateSystem, in_place: bool = True
+    ) -> Optional[Points]:
+        """Converts the element location coordinates to a new coordinate system.
+
+        If inplace is true, changes the coordinates in place and updates the coordinate system.
+        """
+        if (
+            self.metadata.coordinates is None
+            or self.metadata.coordinates.system is None
+            or self.metadata.coordinates.points is None
+        ):
             return None
+
         new_coordinates = tuple(
             self.metadata.coordinates.system.convert_coordinates_to_new_system(
                 new_system=new_system,
@@ -601,20 +767,53 @@ class Element(abc.ABC):
             )
             for x, y in self.metadata.coordinates.points
         )
+
         if in_place:
             self.metadata.coordinates.points = new_coordinates
             self.metadata.coordinates.system = new_system
+
         return new_coordinates
 
+    def id_to_hash(self, sequence_number: int) -> str:
+        """Calculates and assigns a deterministic hash as an ID.
+
+        The hash ID is based on element's text, sequence number on page,
+        page number and its filename.
+
+        Args:
+            sequence_number: index on page
+
+        Returns: new ID value
+        """
+        data = f"{self.metadata.filename}{self.text}{self.metadata.page_number}{sequence_number}"
+        self._element_id = hashlib.sha256(data.encode()).hexdigest()[:32]
+        return self.id
+
+    @property
+    def id(self):
+        if self._element_id is None:
+            self._element_id = str(uuid.uuid4())
+        return self._element_id
+
+    def to_dict(self) -> dict[str, Any]:
+        return {
+            "type": None,
+            "element_id": self.id,
+            "text": self.text,
+            "metadata": self.metadata.to_dict(),
+        }
+
 
 class CheckBox(Element):
-    """A checkbox with an attribute indicating whether its checked or not. Primarily used
-    in documents that are forms"""
+    """A checkbox with an attribute indicating whether its checked or not.
+
+    Primarily used in documents that are forms.
+    """
 
     def __init__(
         self,
-        element_id: Union[str, uuid.UUID, NoID, UUID] = NoID(),
-        coordinates: Optional[Tuple[Tuple[float, float], ...]] = None,
+        element_id: Optional[str] = None,
+        coordinates: Optional[tuple[tuple[float, float], ...]] = None,
         coordinate_system: Optional[CoordinateSystem] = None,
         checked: bool = False,
         metadata: Optional[ElementMetadata] = None,
@@ -630,12 +829,18 @@ class CheckBox(Element):
         )
         self.checked: bool = checked
 
-    def __eq__(self, other):
-        return (self.checked == other.checked) and (
-            self.metadata.coordinates == other.metadata.coordinates
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, CheckBox):
+            return False
+        return all(
+            (
+                self.checked == other.checked,
+                self.metadata.coordinates == other.metadata.coordinates,
+            )
         )
 
-    def to_dict(self) -> dict:
+    def to_dict(self) -> dict[str, Any]:
+        """Serialize to JSON-compatible (str keys) dict."""
         out = super().to_dict()
         out["type"] = "CheckBox"
         out["checked"] = self.checked
@@ -646,28 +851,19 @@ class CheckBox(Element):
 class Text(Element):
     """Base element for capturing free text from within document."""
 
-    category = "UncategorizedText"
-
     def __init__(
         self,
         text: str,
-        element_id: Union[str, uuid.UUID, NoID, UUID] = NoID(),
-        coordinates: Optional[Tuple[Tuple[float, float], ...]] = None,
+        element_id: Optional[str] = None,
+        coordinates: Optional[tuple[tuple[float, float], ...]] = None,
         coordinate_system: Optional[CoordinateSystem] = None,
         metadata: Optional[ElementMetadata] = None,
         detection_origin: Optional[str] = None,
-        embeddings: Optional[List[float]] = None,
+        embeddings: Optional[list[float]] = None,
     ):
         metadata = metadata if metadata else ElementMetadata()
         self.text: str = text
-        self.embeddings: Optional[List[float]] = embeddings
-
-        if isinstance(element_id, NoID):
-            # NOTE(robinson) - Cut the SHA256 hex in half to get the first 128 bits
-            element_id = hashlib.sha256(text.encode()).hexdigest()[:32]
-
-        elif isinstance(element_id, UUID):
-            element_id = str(uuid.uuid4())
+        self.embeddings: Optional[list[float]] = embeddings
 
         super().__init__(
             element_id=element_id,
@@ -677,20 +873,38 @@ class Text(Element):
             detection_origin=detection_origin,
         )
 
+    def __eq__(self, other: object):
+        if not isinstance(other, Text):
+            return False
+        return all(
+            (
+                self.text == other.text,
+                self.metadata.coordinates == other.metadata.coordinates,
+                self.category == other.category,
+                self.embeddings == other.embeddings,
+            ),
+        )
+
     def __str__(self):
         return self.text
 
-    def __eq__(self, other):
-        return all(
-            [
-                (self.text == other.text),
-                (self.metadata.coordinates == other.metadata.coordinates),
-                (self.category == other.category),
-                (self.embeddings == other.embeddings),
-            ],
-        )
+    def apply(self, *cleaners: Callable[[str], str]):
+        """Applies a cleaning brick to the text element.
 
-    def to_dict(self) -> dict:
+        The function that's passed in should take a string as input and produce a string as
+        output.
+        """
+        cleaned_text = self.text
+        for cleaner in cleaners:
+            cleaned_text = cleaner(cleaned_text)
+
+        if not isinstance(cleaned_text, str):  # pyright: ignore[reportUnnecessaryIsInstance]
+            raise ValueError("Cleaner produced a non-string output.")
+
+        self.text = cleaned_text
+
+    def to_dict(self) -> dict[str, Any]:
+        """Serialize to JSON-compatible (str keys) dict."""
         out = super().to_dict()
         out["element_id"] = self.id
         out["type"] = self.category
@@ -699,18 +913,6 @@ class Text(Element):
             out["embeddings"] = self.embeddings
         return out
 
-    def apply(self, *cleaners: Callable):
-        """Applies a cleaning brick to the text element. The function that's passed in
-        should take a string as input and produce a string as output."""
-        cleaned_text = self.text
-        for cleaner in cleaners:
-            cleaned_text = cleaner(cleaned_text)
-
-        if not isinstance(cleaned_text, str):
-            raise ValueError("Cleaner produced a non-string output.")
-
-        self.text = cleaned_text
-
 
 class Formula(Text):
     "An element containing formulas in a document"
@@ -719,7 +921,12 @@ class Formula(Text):
 
 
 class CompositeElement(Text):
-    """A section of text consisting of a combination of elements."""
+    """A chunk formed from text (non-Table) elements.
+
+    Only produced by chunking. An instance may be formed by combining one or more sequential
+    elements produced by partitioning. It it also used when text-splitting an "oversized" element,
+    a single element that by itself is larger than the requested chunk size.
+    """
 
     category = "CompositeElement"
 
@@ -764,7 +971,7 @@ class EmailAddress(Text):
 class Image(Text):
     """A text element for capturing image metadata."""
 
-    category = "Image"
+    category = ElementType.IMAGE
 
 
 class PageBreak(Text):
@@ -797,54 +1004,35 @@ class Footer(Text):
     category = "Footer"
 
 
-class ElementType:
-    TITLE = "Title"
-    TEXT = "Text"
-    UNCATEGORIZED_TEXT = "UncategorizedText"
-    NARRATIVE_TEXT = "NarrativeText"
-    BULLETED_TEXT = "BulletedText"
-    ABSTRACT = "Abstract"
-    THREADING = "Threading"
-    FORM = "Form"
-    FIELD_NAME = "Field-Name"
-    VALUE = "Value"
-    LINK = "Link"
-    COMPOSITE_ELEMENT = "CompositeElement"
-    IMAGE = "Image"
-    PICTURE = "Picture"
-    FIGURE_CAPTION = "FigureCaption"
-    FIGURE = "Figure"
-    CAPTION = "Caption"
-    LIST = "List"
-    LIST_ITEM = "ListItem"
-    LIST_ITEM_OTHER = "List-item"
-    CHECKED = "Checked"
-    UNCHECKED = "Unchecked"
-    ADDRESS = "Address"
-    EMAIL_ADDRESS = "EmailAddress"
-    PAGE_BREAK = "PageBreak"
-    FORMULA = "Formula"
-    TABLE = "Table"
-    HEADER = "Header"
-    HEADLINE = "Headline"
-    SUB_HEADLINE = "Subheadline"
-    PAGE_HEADER = "Page-header"  # Title?
-    SECTION_HEADER = "Section-header"
-    FOOTER = "Footer"
-    FOOTNOTE = "Footnote"
-    PAGE_FOOTER = "Page-footer"
+class CodeSnippet(Text):
+    """An element for capturing code snippets."""
+
+    category = "CodeSnippet"
 
 
-TYPE_TO_TEXT_ELEMENT_MAP: Dict[str, Any] = {
+class PageNumber(Text):
+    """An element for capturing page numbers."""
+
+    category = "PageNumber"
+
+
+class FormKeysValues(Text):
+    """An element for capturing Key-Value dicts (forms)."""
+
+    category = "FormKeysValues"
+
+
+TYPE_TO_TEXT_ELEMENT_MAP: dict[str, type[Text]] = {
     ElementType.TITLE: Title,
     ElementType.SECTION_HEADER: Title,
     ElementType.HEADLINE: Title,
     ElementType.SUB_HEADLINE: Title,
     ElementType.FIELD_NAME: Title,
     ElementType.UNCATEGORIZED_TEXT: Text,
-    ElementType.COMPOSITE_ELEMENT: Text,
+    ElementType.COMPOSITE_ELEMENT: CompositeElement,
     ElementType.TEXT: NarrativeText,
     ElementType.NARRATIVE_TEXT: NarrativeText,
+    ElementType.PARAGRAPH: NarrativeText,
     # this mapping favors ensures yolox produces backward compatible categories
     ElementType.ABSTRACT: NarrativeText,
     ElementType.THREADING: NarrativeText,
@@ -869,4 +1057,45 @@ TYPE_TO_TEXT_ELEMENT_MAP: Dict[str, Any] = {
     ElementType.EMAIL_ADDRESS: EmailAddress,
     ElementType.FORMULA: Formula,
     ElementType.PAGE_BREAK: PageBreak,
+    ElementType.CODE_SNIPPET: CodeSnippet,
+    ElementType.PAGE_NUMBER: PageNumber,
+    ElementType.FORM_KEYS_VALUES: FormKeysValues,
 }
+
+
+def _kvform_rehydrate_internal_elements(kv_pairs: list[dict]) -> list[FormKeyValuePair]:
+    """
+    The key_value_pairs metadata field contains (in the vast majority of cases)
+    nested Text elements. Those need to be turned from dicts into Elements explicitly,
+    e.g. when partition_json is used.
+    """
+    from unstructured.staging.base import elements_from_dicts
+
+    # safe to overwrite - deepcopy already happened
+    for kv_pair in kv_pairs:
+        if kv_pair["key"]["custom_element"] is not None:
+            (kv_pair["key"]["custom_element"],) = elements_from_dicts(
+                [kv_pair["key"]["custom_element"]]
+            )
+        if kv_pair["value"] is not None and kv_pair["value"]["custom_element"] is not None:
+            (kv_pair["value"]["custom_element"],) = elements_from_dicts(
+                [kv_pair["value"]["custom_element"]]
+            )
+    return kv_pairs
+
+
+def _kvform_pairs_to_dict(kv_pairs: list[FormKeyValuePair]) -> list[dict]:
+    """
+    The key_value_pairs metadata field contains (in the vast majority of cases)
+    nested Text elements. Those need to be turned from Elements to dicts recursively,
+    e.g. when FormKeysValues.to_dict() is used.
+
+    """
+    kv_pairs: list[dict] = copy.deepcopy(kv_pairs)
+    for kv_pair in kv_pairs:
+        if kv_pair["key"]["custom_element"] is not None:
+            kv_pair["key"]["custom_element"] = kv_pair["key"]["custom_element"].to_dict()
+        if kv_pair["value"] is not None and kv_pair["value"]["custom_element"] is not None:
+            kv_pair["value"]["custom_element"] = kv_pair["value"]["custom_element"].to_dict()
+
+    return kv_pairs
