@@ -2,12 +2,11 @@ from __future__ import annotations
 
 import enum
 import functools
-import inspect
 import json
 import os
 import re
 import zipfile
-from typing import IO, Any, Callable, Dict, List, Optional
+from typing import IO, Callable, List, Optional
 
 from typing_extensions import ParamSpec
 
@@ -15,11 +14,12 @@ from unstructured.documents.elements import Element
 from unstructured.file_utils.encoding import detect_file_encoding, format_encoding_str
 from unstructured.nlp.patterns import LIST_OF_DICTS_PATTERN
 from unstructured.partition.common import (
-    _add_element_metadata,
-    _remove_element_metadata,
+    add_element_metadata,
     exactly_one,
+    remove_element_metadata,
     set_element_hierarchy,
 )
+from unstructured.utils import get_call_args_applying_defaults
 
 try:
     import magic
@@ -74,6 +74,8 @@ class FileType(enum.Enum):
     JPG = 30
     PNG = 31
     TIFF = 32
+    BMP = 33
+    HEIC = 34
 
     # Plain Text Types
     EML = 40
@@ -97,6 +99,9 @@ class FileType(enum.Enum):
     # Open Office Types
     ODT = 70
 
+    # Audio Files
+    WAV = 80
+
     # NOTE(robinson) - This is to support sorting for pandas groupby functions
     def __lt__(self, other):
         return self.name < other.name
@@ -108,7 +113,16 @@ STR_TO_FILETYPE = {
     "application/vnd.openxmlformats-officedocument.wordprocessingml.document": FileType.DOCX,
     "image/jpeg": FileType.JPG,
     "image/png": FileType.PNG,
+    "image/heic": FileType.HEIC,
     "image/tiff": FileType.TIFF,
+    "image/bmp": FileType.BMP,
+    # NOTE(robinson) - https://mimetype.io/application/yaml
+    # In the future, we may have special processing for YAML
+    # files instead of treating them as plaintext
+    "application/yaml": FileType.TXT,
+    "application/x-yaml": FileType.TXT,
+    "text/x-yaml": FileType.TXT,
+    "text/yaml": FileType.TXT,
     "text/plain": FileType.TXT,
     "text/x-csv": FileType.CSV,
     "application/csv": FileType.CSV,
@@ -136,6 +150,12 @@ STR_TO_FILETYPE = {
     "message/rfc822": FileType.EML,
     "application/x-ole-storage": FileType.MSG,
     "application/vnd.ms-outlook": FileType.MSG,
+    # NOTE(robinson) - https://mimetype.io/audio/wav
+    "audio/vnd.wav": FileType.WAV,
+    "audio/vnd.wave": FileType.WAV,
+    "audio/wave": FileType.WAV,
+    "audio/x-pn-wav": FileType.WAV,
+    "audio/x-wav": FileType.WAV,
     "inode/x-empty": FileType.EMPTY,
 }
 
@@ -161,6 +181,7 @@ EXT_TO_FILETYPE = {
     ".log": FileType.TXT,
     ".eml": FileType.EML,
     ".xml": FileType.XML,
+    ".heic": FileType.HEIC,
     ".htm": FileType.HTML,
     ".html": FileType.HTML,
     ".md": FileType.MD,
@@ -168,6 +189,7 @@ EXT_TO_FILETYPE = {
     ".rst": FileType.RST,
     ".xlsx": FileType.XLSX,
     ".pptx": FileType.PPTX,
+    ".p7s": FileType.EML,
     ".png": FileType.PNG,
     ".doc": FileType.DOC,
     ".zip": FileType.ZIP,
@@ -182,6 +204,8 @@ EXT_TO_FILETYPE = {
     ".tsv": FileType.TSV,
     ".tab": FileType.TSV,
     ".tiff": FileType.TIFF,
+    ".bmp": FileType.BMP,
+    ".wav": FileType.WAV,
     # NOTE(robinson) - for now we are treating code files as plain text
     ".js": FileType.TXT,
     ".py": FileType.TXT,
@@ -196,6 +220,8 @@ EXT_TO_FILETYPE = {
     ".swift": FileType.TXT,
     ".ts": FileType.TXT,
     ".go": FileType.TXT,
+    ".yaml": FileType.TXT,
+    ".yml": FileType.TXT,
     None: FileType.UNK,
 }
 
@@ -203,6 +229,7 @@ PLAIN_TEXT_EXTENSIONS = [
     ".txt",
     ".text",
     ".eml",
+    ".p7s",
     ".md",
     ".rtf",
     ".html",
@@ -304,6 +331,7 @@ def detect_filetype(
 
         if extension in [
             ".eml",
+            ".p7s",
             ".md",
             ".rtf",
             ".html",
@@ -336,7 +364,7 @@ def detect_filetype(
             return FileType.EML
 
         if extension in PLAIN_TEXT_EXTENSIONS:
-            return EXT_TO_FILETYPE.get(extension)
+            return EXT_TO_FILETYPE.get(extension, FileType.UNK)
 
         # Safety catch
         if mime_type in STR_TO_FILETYPE:
@@ -449,7 +477,15 @@ def _is_text_file_a_json(
         encoding=encoding,
     )
     try:
-        json.loads(file_text)
+        output = json.loads(file_text)
+        # NOTE(robinson) - Per RFC 4627 which defines the application/json media type,
+        # a string is a valid JSON. For our purposes, however, we want to treat that
+        # as a text file even if it is serializable as json.
+        # References:
+        # https://stackoverflow.com/questions/7487869/is-this-simple-string-considered-valid-json
+        # https://www.ietf.org/rfc/rfc4627.txt
+        if isinstance(output, str):
+            return False
         return True
     except json.JSONDecodeError:
         return False
@@ -544,18 +580,14 @@ def add_metadata(func: Callable[_P, List[Element]]) -> Callable[_P, List[Element
     @functools.wraps(func)
     def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> List[Element]:
         elements = func(*args, **kwargs)
-        sig = inspect.signature(func)
-        params: Dict[str, Any] = dict(**dict(zip(sig.parameters, args)), **kwargs)
-        for param in sig.parameters.values():
-            if param.name not in params and param.default is not param.empty:
-                params[param.name] = param.default
-        include_metadata = params.get("include_metadata", True)
+        call_args = get_call_args_applying_defaults(func, *args, **kwargs)
+        include_metadata = call_args.get("include_metadata", True)
         if include_metadata:
-            if params.get("metadata_filename"):
-                params["filename"] = params.get("metadata_filename")
+            if call_args.get("metadata_filename"):
+                call_args["filename"] = call_args.get("metadata_filename")
 
             metadata_kwargs = {
-                kwarg: params.get(kwarg) for kwarg in ("filename", "url", "text_as_html")
+                kwarg: call_args.get(kwarg) for kwarg in ("filename", "url", "text_as_html")
             }
             # NOTE (yao): do not use cast here as cast(None) still is None
             if not str(kwargs.get("model_name", "")).startswith("chipper"):
@@ -566,16 +598,11 @@ def add_metadata(func: Callable[_P, List[Element]]) -> Callable[_P, List[Element
                 # NOTE(robinson) - Attached files have already run through this logic
                 # in their own partitioning function
                 if element.metadata.attached_to_filename is None:
-                    _add_element_metadata(
-                        element,
-                        **metadata_kwargs,  # type: ignore
-                    )
+                    add_element_metadata(element, **metadata_kwargs)
 
             return elements
         else:
-            return _remove_element_metadata(
-                elements,
-            )
+            return remove_element_metadata(elements)
 
     return wrapper
 
@@ -583,36 +610,32 @@ def add_metadata(func: Callable[_P, List[Element]]) -> Callable[_P, List[Element
 def add_filetype(
     filetype: FileType,
 ) -> Callable[[Callable[_P, List[Element]]], Callable[_P, List[Element]]]:
-    """..."""
+    """Post-process element-metadata for list[Element] from partitioning.
+
+    This decorator adds a post-processing step to a document partitioner.
+
+    - Adds `metadata_filename` and `include_metadata` parameters to docstring if not present.
+    - Adds `.metadata.regex-metadata` when `regex_metadata` keyword-argument is provided.
+    - Updates element.id to a UUID when `unique_element_ids` argument is provided and True.
+
+    """
 
     def decorator(func: Callable[_P, List[Element]]) -> Callable[_P, List[Element]]:
         @functools.wraps(func)
         def wrapper(*args: _P.args, **kwargs: _P.kwargs) -> List[Element]:
             elements = func(*args, **kwargs)
-            sig = inspect.signature(func)
-            params: Dict[str, Any] = dict(**dict(zip(sig.parameters, args)), **kwargs)
-            for param in sig.parameters.values():
-                if param.name not in params and param.default is not param.empty:
-                    params[param.name] = param.default
+            params = get_call_args_applying_defaults(func, *args, **kwargs)
             include_metadata = params.get("include_metadata", True)
             if include_metadata:
-                if params.get("metadata_filename"):
-                    params["filename"] = params.get("metadata_filename")
-
                 for element in elements:
                     # NOTE(robinson) - Attached files have already run through this logic
                     # in their own partitioning function
                     if element.metadata.attached_to_filename is None:
-                        _add_element_metadata(
-                            element,
-                            filetype=FILETYPE_TO_MIMETYPE[filetype],
-                        )
+                        add_element_metadata(element, filetype=FILETYPE_TO_MIMETYPE[filetype])
 
                 return elements
             else:
-                return _remove_element_metadata(
-                    elements,
-                )
+                return remove_element_metadata(elements)
 
         return wrapper
 
