@@ -7,16 +7,11 @@ from unstructured.ingest.cli.base.src import BaseSrcCmd
 from unstructured.ingest.cli.interfaces import (
     CliConfig,
 )
+from unstructured.ingest.connector.biomed import SimpleBiomedConfig
 
 
 @dataclass
-class BiomedCliConfig(CliConfig):
-    api_id: t.Optional[str] = None
-    api_from: t.Optional[str] = None
-    api_until: t.Optional[str] = None
-    path: t.Optional[str] = None
-    max_request_time: int = 45
-
+class BiomedCliConfig(SimpleBiomedConfig, CliConfig):
     @staticmethod
     def get_cli_options() -> t.List[click.Option]:
         options = [
@@ -50,5 +45,8 @@ class BiomedCliConfig(CliConfig):
 
 
 def get_base_src_cmd() -> BaseSrcCmd:
-    cmd_cls = BaseSrcCmd(cmd_name="biomed", cli_config=BiomedCliConfig)
+    cmd_cls = BaseSrcCmd(
+        cmd_name="biomed",
+        cli_config=BiomedCliConfig,
+    )
     return cmd_cls
