@@ -8,14 +8,11 @@ from unstructured.ingest.cli.interfaces import (
     CliConfig,
     DelimitedString,
 )
+from unstructured.ingest.connector.discord import SimpleDiscordConfig
 
 
 @dataclass
-class DiscordCliConfig(CliConfig):
-    channels: t.List[str]
-    token: str
-    period: t.Optional[int] = None
-
+class DiscordCliConfig(SimpleDiscordConfig, CliConfig):
     @staticmethod
     def get_cli_options() -> t.List[click.Option]:
         options = [
@@ -34,6 +31,7 @@ class DiscordCliConfig(CliConfig):
             click.Option(
                 ["--period"],
                 default=None,
+                type=click.IntRange(0),
                 help="Number of days to go back in the history of "
                 "discord channels, must be a number",
             ),
@@ -42,5 +40,8 @@ class DiscordCliConfig(CliConfig):
 
 
 def get_base_src_cmd() -> BaseSrcCmd:
-    cmd_cls = BaseSrcCmd(cmd_name="discord", cli_config=DiscordCliConfig)
+    cmd_cls = BaseSrcCmd(
+        cmd_name="discord",
+        cli_config=DiscordCliConfig,
+    )
     return cmd_cls
