@@ -9,16 +9,11 @@ from unstructured.ingest.cli.interfaces import (
     CliRecursiveConfig,
     DelimitedString,
 )
+from unstructured.ingest.connector.notion.connector import SimpleNotionConfig
 
 
 @dataclass
-class NotionCliConfig(CliConfig):
-    notion_api_key: str
-    page_ids: t.Optional[t.List[str]]
-    database_ids: t.Optional[t.List[str]]
-    max_retries: t.Optional[int] = None
-    max_time: t.Optional[float] = None
-
+class NotionCliConfig(SimpleNotionConfig, CliConfig):
     @staticmethod
     def get_cli_options() -> t.List[click.Option]:
         options = [
