@@ -7,17 +7,11 @@ from unstructured.ingest.cli.base.src import BaseSrcCmd
 from unstructured.ingest.cli.interfaces import (
     CliConfig,
 )
+from unstructured.ingest.connector.reddit import SimpleRedditConfig
 
 
 @dataclass
-class RedditCliConfig(CliConfig):
-    client_id: str
-    client_secret: str
-    subreddit_name: str
-    user_agent: str
-    num_posts: int
-    search_query: t.Optional[str] = None
-
+class RedditCliConfig(SimpleRedditConfig, CliConfig):
     @staticmethod
     def get_cli_options() -> t.List[click.Option]:
         options = [
@@ -66,5 +60,8 @@ class RedditCliConfig(CliConfig):
 
 
 def get_base_src_cmd() -> BaseSrcCmd:
-    cmd_cls = BaseSrcCmd(cmd_name="reddit", cli_config=RedditCliConfig)
+    cmd_cls = BaseSrcCmd(
+        cmd_name="reddit",
+        cli_config=RedditCliConfig,
+    )
     return cmd_cls
