@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import json
 import os.path
 import typing as t
@@ -7,8 +9,10 @@ from gettext import gettext, ngettext
 from pathlib import Path
 
 import click
-from dataclasses_json.core import Json, _decode_dataclass
+from dataclasses_json.core import Json
+from typing_extensions import Self
 
+from unstructured.chunking import CHUNK_MAX_CHARS_DEFAULT, CHUNK_MULTI_PAGE_DEFAULT
 from unstructured.ingest.interfaces import (
     BaseConfig,
     ChunkingConfig,
@@ -28,8 +32,8 @@ class Dict(click.ParamType):
     def convert(
         self,
         value: t.Any,
-        param: t.Optional[click.Parameter],
-        ctx: t.Optional[click.Context],
+        param: t.Optional[click.Parameter] = None,
+        ctx: t.Optional[click.Context] = None,
     ) -> t.Any:
         try:
             return json.loads(value)
@@ -46,11 +50,14 @@ class Dict(click.ParamType):
 class FileOrJson(click.ParamType):
     name = "file-or-json"
 
+    def __init__(self, allow_raw_str: bool = False):
+        self.allow_raw_str = allow_raw_str
+
     def convert(
         self,
         value: t.Any,
-        param: t.Optional[click.Parameter],
-        ctx: t.Optional[click.Context],
+        param: t.Optional[click.Parameter] = None,
+        ctx: t.Optional[click.Context] = None,
     ) -> t.Any:
         # check if valid file
         full_path = os.path.abspath(os.path.expanduser(value))
@@ -60,7 +67,8 @@ class FileOrJson(click.ParamType):
             try:
                 return json.loads(value)
             except json.JSONDecodeError:
-                pass
+                if self.allow_raw_str:
+                    return value
         self.fail(
             gettext(
                 "{value} is not a valid json string nor an existing filepath.",
@@ -80,8 +88,8 @@ class DelimitedString(click.ParamType):
     def convert(
         self,
         value: t.Any,
-        param: t.Optional[click.Parameter],
-        ctx: t.Optional[click.Context],
+        param: t.Optional[click.Parameter] = None,
+        ctx: t.Optional[click.Context] = None,
     ) -> t.Any:
         # In case a list is provided as the default, will not break
         if isinstance(value, list):
@@ -155,12 +163,7 @@ class CliRetryStrategyConfig(RetryStrategyConfig, CliMixin):
         return options
 
     @classmethod
-    def from_dict(
-        cls,
-        kvs: Json,
-        *,
-        infer_missing=False,
-    ):
+    def from_dict(cls, kvs: Json, **kwargs):
         """
         Return None if none of the fields are being populated
         """
@@ -169,7 +172,7 @@ class CliRetryStrategyConfig(RetryStrategyConfig, CliMixin):
             field_values = [kvs.get(n) for n in field_names if kvs.get(n)]
             if not field_values:
                 return None
-        return _decode_dataclass(cls, kvs, infer_missing)
+        return super().from_dict(kvs=kvs, **kwargs)
 
 
 class CliProcessorConfig(ProcessorConfig, CliMixin):
@@ -395,6 +398,13 @@ class CliFilesStorageConfig(FileStorageConfig, CliMixin):
                 help="Recursively download files in their respective folders "
                 "otherwise stop at the files in provided folder level.",
             ),
+            click.Option(
+                ["--file-glob"],
+                default=None,
+                type=DelimitedString(),
+                help="A comma-separated list of file globs to limit which types of "
+                "local files are accepted, e.g. '*.html,*.txt'",
+            ),
         ]
         return options
 
@@ -424,16 +434,29 @@ class CliEmbeddingConfig(EmbeddingConfig, CliMixin):
                 type=str,
                 default=None,
             ),
+            click.Option(
+                ["--embedding-aws-access-key-id"],
+                help="AWS access key used for AWS-based embedders, such as bedrock",
+                type=str,
+                default=None,
+            ),
+            click.Option(
+                ["--embedding-aws-secret-access-key"],
+                help="AWS secret key used for AWS-based embedders, such as bedrock",
+                type=str,
+                default=None,
+            ),
+            click.Option(
+                ["--embedding-aws-region"],
+                help="AWS region used for AWS-based embedders, such as bedrock",
+                type=str,
+                default="us-west-2",
+            ),
         ]
         return options
 
     @classmethod
-    def from_dict(
-        cls,
-        kvs: Json,
-        *,
-        infer_missing=False,
-    ):
+    def from_dict(cls, kvs: Json, **kwargs):
         """
         Extension of the dataclass from_dict() to avoid a naming conflict with other CLI params.
         This allows CLI arguments to be prepended with embedding_ during CLI invocation but
@@ -447,10 +470,10 @@ class CliEmbeddingConfig(EmbeddingConfig, CliMixin):
             }
             if len(new_kvs.keys()) == 0:
                 return None
-            if not new_kvs.get("provider", None):
+            if not new_kvs.get("provider"):
                 return None
-            return _decode_dataclass(cls, new_kvs, infer_missing)
-        return _decode_dataclass(cls, kvs, infer_missing)
+            return super().from_dict(new_kvs, **kwargs)
+        return super().from_dict(kvs, **kwargs)
 
 
 class CliChunkingConfig(ChunkingConfig, CliMixin):
@@ -461,57 +484,117 @@ class CliChunkingConfig(ChunkingConfig, CliMixin):
                 ["--chunk-elements"],
                 is_flag=True,
                 default=False,
+                help="Deprecated, use --chunking-strategy instead.",
             ),
             click.Option(
-                ["--chunk-multipage-sections"],
-                is_flag=True,
-                default=False,
+                ["--chunking-strategy"],
+                type=str,
+                help="The rule-set to use to form chunks. Omit to disable chunking.",
             ),
             click.Option(
-                ["--chunk-combine-under-n-chars"],
+                ["--chunk-combine-text-under-n-chars"],
                 type=int,
-                default=500,
+                help=(
+                    "Combine consecutive chunks when the first does not exceed this length and"
+                    " the second will fit without exceeding the hard-maximum length. Only"
+                    " operative for 'by_title' chunking-strategy."
+                ),
+            ),
+            click.Option(
+                ["--chunk-include-orig-elements/--chunk-no-include-orig-elements"],
+                is_flag=True,
+                default=True,
+                help=(
+                    "When chunking, add the original elements consolidated to form each chunk to"
+                    " `.metadata.orig_elements` on that chunk."
+                ),
+            ),
+            click.Option(
+                ["--chunk-max-characters"],
+                type=int,
+                default=CHUNK_MAX_CHARS_DEFAULT,
                 show_default=True,
+                help=(
+                    "Hard maximum chunk length. No chunk will exceed this length. An oversized"
+                    " element will be divided by text-splitting to fit this window."
+                ),
+            ),
+            click.Option(
+                ["--chunk-multipage-sections/--chunk-no-multipage-sections"],
+                is_flag=True,
+                default=CHUNK_MULTI_PAGE_DEFAULT,
+                help=(
+                    "Ignore page boundaries when chunking such that elements from two different"
+                    " pages can appear in the same chunk. Only operative for 'by_title'"
+                    " chunking-strategy."
+                ),
             ),
             click.Option(
                 ["--chunk-new-after-n-chars"],
                 type=int,
-                default=1500,
+                help=(
+                    "Soft-maximum chunk length. Another element will not be added to a chunk of"
+                    " this length even when it would fit without exceeding the hard-maximum"
+                    " length."
+                ),
+            ),
+            click.Option(
+                ["--chunk-overlap"],
+                type=int,
+                default=0,
                 show_default=True,
+                help=(
+                    "Prefix chunk text with last overlap=N characters of prior chunk. Only"
+                    " applies to oversized chunks divided by text-splitting. To apply overlap to"
+                    " non-oversized chunks use the --overlap-all option."
+                ),
+            ),
+            click.Option(
+                ["--chunk-overlap-all"],
+                is_flag=True,
+                default=False,
+                help=(
+                    "Apply overlap to chunks formed from whole elements as well as those formed"
+                    " by text-splitting oversized elements. Overlap length is take from --overlap"
+                    " option value."
+                ),
             ),
         ]
         return options
 
     @classmethod
-    def from_dict(
-        cls,
-        kvs: Json,
-        *,
-        infer_missing=False,
-    ):
+    def from_dict(cls, kvs: Json, **kwargs: t.Any) -> t.Optional[Self]:
+        """Extension of dataclass from_dict() to avoid a naming conflict with other CLI params.
+
+        This allows CLI arguments to be prefixed with "chunking_" during CLI invocation but doesn't
+        require that as part of the field names in this class
         """
-        Extension of the dataclass from_dict() to avoid a naming conflict with other CLI params.
-        This allows CLI arguments to be prepended with chunking_ during CLI invocation but
-        doesn't require that as part of the field names in this class
-        """
-        if isinstance(kvs, dict):
-            new_kvs = {}
-            if "chunk_elements" in kvs:
-                chunk_elements = kvs.pop("chunk_elements")
-                if not chunk_elements:
-                    return None
-                new_kvs["chunk_elements"] = chunk_elements
-            new_kvs.update(
-                {
-                    k[len("chunking_") :]: v  # noqa: E203
-                    for k, v in kvs.items()
-                    if k.startswith("chunking_")
-                },
+        if not isinstance(kvs, dict):
+            return super().from_dict(kvs=kvs, **kwargs)
+
+        options: t.Dict[str, t.Any] = kvs.copy()
+        chunk_elements = options.pop("chunk_elements", None)
+        chunking_strategy = options.pop("chunking_strategy", None)
+        # -- when neither are specified, chunking is not requested --
+        if not chunk_elements and not chunking_strategy:
+            return None
+
+        def iter_kv_pairs() -> t.Iterator[t.Tuple[str, t.Any]]:
+            # -- newer `chunking_strategy` option takes precedence over legacy `chunk_elements` --
+            if chunking_strategy:
+                yield "chunking_strategy", chunking_strategy
+            # -- but legacy case is still supported, equivalent to `chunking_strategy="by_title" --
+            elif chunk_elements:
+                yield "chunking_strategy", "by_title"
+
+            yield from (
+                (key[len("chunk_") :], value)
+                for key, value in options.items()
+                if key.startswith("chunk_")
             )
-            if len(new_kvs.keys()) == 0:
-                return None
-            return _decode_dataclass(cls, new_kvs, infer_missing)
-        return _decode_dataclass(cls, kvs, infer_missing)
+
+        new_kvs = dict(iter_kv_pairs())
+        return None if len(new_kvs) == 0 else super().from_dict(kvs=new_kvs, **kwargs)
 
 
 class CliPermissionsConfig(PermissionsConfig, CliMixin):
@@ -537,12 +620,7 @@ class CliPermissionsConfig(PermissionsConfig, CliMixin):
         return options
 
     @classmethod
-    def from_dict(
-        cls,
-        kvs: Json,
-        *,
-        infer_missing=False,
-    ):
+    def from_dict(cls, kvs: Json, **kwargs):
         """
         Extension of the dataclass from_dict() to avoid a naming conflict with other CLI params.
         This allows CLI arguments to be prepended with permissions_ during CLI invocation but
@@ -574,5 +652,5 @@ class CliPermissionsConfig(PermissionsConfig, CliMixin):
             }
             if len(new_kvs.keys()) == 0:
                 return None
-            return _decode_dataclass(cls, new_kvs, infer_missing)
-        return _decode_dataclass(cls, kvs, infer_missing)
+            return super().from_dict(kvs=new_kvs, **kwargs)
+        return super().from_dict(kvs=kvs, **kwargs)
