@@ -1,5 +1,4 @@
 import math
-import os
 import typing as t
 from dataclasses import dataclass, field
 from datetime import datetime
@@ -7,8 +6,10 @@ from pathlib import Path
 
 import requests
 
+from unstructured.ingest.enhanced_dataclass import enhanced_field
 from unstructured.ingest.error import SourceConnectionError, SourceConnectionNetworkError
 from unstructured.ingest.interfaces import (
+    AccessConfig,
     BaseConnectorConfig,
     BaseSingleIngestDoc,
     BaseSourceConnector,
@@ -23,6 +24,11 @@ if t.TYPE_CHECKING:
     from atlassian import Confluence
 
 
+@dataclass
+class ConfluenceAccessConfig(AccessConfig):
+    api_token: str = enhanced_field(sensitive=True)
+
+
 @dataclass
 class SimpleConfluenceConfig(BaseConnectorConfig):
     """Connector config where:
@@ -35,10 +41,10 @@ class SimpleConfluenceConfig(BaseConnectorConfig):
     """
 
     user_email: str
-    api_token: str
+    access_config: ConfluenceAccessConfig
     url: str
-    max_number_of_spaces: int
-    max_number_of_docs_from_each_space: int
+    max_num_of_spaces: int = 500
+    max_num_of_docs_from_each_space: int = 100
     spaces: t.List[str] = field(default_factory=list)
 
 
@@ -60,7 +66,7 @@ def scroll_wrapper(func):
         del kwargs["number_of_items_to_fetch"]
 
         kwargs["limit"] = min(100, number_of_items_to_fetch)
-        kwargs["start"] = 0 if "start" not in kwargs else kwargs["start"]
+        kwargs["start"] = kwargs.get("start", 0)
 
         all_results = []
         num_iterations = math.ceil(number_of_items_to_fetch / kwargs["limit"])
@@ -126,7 +132,7 @@ class ConfluenceIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
             confluence = Confluence(
                 self.connector_config.url,
                 username=self.connector_config.user_email,
-                password=self.connector_config.api_token,
+                password=self.connector_config.access_config.api_token,
             )
             result = confluence.get_page_by_id(
                 page_id=self.document_meta.document_id,
@@ -170,8 +176,6 @@ class ConfluenceIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
     @requires_dependencies(["atlassian"], extras="confluence")
     @BaseSingleIngestDoc.skip_if_file_exists
     def get_file(self):
-        logger.debug(f"Fetching {self} - PID: {os.getpid()}")
-
         # TODO: instead of having a separate connection object for each doc,
         # have a separate connection object for each process
 
@@ -200,7 +204,7 @@ class ConfluenceSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector
             self._confluence = Confluence(
                 url=self.connector_config.url,
                 username=self.connector_config.user_email,
-                password=self.connector_config.api_token,
+                password=self.connector_config.access_config.api_token,
             )
         return self._confluence
 
@@ -218,7 +222,7 @@ class ConfluenceSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector
         self.list_of_spaces = None
         if self.connector_config.spaces:
             self.list_of_spaces = self.connector_config.spaces
-            if self.connector_config.max_number_of_spaces:
+            if self.connector_config.max_num_of_spaces:
                 logger.warning(
                     """--confluence-list-of-spaces and --confluence-num-of-spaces cannot
                     be used at the same time. Connector will only fetch the
@@ -232,7 +236,7 @@ class ConfluenceSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector
         get_spaces_with_scroll = scroll_wrapper(self.confluence.get_all_spaces)
 
         all_results = get_spaces_with_scroll(
-            number_of_items_to_fetch=self.connector_config.max_number_of_spaces,
+            number_of_items_to_fetch=self.connector_config.max_num_of_spaces,
         )
 
         space_ids = [space["key"] for space in all_results]
@@ -247,7 +251,7 @@ class ConfluenceSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector
         get_pages_with_scroll = scroll_wrapper(self.confluence.get_all_pages_from_space)
         results = get_pages_with_scroll(
             space=space_id,
-            number_of_items_to_fetch=self.connector_config.max_number_of_docs_from_each_space,
+            number_of_items_to_fetch=self.connector_config.max_num_of_docs_from_each_space,
             content_type=content_type,
         )
 
