@@ -1,4 +1,3 @@
-import json
 import os
 import typing as t
 from dataclasses import dataclass
@@ -6,8 +5,6 @@ from datetime import datetime as dt
 from multiprocessing import Process
 from pathlib import Path
 
-import pandas as pd
-
 from unstructured.ingest.error import SourceConnectionError, SourceConnectionNetworkError
 from unstructured.ingest.interfaces import (
     BaseConnectorConfig,
@@ -20,7 +17,6 @@ from unstructured.ingest.interfaces import (
     WriteConfig,
 )
 from unstructured.ingest.logger import logger
-from unstructured.ingest.utils.table import convert_to_pandas_dataframe
 from unstructured.utils import requires_dependencies
 
 if t.TYPE_CHECKING:
@@ -33,11 +29,6 @@ class SimpleDeltaTableConfig(BaseConnectorConfig):
     version: t.Optional[int] = None
     storage_options: t.Optional[t.Dict[str, str]] = None
     without_files: bool = False
-    columns: t.Optional[t.List[str]] = None
-
-    @staticmethod
-    def storage_options_from_str(options_str: str) -> t.Dict[str, str]:
-        return {s.split("=")[0].strip(): s.split("=")[1].strip() for s in options_str.split(",")}
 
 
 @dataclass
@@ -100,7 +91,6 @@ class DeltaTableIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
         self.update_source_metadata(fs=fs)
         logger.info(f"using a {fs} filesystem to collect table data")
         self._create_full_tmp_dir_path()
-        logger.debug(f"Fetching {self} - PID: {os.getpid()}")
 
         df = self._get_df(filesystem=fs)
 
@@ -108,7 +98,7 @@ class DeltaTableIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
         df.to_csv(self.filename)
 
     @SourceConnectionNetworkError.wrap
-    def _get_df(self, filesystem) -> pd.DataFrame:
+    def _get_df(self, filesystem):
         import pyarrow.parquet as pq
 
         return pq.ParquetDataset(self.uri, filesystem=filesystem).read_pandas().to_pandas()
@@ -162,8 +152,9 @@ class DeltaTableSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector
 @dataclass
 class DeltaTableWriteConfig(WriteConfig):
     drop_empty_cols: bool = False
-    overwrite_schema: bool = False
     mode: t.Literal["error", "append", "overwrite", "ignore"] = "error"
+    schema_mode: t.Optional[t.Literal["merge", "overwrite"]] = None
+    engine: t.Literal["pyarrow", "rust"] = "pyarrow"
 
 
 @dataclass
@@ -178,9 +169,12 @@ class DeltaTableDestinationConnector(BaseDestinationConnector):
     def check_connection(self):
         pass
 
+    @requires_dependencies(["deltalake"], extras="delta-table")
     def write_dict(self, *args, elements_dict: t.List[t.Dict[str, t.Any]], **kwargs) -> None:
         from deltalake.writer import write_deltalake
 
+        from unstructured.ingest.utils.table import convert_to_pandas_dataframe
+
         df = convert_to_pandas_dataframe(
             elements_dict=elements_dict,
             drop_empty_cols=self.write_config.drop_empty_cols,
@@ -189,29 +183,21 @@ class DeltaTableDestinationConnector(BaseDestinationConnector):
             f"writing {len(df)} rows to destination table "
             f"at {self.connector_config.table_uri}\ndtypes: {df.dtypes}",
         )
+        writer_kwargs = {
+            "table_or_uri": self.connector_config.table_uri,
+            "data": df,
+            "mode": self.write_config.mode,
+            "engine": self.write_config.engine,
+        }
+        if self.write_config.schema_mode is not None:
+            writer_kwargs["schema_mode"] = self.write_config.schema_mode
         # NOTE: deltalake writer on Linux sometimes can finish but still trigger a SIGABRT and cause
         # ingest to fail, even though all tasks are completed normally. Putting the writer into a
         # process mitigates this issue by ensuring python interpreter waits properly for deltalake's
         # rust backend to finish
         writer = Process(
             target=write_deltalake,
-            kwargs={
-                "table_or_uri": self.connector_config.table_uri,
-                "data": df,
-                "mode": self.write_config.mode,
-                "overwrite_schema": self.write_config.overwrite_schema,
-            },
+            kwargs=writer_kwargs,
         )
         writer.start()
         writer.join()
-
-    @requires_dependencies(["deltalake"], extras="delta-table")
-    def write(self, docs: t.List[BaseSingleIngestDoc]) -> None:
-        elements_dict: t.List[t.Dict[str, t.Any]] = []
-        for doc in docs:
-            local_path = doc._output_filename
-            with open(local_path) as json_file:
-                element_dict = json.load(json_file)
-                logger.info(f"converting {len(element_dict)} rows from content in {local_path}")
-                elements_dict.extend(element_dict)
-        self.write_dict(elements_dict=elements_dict)
