@@ -17,7 +17,7 @@ if t.TYPE_CHECKING:
 
 
 @dataclass
-class SimpleGitLabConfig(SimpleGitConfig):
+class SimpleGitlabConfig(SimpleGitConfig):
     base_url: str = "https://gitlab.com"
 
     def __post_init__(self):
@@ -34,13 +34,13 @@ class SimpleGitLabConfig(SimpleGitConfig):
     def get_project(self) -> "Project":
         from gitlab import Gitlab
 
-        gitlab = Gitlab(self.base_url, private_token=self.access_token)
+        gitlab = Gitlab(self.base_url, private_token=self.access_config.access_token)
         return gitlab.projects.get(self.repo_path)
 
 
 @dataclass
 class GitLabIngestDoc(GitIngestDoc):
-    connector_config: SimpleGitLabConfig
+    connector_config: SimpleGitlabConfig
     registry_name: str = "gitlab"
 
     @property
@@ -98,10 +98,9 @@ class GitLabIngestDoc(GitIngestDoc):
             f.write(contents)
 
 
-@requires_dependencies(["gitlab"], extras="gitlab")
 @dataclass
 class GitLabSourceConnector(GitSourceConnector):
-    connector_config: SimpleGitLabConfig
+    connector_config: SimpleGitlabConfig
 
     @requires_dependencies(["gitlab"], extras="gitlab")
     def check_connection(self):
@@ -110,7 +109,8 @@ class GitLabSourceConnector(GitSourceConnector):
 
         try:
             gitlab = Gitlab(
-                self.connector_config.base_url, private_token=self.connector_config.access_token
+                self.connector_config.base_url,
+                private_token=self.connector_config.access_config.access_token,
             )
             gitlab.auth()
         except GitlabError as gitlab_error:
