@@ -9,8 +9,10 @@ from pathlib import Path
 
 from unstructured.file_utils.filetype import EXT_TO_FILETYPE
 from unstructured.file_utils.google_filetype import GOOGLE_DRIVE_EXPORT_TYPES
+from unstructured.ingest.enhanced_dataclass import enhanced_field
 from unstructured.ingest.error import SourceConnectionError, SourceConnectionNetworkError
 from unstructured.ingest.interfaces import (
+    AccessConfig,
     BaseConnectorConfig,
     BaseSessionHandle,
     BaseSingleIngestDoc,
@@ -22,6 +24,7 @@ from unstructured.ingest.interfaces import (
     SourceMetadata,
 )
 from unstructured.ingest.logger import logger
+from unstructured.ingest.utils.string_and_date_utils import json_to_dict
 from unstructured.utils import requires_dependencies
 
 if t.TYPE_CHECKING:
@@ -45,7 +48,7 @@ def create_service_account_object(key_path: t.Union[str, dict], id=None):
     Providing a drive id enforces a key validation process.
 
     Args:
-        key_path: Path to Google Drive service account json file.
+        key_path: Path to Google Drive service account json file. (or the actual json)
         id: ID of a file on Google Drive. File has to be either publicly accessible or accessible
             to the service account.
 
@@ -57,6 +60,10 @@ def create_service_account_object(key_path: t.Union[str, dict], id=None):
     from googleapiclient.discovery import build
     from googleapiclient.errors import HttpError
 
+    # Service account key can be a dict or a file path(str)
+    # But the dict may come in as a string
+    key_path = json_to_dict(key_path)
+
     try:
         if isinstance(key_path, dict):
             creds = service_account.Credentials.from_service_account_info(key_path)
@@ -87,6 +94,11 @@ def create_service_account_object(key_path: t.Union[str, dict], id=None):
     return service
 
 
+@dataclass
+class GoogleDriveAccessConfig(AccessConfig):
+    service_account_key: t.Union[str, dict] = enhanced_field(sensitive=True)
+
+
 @dataclass
 class SimpleGoogleDriveConfig(ConfigSessionHandleMixin, BaseConnectorConfig):
     """Connector config where drive_id is the id of the document to process or
@@ -94,7 +106,7 @@ class SimpleGoogleDriveConfig(ConfigSessionHandleMixin, BaseConnectorConfig):
 
     # Google Drive Specific Options
     drive_id: str
-    service_account_key: t.Union[str, dict]
+    access_config: GoogleDriveAccessConfig
     extension: t.Optional[str] = None
     recursive: bool = False
 
@@ -108,7 +120,7 @@ class SimpleGoogleDriveConfig(ConfigSessionHandleMixin, BaseConnectorConfig):
     def create_session_handle(
         self,
     ) -> GoogleDriveSessionHandle:
-        service = create_service_account_object(self.service_account_key)
+        service = create_service_account_object(self.access_config.service_account_key)
         return GoogleDriveSessionHandle(service=service)
 
 
