@@ -1,12 +1,14 @@
-import os
 import typing as t
 from dataclasses import dataclass, field
 from pathlib import Path
+from uuid import UUID
 
 import httpx
 
+from unstructured.ingest.enhanced_dataclass import enhanced_field
 from unstructured.ingest.error import SourceConnectionError
 from unstructured.ingest.interfaces import (
+    AccessConfig,
     BaseConnectorConfig,
     BaseSingleIngestDoc,
     BaseSourceConnector,
@@ -24,15 +26,27 @@ if t.TYPE_CHECKING:
     from unstructured.ingest.connector.notion.client import Client as NotionClient
 
 
+@dataclass
+class NotionAccessConfig(AccessConfig):
+    notion_api_key: str = enhanced_field(sensitive=True)
+
+
 @dataclass
 class SimpleNotionConfig(BaseConnectorConfig):
     """Connector config to process all messages by channel id's."""
 
-    notion_api_key: str
-    page_ids: t.List[str] = field(default_factory=list)
-    database_ids: t.List[str] = field(default_factory=list)
+    access_config: NotionAccessConfig
+    page_ids: t.Optional[t.List[str]] = None
+    database_ids: t.Optional[t.List[str]] = None
     recursive: bool = False
 
+    def __post_init__(self):
+        if self.page_ids:
+            self.page_ids = [str(UUID(p.strip())) for p in self.page_ids]
+
+        if self.database_ids:
+            self.database_ids = [str(UUID(d.strip())) for d in self.database_ids]
+
 
 @dataclass
 class NotionPageIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
@@ -44,7 +58,6 @@ class NotionPageIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
     """
 
     page_id: str
-    api_key: str
     connector_config: SimpleNotionConfig
     registry_name: str = "notion_page"
     retry_strategy_config: t.Optional[RetryStrategyConfig] = None
@@ -68,7 +81,7 @@ class NotionPageIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
         # Pin the version of the api to avoid schema changes
         return NotionClient(
             notion_version=NOTION_API_VERSION,
-            auth=self.api_key,
+            auth=self.connector_config.access_config.notion_api_key,
             logger=logger,
             log_level=logger.level,
             retry_strategy_config=self.retry_strategy_config,
@@ -83,8 +96,6 @@ class NotionPageIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
 
         self._create_full_tmp_dir_path()
 
-        logger.debug(f"fetching page {self.page_id} - PID: {os.getpid()}")
-
         client = self.get_client()
 
         try:
@@ -167,7 +178,6 @@ class NotionDatabaseIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
     """
 
     database_id: str
-    api_key: str
     connector_config: SimpleNotionConfig
     retry_strategy_config: t.Optional[RetryStrategyConfig] = None
     registry_name: str = "notion_database"
@@ -191,7 +201,7 @@ class NotionDatabaseIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
         # Pin the version of the api to avoid schema changes
         return NotionClient(
             notion_version=NOTION_API_VERSION,
-            auth=self.api_key,
+            auth=self.connector_config.access_config.notion_api_key,
             logger=logger,
             log_level=logger.level,
             retry_strategy_config=self.retry_strategy_config,
@@ -206,8 +216,6 @@ class NotionDatabaseIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
 
         self._create_full_tmp_dir_path()
 
-        logger.debug(f"fetching database {self.database_id} - PID: {os.getpid()}")
-
         client = self.get_client()
 
         try:
@@ -282,7 +290,6 @@ class NotionDatabaseIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
         return self._tmp_download_file()
 
 
-@requires_dependencies(dependencies=["notion_client"], extras="notion")
 @dataclass
 class NotionSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
     """Objects of this class support fetching document(s) from"""
@@ -293,17 +300,21 @@ class NotionSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
 
     @property
     def client(self) -> "NotionClient":
+        if self._client is None:
+            self._client = self.create_client()
+        return self._client
+
+    @requires_dependencies(dependencies=["notion_client"], extras="notion")
+    def create_client(self) -> "NotionClient":
         from unstructured.ingest.connector.notion.client import Client as NotionClient
 
-        if self._client is None:
-            self._client = NotionClient(
-                notion_version=NOTION_API_VERSION,
-                auth=self.connector_config.notion_api_key,
-                logger=logger,
-                log_level=logger.level,
-                retry_strategy_config=self.retry_strategy_config,
-            )
-        return self._client
+        return NotionClient(
+            notion_version=NOTION_API_VERSION,
+            auth=self.connector_config.access_config.notion_api_key,
+            logger=logger,
+            log_level=logger.level,
+            retry_strategy_config=self.retry_strategy_config,
+        )
 
     def check_connection(self):
         try:
@@ -381,7 +392,6 @@ class NotionSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
                     retry_strategy_config=self.retry_strategy_config,
                     read_config=self.read_config,
                     page_id=page_id,
-                    api_key=self.connector_config.notion_api_key,
                 )
                 for page_id in self.connector_config.page_ids
             ]
@@ -393,7 +403,6 @@ class NotionSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
                     retry_strategy_config=self.retry_strategy_config,
                     read_config=self.read_config,
                     database_id=database_id,
-                    api_key=self.connector_config.notion_api_key,
                 )
                 for database_id in self.connector_config.database_ids
             ]
@@ -401,24 +410,28 @@ class NotionSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
             logger.info("Getting recursive content")
             child_pages = []
             child_databases = []
-            for page_id in self.connector_config.page_ids:
-                child_content = self.get_child_page_content(page_id=page_id)
-                child_pages.extend(child_content.child_pages)
-                child_databases.extend(child_content.child_databases)
+            if self.connector_config.page_ids:
+                for page_id in self.connector_config.page_ids:
+                    child_content = self.get_child_page_content(page_id=page_id)
+                    child_pages.extend(child_content.child_pages)
+                    child_databases.extend(child_content.child_databases)
 
-            for database_id in self.connector_config.database_ids:
-                child_content = self.get_child_database_content(database_id=database_id)
-                child_pages.extend(child_content.child_pages)
-                child_databases.extend(child_content.child_databases)
+            if self.connector_config.database_ids:
+                for database_id in self.connector_config.database_ids:
+                    child_content = self.get_child_database_content(database_id=database_id)
+                    child_pages.extend(child_content.child_pages)
+                    child_databases.extend(child_content.child_databases)
 
             # Remove duplicates
             child_pages = list(set(child_pages))
-            child_pages = [c for c in child_pages if c not in self.connector_config.page_ids]
+            if self.connector_config.page_ids:
+                child_pages = [c for c in child_pages if c not in self.connector_config.page_ids]
 
             child_databases = list(set(child_databases))
-            child_databases = [
-                db for db in child_databases if db not in self.connector_config.database_ids
-            ]
+            if self.connector_config.database_ids:
+                child_databases = [
+                    db for db in child_databases if db not in self.connector_config.database_ids
+                ]
 
             if child_pages:
                 logger.info(
@@ -431,7 +444,6 @@ class NotionSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
                         retry_strategy_config=self.retry_strategy_config,
                         read_config=self.read_config,
                         page_id=page_id,
-                        api_key=self.connector_config.notion_api_key,
                     )
                     for page_id in child_pages
                 ]
@@ -449,7 +461,6 @@ class NotionSourceConnector(SourceConnectorCleanupMixin, BaseSourceConnector):
                         retry_strategy_config=self.retry_strategy_config,
                         read_config=self.read_config,
                         database_id=database_id,
-                        api_key=self.connector_config.notion_api_key,
                     )
                     for database_id in child_databases
                 ]
