@@ -1,11 +1,12 @@
 import typing as t
 from dataclasses import dataclass, field
-from datetime import datetime
 from pathlib import Path
 
 from unstructured.file_utils.filetype import EXT_TO_FILETYPE
+from unstructured.ingest.enhanced_dataclass import enhanced_field
 from unstructured.ingest.error import SourceConnectionError, SourceConnectionNetworkError
 from unstructured.ingest.interfaces import (
+    AccessConfig,
     BaseConnectorConfig,
     BaseSingleIngestDoc,
     BaseSourceConnector,
@@ -14,19 +15,24 @@ from unstructured.ingest.interfaces import (
     SourceMetadata,
 )
 from unstructured.ingest.logger import logger
+from unstructured.ingest.utils.string_and_date_utils import ensure_isoformat_datetime
 from unstructured.utils import requires_dependencies
 
 if t.TYPE_CHECKING:
     from office365.graph_client import GraphClient
     from office365.onedrive.driveitems.driveItem import DriveItem
-
 MAX_MB_SIZE = 512_000_000
 
 
+@dataclass
+class OneDriveAccessConfig(AccessConfig):
+    client_credential: str = enhanced_field(repr=False, sensitive=True, overload_name="client_cred")
+
+
 @dataclass
 class SimpleOneDriveConfig(BaseConnectorConfig):
+    access_config: OneDriveAccessConfig
     client_id: str
-    client_credential: str = field(repr=False)
     user_pname: str
     tenant: str = field(repr=False)
     authority_url: t.Optional[str] = field(repr=False, default="https://login.microsoftonline.com")
@@ -34,7 +40,7 @@ class SimpleOneDriveConfig(BaseConnectorConfig):
     recursive: bool = False
 
     def __post_init__(self):
-        if not (self.client_id and self.client_credential and self.user_pname):
+        if not (self.client_id and self.access_config.client_credential and self.user_pname):
             raise ValueError(
                 "Please provide all the following mandatory values:"
                 "\n-ms-client_id\n-ms-client_cred\n-ms-user-pname",
@@ -50,7 +56,7 @@ class SimpleOneDriveConfig(BaseConnectorConfig):
             app = ConfidentialClientApplication(
                 authority=f"{self.authority_url}/{self.tenant}",
                 client_id=self.client_id,
-                client_credential=self.client_credential,
+                client_credential=self.access_config.client_credential,
             )
             token = app.acquire_token_for_client(scopes=["https://graph.microsoft.com/.default"])
         except ValueError as exc:
@@ -137,11 +143,8 @@ class OneDriveIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
             version = file.versions[n_versions - 1].properties.get("id", None)
 
         self.source_metadata = SourceMetadata(
-            date_created=datetime.strptime(file.created_datetime, "%Y-%m-%dT%H:%M:%SZ").isoformat(),
-            date_modified=datetime.strptime(
-                file.last_modified_datetime,
-                "%Y-%m-%dT%H:%M:%SZ",
-            ).isoformat(),
+            date_created=ensure_isoformat_datetime(timestamp=file.created_datetime),
+            date_modified=ensure_isoformat_datetime(timestamp=file.last_modified_datetime),
             version=version,
             source_url=file.parent_reference.path + "/" + self.file_name,
             exists=True,
