@@ -1,15 +1,16 @@
 import json
 import os
 import typing as t
-from dataclasses import dataclass, field
-from datetime import datetime
+from dataclasses import dataclass
 from html import unescape
 from pathlib import Path
 from urllib.parse import urlparse
 
 from unstructured.file_utils.filetype import EXT_TO_FILETYPE
+from unstructured.ingest.enhanced_dataclass import enhanced_field
 from unstructured.ingest.error import SourceConnectionError, SourceConnectionNetworkError
 from unstructured.ingest.interfaces import (
+    AccessConfig,
     BaseConnectorConfig,
     BaseSingleIngestDoc,
     BaseSourceConnector,
@@ -19,6 +20,7 @@ from unstructured.ingest.interfaces import (
 )
 from unstructured.ingest.interfaces import PermissionsConfig as SharepointPermissionsConfig
 from unstructured.ingest.logger import logger
+from unstructured.ingest.utils.string_and_date_utils import ensure_isoformat_datetime
 from unstructured.utils import requires_dependencies
 
 if t.TYPE_CHECKING:
@@ -30,22 +32,29 @@ MAX_MB_SIZE = 512_000_000
 CONTENT_LABELS = ["CanvasContent1", "LayoutWebpartsContent1", "TimeCreated"]
 
 
+@dataclass
+class SharepointAccessConfig(AccessConfig):
+    client_cred: str = enhanced_field(repr=False, sensitive=True)
+
+
 @dataclass
 class SimpleSharepointConfig(BaseConnectorConfig):
+    access_config: SharepointAccessConfig
     client_id: str
-    client_credential: str = field(repr=False)
-    site_url: str
+    site: str
     path: str
-    process_pages: bool = False
+    process_pages: bool = enhanced_field(default=True, init=False)
     recursive: bool = False
+    files_only: bool = False
     permissions_config: t.Optional[SharepointPermissionsConfig] = None
 
     def __post_init__(self):
-        if not (self.client_id and self.client_credential and self.site_url):
+        if not (self.client_id and self.access_config.client_cred and self.site):
             raise ValueError(
                 "Please provide one of the following mandatory values:"
                 "\n--client-id\n--client-cred\n--site",
             )
+        self.process_pages = not self.files_only
 
     @requires_dependencies(["office365"], extras="sharepoint")
     def get_site_client(self, site_url: str = "") -> "ClientContext":
@@ -53,8 +62,8 @@ class SimpleSharepointConfig(BaseConnectorConfig):
         from office365.sharepoint.client_context import ClientContext
 
         try:
-            site_client = ClientContext(site_url or self.site_url).with_credentials(
-                ClientCredential(self.client_id, self.client_credential),
+            site_client = ClientContext(site_url or self.site).with_credentials(
+                ClientCredential(self.client_id, self.access_config.client_cred),
             )
         except Exception:
             logger.error("Couldn't set Sharepoint client.")
@@ -207,9 +216,11 @@ class SharepointIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
                 version=page.get_property("Version", ""),
                 source_url=page.absolute_url,
                 exists=True,
-                permissions_data=self.update_permissions_data()
-                if self.connector_config.permissions_config
-                else None,
+                permissions_data=(
+                    self.update_permissions_data()
+                    if self.connector_config.permissions_config
+                    else None
+                ),
             )
             return
 
@@ -220,17 +231,14 @@ class SharepointIngestDoc(IngestDocCleanupMixin, BaseSingleIngestDoc):
             )
             return
         self.source_metadata = SourceMetadata(
-            date_created=datetime.strptime(file.time_created, "%Y-%m-%dT%H:%M:%SZ").isoformat(),
-            date_modified=datetime.strptime(
-                file.time_last_modified,
-                "%Y-%m-%dT%H:%M:%SZ",
-            ).isoformat(),
+            date_created=ensure_isoformat_datetime(timestamp=file.time_created),
+            date_modified=ensure_isoformat_datetime(timestamp=file.time_last_modified),
             version=file.major_version,
             source_url=file.properties.get("LinkingUrl", None),
             exists=True,
-            permissions_data=self.update_permissions_data()
-            if self.connector_config.permissions_config
-            else None,
+            permissions_data=(
+                self.update_permissions_data() if self.connector_config.permissions_config else None
+            ),
         )
 
     def _download_page(self):
@@ -438,8 +446,8 @@ class SharepointPermissionsConnector:
         if response.status_code == 200:
             return response.json()
         else:
-            print(f"Request failed with status code {response.status_code}:")
-            print(response.text)
+            logger.info(f"Request failed with status code {response.status_code}:")
+            logger.info(response.text)
 
     @requires_dependencies(["requests"], extras="sharepoint")
     def get_sites(self):
@@ -529,14 +537,14 @@ class SharepointPermissionsConnector:
         sites = [(site["id"], site["webUrl"]) for site in self.get_sites()["value"]]
         drive_ids = []
 
-        print("Obtaining drive data for sites for permissions (rbac)")
+        logger.info("Obtaining drive data for sites for permissions (rbac)")
         for site_id, site_url in sites:
             drives = self.get_drives(site_id)
             if drives:
                 drives_for_site = drives["value"]
                 drive_ids.extend([(site_id, drive["id"]) for drive in drives_for_site])
 
-        print("Obtaining item data from drives for permissions (rbac)")
+        logger.info("Obtaining item data from drives for permissions (rbac)")
         item_ids = []
         for site, drive_id in drive_ids:
             drive_items = self.get_drive_items(site, drive_id)
@@ -550,7 +558,7 @@ class SharepointPermissionsConnector:
 
         permissions_dir = Path(output_dir) / "permissions_data"
 
-        print("Writing permissions data to disk")
+        logger.info("Writing permissions data to disk")
         for site, drive_id, item_id, item_name, item_web_url in item_ids:
             res = self.get_permissions_for_drive_item(site, drive_id, item_id)
             if res:
