@@ -1,29 +1,35 @@
 """Defines Abstract Base Classes (ABC's) core to batch processing documents
 through Unstructured."""
 
+from __future__ import annotations
+
 import functools
+import json
 import os
 import re
-import typing as t
 from abc import ABC, abstractmethod
-from dataclasses import dataclass, field
+from dataclasses import InitVar, dataclass, field
 from datetime import datetime
 from pathlib import Path
+from typing import Any, Optional, Type, TypeVar
 
 from dataclasses_json import DataClassJsonMixin
-from dataclasses_json.core import Json, _asdict, _decode_dataclass
+from dataclasses_json.core import Json, _decode_dataclass
 
-from unstructured.chunking.title import chunk_by_title
 from unstructured.documents.elements import DataSourceMetadata
-from unstructured.embed import EMBEDDING_PROVIDER_TO_CLASS_MAP
 from unstructured.embed.interfaces import BaseEmbeddingEncoder, Element
+from unstructured.ingest.enhanced_dataclass import EnhancedDataClassJsonMixin, enhanced_field
+from unstructured.ingest.enhanced_dataclass.core import _asdict
 from unstructured.ingest.error import PartitionError, SourceConnectionError
 from unstructured.ingest.logger import logger
 from unstructured.partition.api import partition_via_api
-from unstructured.partition.auto import partition
-from unstructured.staging.base import convert_to_dict, flatten_dict
+from unstructured.staging.base import elements_to_dicts, flatten_dict
 
-A = t.TypeVar("A", bound="DataClassJsonMixin")
+A = TypeVar("A", bound="DataClassJsonMixin")
+
+# -- Needed to resolve TypeError raised by using InitVar and __future__.annotations
+# -- See more here: https://stackoverflow.com/questions/70400639/
+InitVar.__call__ = lambda *args: None  # type: ignore
 
 SUPPORTED_REMOTE_FSSPEC_PROTOCOLS = [
     "s3",
@@ -34,6 +40,7 @@ SUPPORTED_REMOTE_FSSPEC_PROTOCOLS = [
     "gcs",
     "box",
     "dropbox",
+    "sftp",
 ]
 
 
@@ -43,10 +50,17 @@ class BaseSessionHandle(ABC):
     e.g., a connection for making a request for fetching documents."""
 
 
-class BaseConfig(DataClassJsonMixin, ABC):
+@dataclass
+class BaseConfig(EnhancedDataClassJsonMixin, ABC):
     pass
 
 
+@dataclass
+class AccessConfig(BaseConfig):
+    """Meant to designate holding any sensitive information associated with other configs
+    and also for access specific configs."""
+
+
 @dataclass
 class RetryStrategyConfig(BaseConfig):
     """
@@ -65,8 +79,8 @@ class RetryStrategyConfig(BaseConfig):
             evaluated at runtime and its return value used.
     """
 
-    max_retries: t.Optional[int] = None
-    max_retry_time: t.Optional[float] = None
+    max_retries: Optional[int] = None
+    max_retry_time: Optional[float] = None
 
 
 @dataclass
@@ -74,20 +88,20 @@ class PartitionConfig(BaseConfig):
     # where to write structured data outputs
     pdf_infer_table_structure: bool = False
     strategy: str = "auto"
-    ocr_languages: t.Optional[t.List[str]] = None
-    encoding: t.Optional[str] = None
-    additional_partition_args: dict = field(default_factory=dict)
-    skip_infer_table_types: t.Optional[t.List[str]] = None
-    fields_include: t.List[str] = field(
+    ocr_languages: Optional[list[str]] = None
+    encoding: Optional[str] = None
+    additional_partition_args: dict[str, Any] = field(default_factory=dict)
+    skip_infer_table_types: Optional[list[str]] = None
+    fields_include: list[str] = field(
         default_factory=lambda: ["element_id", "text", "type", "metadata", "embeddings"],
     )
     flatten_metadata: bool = False
-    metadata_exclude: t.List[str] = field(default_factory=list)
-    metadata_include: t.List[str] = field(default_factory=list)
-    partition_endpoint: t.Optional[str] = "https://api.unstructured.io/general/v0/general"
+    metadata_exclude: list[str] = field(default_factory=list)
+    metadata_include: list[str] = field(default_factory=list)
+    partition_endpoint: Optional[str] = "https://api.unstructured.io/general/v0/general"
     partition_by_api: bool = False
-    api_key: t.Optional[str] = None
-    hi_res_model_name: t.Optional[str] = None
+    api_key: Optional[str] = str(enhanced_field(default=None, sensitive=True)) or None
+    hi_res_model_name: Optional[str] = None
 
 
 @dataclass
@@ -105,18 +119,22 @@ class FileStorageConfig(BaseConfig):
     remote_url: str
     uncompress: bool = False
     recursive: bool = False
+    file_glob: Optional[list[str]] = None
 
 
 @dataclass
 class FsspecConfig(FileStorageConfig):
-    access_kwargs: dict = field(default_factory=dict)
+    access_config: Optional[AccessConfig] = None
     protocol: str = field(init=False)
     path_without_protocol: str = field(init=False)
     dir_path: str = field(init=False)
     file_path: str = field(init=False)
 
-    def get_access_kwargs(self) -> dict:
-        return self.access_kwargs
+    def get_access_config(self) -> dict[str, Any]:
+        if self.access_config:
+            return self.access_config.to_dict(apply_name_overload=False)
+        else:
+            return {}
 
     def __post_init__(self):
         self.protocol, self.path_without_protocol = self.remote_url.split("://")
@@ -161,54 +179,105 @@ class FsspecConfig(FileStorageConfig):
 @dataclass
 class ReadConfig(BaseConfig):
     # where raw documents are stored for processing, and then removed if not preserve_downloads
-    download_dir: str = ""
+    download_dir: Optional[str] = ""
     re_download: bool = False
     preserve_downloads: bool = False
     download_only: bool = False
-    max_docs: t.Optional[int] = None
+    max_docs: Optional[int] = None
 
 
 @dataclass
 class EmbeddingConfig(BaseConfig):
     provider: str
-    api_key: t.Optional[str] = None
-    model_name: t.Optional[str] = None
+    api_key: Optional[str] = str(enhanced_field(default=None, sensitive=True)) or None
+    model_name: Optional[str] = None
+    aws_access_key_id: Optional[str] = None
+    aws_secret_access_key: Optional[str] = None
+    aws_region: Optional[str] = None
 
     def get_embedder(self) -> BaseEmbeddingEncoder:
-        kwargs = {}
+        kwargs: dict[str, Any] = {}
         if self.api_key:
             kwargs["api_key"] = self.api_key
         if self.model_name:
             kwargs["model_name"] = self.model_name
+        # TODO make this more dynamic to map to encoder configs
+        if self.provider == "langchain-openai":
+            from unstructured.embed.openai import OpenAIEmbeddingConfig, OpenAIEmbeddingEncoder
 
-        cls = EMBEDDING_PROVIDER_TO_CLASS_MAP[self.provider]
-        return cls(**kwargs)
+            return OpenAIEmbeddingEncoder(config=OpenAIEmbeddingConfig(**kwargs))
+        elif self.provider == "langchain-huggingface":
+            from unstructured.embed.huggingface import (
+                HuggingFaceEmbeddingConfig,
+                HuggingFaceEmbeddingEncoder,
+            )
+
+            return HuggingFaceEmbeddingEncoder(config=HuggingFaceEmbeddingConfig(**kwargs))
+        elif self.provider == "octoai":
+            from unstructured.embed.octoai import OctoAiEmbeddingConfig, OctoAIEmbeddingEncoder
+
+            return OctoAIEmbeddingEncoder(config=OctoAiEmbeddingConfig(**kwargs))
+        elif self.provider == "langchain-aws-bedrock":
+            from unstructured.embed.bedrock import BedrockEmbeddingConfig, BedrockEmbeddingEncoder
+
+            return BedrockEmbeddingEncoder(
+                config=BedrockEmbeddingConfig(
+                    aws_access_key_id=self.aws_access_key_id,
+                    aws_secret_access_key=self.aws_secret_access_key,
+                    region_name=self.aws_region,
+                )
+            )
+        elif self.provider == "langchain-vertexai":
+            from unstructured.embed.vertexai import (
+                VertexAIEmbeddingConfig,
+                VertexAIEmbeddingEncoder,
+            )
+
+            return VertexAIEmbeddingEncoder(config=VertexAIEmbeddingConfig(**kwargs))
+        elif self.provider == "langchain-voyageai":
+            from unstructured.embed.voyageai import (
+                VoyageAIEmbeddingConfig,
+                VoyageAIEmbeddingEncoder,
+            )
+
+            return VoyageAIEmbeddingEncoder(config=VoyageAIEmbeddingConfig(**kwargs))
+        else:
+            raise ValueError(f"{self.provider} not a recognized encoder")
 
 
 @dataclass
 class ChunkingConfig(BaseConfig):
-    chunk_elements: bool = False
-    multipage_sections: bool = True
-    combine_text_under_n_chars: int = 500
-    max_characters: int = 1500
+    chunk_elements: InitVar[bool] = False
+    chunking_strategy: Optional[str] = None
+    combine_text_under_n_chars: Optional[int] = None
+    include_orig_elements: Optional[bool] = None
+    max_characters: Optional[int] = None
+    multipage_sections: Optional[bool] = None
+    new_after_n_chars: Optional[int] = None
+    overlap: Optional[int] = None
+    overlap_all: Optional[bool] = None
 
-    def chunk(self, elements: t.List[Element]) -> t.List[Element]:
-        if self.chunk_elements:
-            return chunk_by_title(
-                elements=elements,
-                multipage_sections=self.multipage_sections,
-                combine_text_under_n_chars=self.combine_text_under_n_chars,
-                max_characters=self.max_characters,
-            )
-        else:
-            return elements
+    def __post_init__(self, chunk_elements: bool) -> None:
+        """Resolve chunking_strategy if chunk_elements is True.
+
+        If chunk_elements is True and chunking_strategy is None, default to 'by_title'. Otherwise,
+        do nothing and keep the defined value of chunking_strategy."
+        """
+        if chunk_elements and self.chunking_strategy is None:
+            self.chunking_strategy = "by_title"
 
 
 @dataclass
 class PermissionsConfig(BaseConfig):
-    application_id: t.Optional[str]
-    client_cred: t.Optional[str]
-    tenant: t.Optional[str]
+    application_id: Optional[str] = enhanced_field(overload_name="permissions_application_id")
+    tenant: Optional[str] = enhanced_field(overload_name="permissions_tenant")
+    client_cred: Optional[str] = enhanced_field(
+        default=None, sensitive=True, overload_name="permissions_client_cred"
+    )
+
+
+# module-level variable to store session handle
+global_write_session_handle: Optional[BaseSessionHandle] = None
 
 
 @dataclass
@@ -216,21 +285,22 @@ class WriteConfig(BaseConfig):
     pass
 
 
-class BaseConnectorConfig(ABC):
+@dataclass
+class BaseConnectorConfig(BaseConfig, ABC):
     """Abstract definition on which to define connector-specific attributes."""
 
 
 @dataclass
-class SourceMetadata(DataClassJsonMixin, ABC):
-    date_created: t.Optional[str] = None
-    date_modified: t.Optional[str] = None
-    version: t.Optional[str] = None
-    source_url: t.Optional[str] = None
-    exists: t.Optional[bool] = None
-    permissions_data: t.Optional[t.List[t.Dict[str, t.Any]]] = None
+class SourceMetadata(EnhancedDataClassJsonMixin, ABC):
+    date_created: Optional[str] = None
+    date_modified: Optional[str] = None
+    version: Optional[str] = None
+    source_url: Optional[str] = None
+    exists: Optional[bool] = None
+    permissions_data: Optional[list[dict[str, Any]]] = None
 
 
-class IngestDocJsonMixin(DataClassJsonMixin):
+class IngestDocJsonMixin(EnhancedDataClassJsonMixin):
     """
     Inherently, DataClassJsonMixin does not add in any @property fields to the json/dict
     created from the dataclass. This explicitly sets properties to look for on the IngestDoc
@@ -255,7 +325,7 @@ class IngestDocJsonMixin(DataClassJsonMixin):
         "unique_id",
     ]
 
-    def add_props(self, as_dict: dict, props: t.List[str]):
+    def add_props(self, as_dict: dict[str, Any], props: list[str]):
         for prop in props:
             val = getattr(self, prop)
             if isinstance(val, Path):
@@ -264,16 +334,22 @@ class IngestDocJsonMixin(DataClassJsonMixin):
                 val = val.to_dict(encode_json=False)
             as_dict[prop] = val
 
-    def to_dict(self, encode_json=False) -> t.Dict[str, Json]:
-        as_dict = _asdict(self, encode_json=encode_json)
+    def to_dict(self, **kwargs) -> dict[str, Json]:
+        as_dict = _asdict(self, **kwargs)
+        if "_session_handle" in as_dict:
+            as_dict.pop("_session_handle", None)
         self.add_props(as_dict=as_dict, props=self.properties_to_serialize)
         if getattr(self, "_source_metadata") is not None:
             self.add_props(as_dict=as_dict, props=self.metadata_properties)
         return as_dict
 
     @classmethod
-    def from_dict(cls: t.Type[A], kvs: Json, *, infer_missing=False) -> A:
-        doc = _decode_dataclass(cls, kvs, infer_missing)
+    def from_dict(
+        cls: Type[A], kvs: Json, *, infer_missing=False, apply_name_overload: bool = True
+    ) -> A:
+        doc = super().from_dict(
+            kvs=kvs, infer_missing=infer_missing, apply_name_overload=apply_name_overload
+        )
         if meta := kvs.get("_source_metadata"):
             setattr(doc, "_source_metadata", SourceMetadata.from_dict(meta))
         if date_processed := kvs.get("_date_processed"):
@@ -281,7 +357,7 @@ class IngestDocJsonMixin(DataClassJsonMixin):
         return doc
 
 
-class BatchIngestDocJsonMixin(DataClassJsonMixin):
+class BatchIngestDocJsonMixin(EnhancedDataClassJsonMixin):
     """
     Inherently, DataClassJsonMixin does not add in any @property fields to the json/dict
     created from the dataclass. This explicitly sets properties to look for on the IngestDoc
@@ -290,7 +366,7 @@ class BatchIngestDocJsonMixin(DataClassJsonMixin):
 
     properties_to_serialize = ["unique_id"]
 
-    def add_props(self, as_dict: dict, props: t.List[str]):
+    def add_props(self, as_dict: dict[str, Any], props: list[str]):
         for prop in props:
             val = getattr(self, prop)
             if isinstance(val, Path):
@@ -299,13 +375,13 @@ class BatchIngestDocJsonMixin(DataClassJsonMixin):
                 val = val.to_dict(encode_json=False)
             as_dict[prop] = val
 
-    def to_dict(self, encode_json=False) -> t.Dict[str, Json]:
+    def to_dict(self, encode_json=False) -> dict[str, Json]:
         as_dict = _asdict(self, encode_json=encode_json)
         self.add_props(as_dict=as_dict, props=self.properties_to_serialize)
         return as_dict
 
     @classmethod
-    def from_dict(cls: t.Type[A], kvs: Json, *, infer_missing=False) -> A:
+    def from_dict(cls: Type[A], kvs: Json, *, infer_missing=False) -> A:
         doc = _decode_dataclass(cls, kvs, infer_missing)
         return doc
 
@@ -332,8 +408,8 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
     Crucially, it is not responsible for the actual processing of the raw document.
     """
 
-    _source_metadata: t.Optional[SourceMetadata] = field(init=False, default=None)
-    _date_processed: t.Optional[str] = field(init=False, default=None)
+    _source_metadata: Optional[SourceMetadata] = field(init=False, default=None)
+    _date_processed: Optional[str] = field(init=False, default=None)
 
     @property
     def source_metadata(self) -> SourceMetadata:
@@ -349,25 +425,25 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         self._source_metadata = value
 
     @property
-    def date_created(self) -> t.Optional[str]:
+    def date_created(self) -> Optional[str]:
         """The date the document was created on the source system."""
-        return self.source_metadata.date_created  # type: ignore
+        return self.source_metadata.date_created
 
     @property
-    def date_modified(self) -> t.Optional[str]:
+    def date_modified(self) -> Optional[str]:
         """The date the document was last modified on the source system."""
-        return self.source_metadata.date_modified  # type: ignore
+        return self.source_metadata.date_modified
 
     @property
-    def date_processed(self) -> t.Optional[str]:
+    def date_processed(self) -> Optional[str]:
         """The date the document was last processed by Unstructured.
         self._date_processed is assigned internally in self.partition_file()"""
-        return self._date_processed  # type: ignore
+        return self._date_processed
 
     @property
-    def exists(self) -> t.Optional[bool]:
+    def exists(self) -> Optional[bool]:
         """Whether the document exists on the remote source."""
-        return self.source_metadata.exists  # type: ignore
+        return self.source_metadata.exists
 
     @property
     @abstractmethod
@@ -375,7 +451,7 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         """The local filename of the document after fetching from remote source."""
 
     @property
-    def base_filename(self) -> t.Optional[str]:
+    def base_filename(self) -> Optional[str]:
         if self.read_config.download_dir and self.filename:
             download_path = str(Path(self.read_config.download_dir).resolve())
             full_path = str(self.filename)
@@ -384,7 +460,7 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         return None
 
     @property
-    def base_output_filename(self) -> t.Optional[str]:
+    def base_output_filename(self) -> Optional[str]:
         if self.processor_config.output_dir and self._output_filename:
             output_path = str(Path(self.processor_config.output_dir).resolve())
             full_path = str(self._output_filename)
@@ -398,7 +474,7 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         """Filename of the structured output for this doc."""
 
     @property
-    def record_locator(self) -> t.Optional[t.Dict[str, t.Any]]:  # Values must be JSON-serializable
+    def record_locator(self) -> Optional[dict[str, Any]]:  # Values must be JSON-serializable
         """A dictionary with any data necessary to uniquely identify the document on
         the source system."""
         return None
@@ -408,19 +484,19 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         return self.filename
 
     @property
-    def source_url(self) -> t.Optional[str]:
+    def source_url(self) -> Optional[str]:
         """The url of the source document."""
         return self.source_metadata.source_url  # type: ignore
 
     @property
-    def version(self) -> t.Optional[str]:
+    def version(self) -> Optional[str]:
         """The version of the source document, this could be the last modified date, an
         explicit version number, or anything else that can be used to uniquely identify
         the version of the document."""
         return self.source_metadata.version  # type: ignore
 
     @property
-    def permissions_data(self) -> t.Optional[t.List[t.Dict[str, t.Any]]]:
+    def permissions_data(self) -> Optional[list[dict[str, Any]]]:
         """Access control data, aka permissions or sharing, from the source system."""
         if self.source_metadata is None:
             self.update_source_metadata()
@@ -457,7 +533,7 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         """Sets the _permissions_data property for the doc.
         This property is later used to fill the corresponding SourceMetadata.permissions_data field,
         and after that carries on to the permissions_data property."""
-        self._permissions_data: t.Optional[t.List[t.Dict]] = None
+        self._permissions_data: Optional[list[dict[str, Any]]] = None
 
     # NOTE(crag): Future BaseIngestDoc classes could define get_file_object() methods
     # in addition to or instead of get_file()
@@ -475,7 +551,9 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         self,
         partition_config: PartitionConfig,
         **partition_kwargs,
-    ) -> t.List[Element]:
+    ) -> list[Element]:
+        from unstructured.partition.auto import partition
+
         if not partition_config.partition_by_api:
             logger.debug("Using local partition")
             elements = partition(
@@ -496,14 +574,11 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
 
             logger.debug(f"Using remote partition ({endpoint})")
 
-            passthrough_partition_kwargs = {
-                k: str(v) for k, v in partition_kwargs.items() if v is not None
-            }
             elements = partition_via_api(
                 filename=str(self.filename),
                 api_key=partition_config.api_key,
                 api_url=endpoint,
-                **passthrough_partition_kwargs,
+                **partition_kwargs,
             )
             # TODO: add m_data_source_metadata to unstructured-api pipeline_api and then
             # pass the stringified json here
@@ -513,18 +588,17 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
         self,
         partition_config: PartitionConfig,
         **partition_kwargs,
-    ) -> t.Optional[t.List[t.Dict[str, t.Any]]]:
+    ) -> Optional[list[dict[str, Any]]]:
         self._date_processed = datetime.utcnow().isoformat()
         if self.read_config.download_only:
             return None
         logger.info(f"Processing {self.filename}")
 
-        isd_elems_raw = self.partition_file(partition_config=partition_config, **partition_kwargs)
-        isd_elems = convert_to_dict(isd_elems_raw)
+        elements = self.partition_file(partition_config=partition_config, **partition_kwargs)
+        element_dicts = elements_to_dicts(elements)
 
-        self.isd_elems_no_filename: t.List[t.Dict[str, t.Any]] = []
-        for elem in isd_elems:
-            # type: ignore
+        self.isd_elems_no_filename: list[dict[str, Any]] = []
+        for elem in element_dicts:
             if partition_config.metadata_exclude and partition_config.metadata_include:
                 raise ValueError(
                     "Arguments `--metadata-include` and `--metadata-exclude` are "
@@ -563,7 +637,7 @@ class BaseSingleIngestDoc(BaseIngestDoc, IngestDocJsonMixin, ABC):
 
 @dataclass
 class BaseIngestDocBatch(BaseIngestDoc, BatchIngestDocJsonMixin, ABC):
-    ingest_docs: t.List[BaseSingleIngestDoc] = field(default_factory=list)
+    ingest_docs: list[BaseSingleIngestDoc] = field(default_factory=list)
 
     @abstractmethod
     @SourceConnectionError.wrap
@@ -572,7 +646,7 @@ class BaseIngestDocBatch(BaseIngestDoc, BatchIngestDocJsonMixin, ABC):
 
 
 @dataclass
-class BaseConnector(DataClassJsonMixin, ABC):
+class BaseConnector(EnhancedDataClassJsonMixin, ABC):
     @abstractmethod
     def check_connection(self):
         pass
@@ -615,22 +689,56 @@ class BaseDestinationConnector(BaseConnector, ABC):
         self.write_config = write_config
         self.connector_config = connector_config
 
+    def conform_dict(self, data: dict[str, Any]) -> None:
+        """
+        When the original dictionary needs to be modified in place
+        """
+        return
+
+    def normalize_dict(self, element_dict: dict[str, Any]) -> dict[str, Any]:
+        """
+        When the original dictionary needs to be mapped to a new one
+        """
+        return element_dict
+
     @abstractmethod
     def initialize(self):
         """Initializes the connector. Should also validate the connector is properly
         configured."""
 
-    @abstractmethod
-    def write(self, docs: t.List[BaseSingleIngestDoc]) -> None:
-        pass
+    def write(self, docs: list[BaseSingleIngestDoc]) -> None:
+        elements_dict = self.get_elements_dict(docs=docs)
+        self.modify_and_write_dict(elements_dict=elements_dict)
+
+    def get_elements_dict(self, docs: list[BaseSingleIngestDoc]) -> list[dict[str, Any]]:
+        dict_list: list[dict[str, Any]] = []
+        for doc in docs:
+            local_path = doc._output_filename
+            with open(local_path) as json_file:
+                dict_content = json.load(json_file)
+                logger.info(
+                    f"Extending {len(dict_content)} json elements from content in {local_path}",
+                )
+                dict_list.extend(dict_content)
+        return dict_list
 
     @abstractmethod
-    def write_dict(self, *args, elements_dict: t.List[t.Dict[str, t.Any]], **kwargs) -> None:
+    def write_dict(self, *args, elements_dict: list[dict[str, Any]], **kwargs) -> None:
         pass
 
-    def write_elements(self, elements: t.List[Element], *args, **kwargs) -> None:
+    def modify_and_write_dict(self, *args, elements_dict: list[dict[str, Any]], **kwargs) -> None:
+        """
+        Modify in this instance means this method wraps calls to conform_dict() and
+        normalize() before actually processing the content via write_dict()
+        """
+        for d in elements_dict:
+            self.conform_dict(data=d)
+        elements_dict_normalized = [self.normalize_dict(element_dict=d) for d in elements_dict]
+        return self.write_dict(*args, elements_dict=elements_dict_normalized, **kwargs)
+
+    def write_elements(self, elements: list[Element], *args, **kwargs) -> None:
         elements_dict = [e.to_dict() for e in elements]
-        self.write_dict(*args, elements_dict=elements_dict, **kwargs)
+        self.modify_and_write_dict(*args, elements_dict=elements_dict, **kwargs)
 
 
 class SourceConnectorCleanupMixin:
@@ -670,7 +778,7 @@ class PermissionsCleanupMixin:
         """Recursively clean up downloaded files and directories."""
         if cur_dir is None:
             cur_dir = Path(self.processor_config.output_dir, "permissions_data")
-        if cur_dir is None:
+        if not Path(cur_dir).exists():
             return
         if Path(cur_dir).is_file():
             cur_file = cur_dir
@@ -716,7 +824,7 @@ class ConfigSessionHandleMixin:
 @dataclass
 class IngestDocSessionHandleMixin:
     connector_config: ConfigSessionHandleMixin
-    _session_handle: t.Optional[BaseSessionHandle] = field(default=None, init=False)
+    _session_handle: Optional[BaseSessionHandle] = field(default=None, init=False)
 
     @property
     def session_handle(self):
