@@ -1,16 +1,16 @@
+from __future__ import annotations
+
 import contextlib
-from typing import (
-    IO,
-    List,
-    Optional,
-)
+from typing import IO, Any, Optional, Sequence
 
 import requests
+from unstructured_client import UnstructuredClient
+from unstructured_client.models import shared
 
 from unstructured.documents.elements import Element
 from unstructured.logger import logger
 from unstructured.partition.common import exactly_one
-from unstructured.staging.base import dict_to_elements, elements_from_json
+from unstructured.staging.base import elements_from_dicts, elements_from_json
 
 
 def partition_via_api(
@@ -21,8 +21,8 @@ def partition_via_api(
     api_url: str = "https://api.unstructured.io/general/v0/general",
     api_key: str = "",
     metadata_filename: Optional[str] = None,
-    **request_kwargs,
-) -> List[Element]:
+    **request_kwargs: Any,
+) -> list[Element]:
     """Partitions a document using the Unstructured REST API. This is equivalent to
     running the document through partition.
 
@@ -63,40 +63,31 @@ def partition_via_api(
             "Please use metadata_filename instead.",
         )
 
-    headers = {
-        "ACCEPT": "application/json",
-        "UNSTRUCTURED-API-KEY": api_key,
-    }
+    # Note(austin) - the sdk takes the base url, but we have the full api_url
+    # For consistency, just strip off the path when it's given
+    base_url = api_url[:-19] if "/general/v0/general" in api_url else api_url
+    sdk = UnstructuredClient(api_key_auth=api_key, server_url=base_url)
 
     if filename is not None:
         with open(filename, "rb") as f:
-            files = [
-                ("files", (filename, f, content_type)),
-            ]
-            response = requests.post(
-                api_url,
-                headers=headers,
-                data=request_kwargs,
-                files=files,  # type: ignore
+            files = shared.Files(
+                content=f.read(),
+                file_name=filename,
             )
+
     elif file is not None:
         if metadata_filename is None:
             raise ValueError(
                 "If file is specified in partition_via_api, "
                 "metadata_filename must be specified as well.",
             )
-        files = [
-            ("files", (metadata_filename, file, content_type)),  # type: ignore
-        ]
-        response = requests.post(
-            api_url,
-            headers=headers,
-            data=request_kwargs,
-            files=files,  # type: ignore
-        )
+        files = shared.Files(content=file, file_name=metadata_filename)
+
+    req = shared.PartitionParameters(files=files, **request_kwargs)
+    response = sdk.general.partition(req)
 
     if response.status_code == 200:
-        return elements_from_json(text=response.text)
+        return elements_from_json(text=response.raw_response.text)
     else:
         raise ValueError(
             f"Receive unexpected status code {response.status_code} from the API.",
@@ -104,16 +95,16 @@ def partition_via_api(
 
 
 def partition_multiple_via_api(
-    filenames: Optional[List[str]] = None,
-    content_types: Optional[List[str]] = None,
-    files: Optional[List[str]] = None,
-    file_filenames: Optional[List[str]] = None,
+    filenames: Optional[list[str]] = None,
+    content_types: Optional[list[str]] = None,
+    files: Optional[Sequence[IO[bytes]]] = None,
+    file_filenames: Optional[list[str]] = None,
     api_url: str = "https://api.unstructured.io/general/v0/general",
     api_key: str = "",
-    metadata_filenames: Optional[List[str]] = None,
-    **request_kwargs,
-) -> List[List[Element]]:
-    """Partitions multiple document using the Unstructured REST API by batching
+    metadata_filenames: Optional[list[str]] = None,
+    **request_kwargs: Any,
+) -> list[list[Element]]:
+    """Partitions multiple documents using the Unstructured REST API by batching
     the documents into a single HTTP request.
 
     See https://api.unstructured.io/general/docs for the hosted API documentation or
@@ -207,7 +198,7 @@ def partition_multiple_via_api(
             response_list = [response_list]
 
         for document in response_list:
-            documents.append(dict_to_elements(document))
+            documents.append(elements_from_dicts(document))
         return documents
     else:
         raise ValueError(
