@@ -1,9 +1,13 @@
+"""Provides partitioning with automatic file-type detection."""
+
+from __future__ import annotations
+
 import io
-from typing import IO, Callable, Dict, List, Optional, Tuple
+from typing import IO, Any, Callable, Literal, Optional
 
 import requests
 
-from unstructured.documents.elements import DataSourceMetadata
+from unstructured.documents.elements import DataSourceMetadata, Element
 from unstructured.file_utils.filetype import (
     FILETYPE_TO_MIMETYPE,
     STR_TO_FILETYPE,
@@ -16,15 +20,13 @@ from unstructured.partition.common import exactly_one
 from unstructured.partition.email import partition_email
 from unstructured.partition.html import partition_html
 from unstructured.partition.json import partition_json
-from unstructured.partition.lang import (
-    convert_old_ocr_languages_to_languages,
-)
+from unstructured.partition.lang import check_language_args
 from unstructured.partition.text import partition_text
 from unstructured.partition.utils.constants import PartitionStrategy
 from unstructured.partition.xml import partition_xml
 from unstructured.utils import dependency_exists
 
-PARTITION_WITH_EXTRAS_MAP: Dict[str, Callable] = {}
+PARTITION_WITH_EXTRAS_MAP: dict[str, Callable[..., list[Element]]] = {}
 
 if dependency_exists("pandas"):
     from unstructured.partition.csv import partition_csv
@@ -70,7 +72,7 @@ if dependency_exists("markdown"):
     PARTITION_WITH_EXTRAS_MAP["md"] = partition_md
 
 
-if dependency_exists("msg_parser"):
+if dependency_exists("oxmsg"):
     from unstructured.partition.msg import partition_msg
 
     PARTITION_WITH_EXTRAS_MAP["msg"] = partition_msg
@@ -103,9 +105,18 @@ if dependency_exists("pandas") and dependency_exists("openpyxl"):
     PARTITION_WITH_EXTRAS_MAP["xlsx"] = partition_xlsx
 
 
+IMAGE_FILETYPES = [
+    FileType.HEIC,
+    FileType.PNG,
+    FileType.JPG,
+    FileType.TIFF,
+    FileType.BMP,
+]
+
+
 def _get_partition_with_extras(
     doc_type: str,
-    partition_with_extras_map: Optional[Dict[str, Callable]] = None,
+    partition_with_extras_map: Optional[dict[str, Callable[..., list[Element]]]] = None,
 ):
     if partition_with_extras_map is None:
         partition_with_extras_map = PARTITION_WITH_EXTRAS_MAP
@@ -128,21 +139,27 @@ def partition(
     include_page_breaks: bool = False,
     strategy: str = PartitionStrategy.AUTO,
     encoding: Optional[str] = None,
-    paragraph_grouper: Optional[Callable[[str], str]] = None,
-    headers: Dict[str, str] = {},
-    skip_infer_table_types: List[str] = ["pdf", "jpg", "png", "xls", "xlsx"],
+    paragraph_grouper: Optional[Callable[[str], str]] | Literal[False] = None,
+    headers: dict[str, str] = {},
+    skip_infer_table_types: list[str] = ["pdf", "jpg", "png", "heic"],
     ssl_verify: bool = True,
     ocr_languages: Optional[str] = None,  # changing to optional for deprecation
-    languages: Optional[List[str]] = None,
+    languages: Optional[list[str]] = None,
     detect_language_per_element: bool = False,
     pdf_infer_table_structure: bool = False,
-    pdf_extract_images: bool = False,
-    pdf_image_output_dir_path: Optional[str] = None,
+    extract_images_in_pdf: bool = False,
+    extract_image_block_types: Optional[list[str]] = None,
+    extract_image_block_output_dir: Optional[str] = None,
+    extract_image_block_to_payload: bool = False,
     xml_keep_tags: bool = False,
     data_source_metadata: Optional[DataSourceMetadata] = None,
     metadata_filename: Optional[str] = None,
     request_timeout: Optional[int] = None,
-    **kwargs,
+    hi_res_model_name: Optional[str] = None,
+    model_name: Optional[str] = None,  # to be deprecated
+    date_from_file_object: bool = False,
+    starting_page_number: int = 1,
+    **kwargs: Any,
 ):
     """Partitions a document into its constituent elements. Will use libmagic to determine
     the file's type and route it to the appropriate partitioning function. Applies the default
@@ -186,22 +203,53 @@ def partition(
             detect_language_per_element
                 Detect language per element instead of at the document level.
     pdf_infer_table_structure
+        Deprecated! Use `skip_infer_table_types` to opt out of table extraction for any document
+        type.
         If True and strategy=hi_res, any Table Elements extracted from a PDF will include an
         additional metadata field, "text_as_html," where the value (string) is a just a
         transformation of the data into an HTML <table>.
         The "text" field for a partitioned Table Element is always present, whether True or False.
-    pdf_extract_images
-        If True and strategy=hi_res, any detected images will be saved in the path specified by
-        pdf_image_output_dir_path.
-    pdf_image_output_dir_path
-        If pdf_extract_images=True and strategy=hi_res, any detected images will be saved in the
-        given path
+    extract_images_in_pdf
+        Only applicable if `strategy=hi_res`.
+        If True, any detected images will be saved in the path specified by
+        'extract_image_block_output_dir' or stored as base64 encoded data within metadata fields.
+        Deprecation Note: This parameter is marked for deprecation. Future versions will use
+        'extract_image_block_types' for broader extraction capabilities.
+    extract_image_block_types
+        Only applicable if `strategy=hi_res`.
+        Images of the element type(s) specified in this list (e.g., ["Image", "Table"]) will be
+        saved in the path specified by 'extract_image_block_output_dir' or stored as base64
+        encoded data within metadata fields.
+    extract_image_block_to_payload
+        Only applicable if `strategy=hi_res`.
+        If True, images of the element type(s) defined in 'extract_image_block_types' will be
+        encoded as base64 data and stored in two metadata fields: 'image_base64' and
+        'image_mime_type'.
+        This parameter facilitates the inclusion of element data directly within the payload,
+        especially for web-based applications or APIs.
+    extract_image_block_output_dir
+        Only applicable if `strategy=hi_res` and `extract_image_block_to_payload=False`.
+        The filesystem path for saving images of the element type(s)
+        specified in 'extract_image_block_types'.
     xml_keep_tags
         If True, will retain the XML tags in the output. Otherwise it will simply extract
         the text from within the tags. Only applies to partition_xml.
     request_timeout
         The timeout for the HTTP request if URL is set. Defaults to None meaning no timeout and
         requests will block indefinitely.
+    hi_res_model_name
+        The layout detection model used when partitioning strategy is set to `hi_res`.
+    model_name
+        The layout detection model used when partitioning strategy is set to `hi_res`. To be
+        deprecated in favor of `hi_res_model_name`.
+    date_from_file_object
+        Applies only when providing file via `file` parameter. If this option is True and inference
+        from message header failed, attempt to infer last_modified metadata from bytes,
+        otherwise set it to None.
+    starting_page_number
+        Indicates what page number should be assigned to the first page in the document.
+        This information will be reflected in elements' metadata and can be be especially
+        useful when partitioning a document that is part of a larger document.
     """
     exactly_one(file=file, filename=filename, url=url)
 
@@ -218,24 +266,15 @@ def partition(
             "Please use metadata_filename instead.",
         )
     kwargs.setdefault("metadata_filename", metadata_filename)
+    kwargs.setdefault("date_from_file_object", date_from_file_object)
 
-    if ocr_languages == "":
-        ocr_languages = None
+    if pdf_infer_table_structure:
+        logger.warning(
+            "The pdf_infer_table_structure kwarg is deprecated. Please use skip_infer_table_types "
+            "instead."
+        )
 
-    if ocr_languages is not None:
-        # check if languages was set to anything not the default value
-        # languages and ocr_languages were therefore both provided - raise error
-        if languages is not None:
-            raise ValueError(
-                "Only one of languages and ocr_languages should be specified. "
-                "languages is preferred. ocr_languages is marked for deprecation.",
-            )
-        else:
-            languages = convert_old_ocr_languages_to_languages(ocr_languages)
-            logger.warning(
-                "The ocr_languages kwarg will be deprecated in a future version of unstructured. "
-                "Please use languages instead.",
-            )
+    languages = check_language_args(languages or [], ocr_languages)
 
     if url is not None:
         file, filetype = file_and_type_from_url(
@@ -276,6 +315,8 @@ def partition(
             infer_table_structure=infer_table_structure,
             languages=languages,
             detect_language_per_element=detect_language_per_element,
+            starting_page_number=starting_page_number,
+            strategy=strategy,
             **kwargs,
         )
     elif filetype == FileType.DOCX:
@@ -286,6 +327,8 @@ def partition(
             infer_table_structure=infer_table_structure,
             languages=languages,
             detect_language_per_element=detect_language_per_element,
+            starting_page_number=starting_page_number,
+            strategy=strategy,
             **kwargs,
         )
     elif filetype == FileType.ODT:
@@ -296,6 +339,8 @@ def partition(
             infer_table_structure=infer_table_structure,
             languages=languages,
             detect_language_per_element=detect_language_per_element,
+            starting_page_number=starting_page_number,
+            strategy=strategy,
             **kwargs,
         )
     elif filetype == FileType.EML:
@@ -382,26 +427,37 @@ def partition(
     elif filetype == FileType.PDF:
         _partition_pdf = _get_partition_with_extras("pdf")
         elements = _partition_pdf(
-            filename=filename,  # type: ignore
-            file=file,  # type: ignore
+            filename=filename,
+            file=file,
             url=None,
             include_page_breaks=include_page_breaks,
             infer_table_structure=infer_table_structure,
             strategy=strategy,
             languages=languages,
-            extract_images_in_pdf=pdf_extract_images,
-            image_output_dir_path=pdf_image_output_dir_path,
+            hi_res_model_name=hi_res_model_name or model_name,
+            extract_images_in_pdf=extract_images_in_pdf,
+            extract_image_block_types=extract_image_block_types,
+            extract_image_block_output_dir=extract_image_block_output_dir,
+            extract_image_block_to_payload=extract_image_block_to_payload,
+            starting_page_number=starting_page_number,
             **kwargs,
         )
-    elif (filetype == FileType.PNG) or (filetype == FileType.JPG) or (filetype == FileType.TIFF):
-        elements = partition_image(
-            filename=filename,  # type: ignore
-            file=file,  # type: ignore
+    elif filetype in IMAGE_FILETYPES:
+        _partition_image = _get_partition_with_extras("image")
+        elements = _partition_image(
+            filename=filename,
+            file=file,
             url=None,
             include_page_breaks=include_page_breaks,
             infer_table_structure=infer_table_structure,
             strategy=strategy,
             languages=languages,
+            hi_res_model_name=hi_res_model_name or model_name,
+            extract_images_in_pdf=extract_images_in_pdf,
+            extract_image_block_types=extract_image_block_types,
+            extract_image_block_output_dir=extract_image_block_output_dir,
+            extract_image_block_to_payload=extract_image_block_to_payload,
+            starting_page_number=starting_page_number,
             **kwargs,
         )
     elif filetype == FileType.TXT:
@@ -434,6 +490,7 @@ def partition(
             infer_table_structure=infer_table_structure,
             languages=languages,
             detect_language_per_element=detect_language_per_element,
+            strategy=strategy,
             **kwargs,
         )
     elif filetype == FileType.PPTX:
@@ -445,6 +502,8 @@ def partition(
             infer_table_structure=infer_table_structure,
             languages=languages,
             detect_language_per_element=detect_language_per_element,
+            starting_page_number=starting_page_number,
+            strategy=strategy,
             **kwargs,
         )
     elif filetype == FileType.JSON:
@@ -462,6 +521,7 @@ def partition(
             infer_table_structure=infer_table_structure,
             languages=languages,
             detect_language_per_element=detect_language_per_element,
+            starting_page_number=starting_page_number,
             **kwargs,
         )
     elif filetype == FileType.CSV:
@@ -506,14 +566,16 @@ def partition(
 def file_and_type_from_url(
     url: str,
     content_type: Optional[str] = None,
-    headers: Dict[str, str] = {},
+    headers: dict[str, str] = {},
     ssl_verify: bool = True,
     request_timeout: Optional[int] = None,
-) -> Tuple[io.BytesIO, Optional[FileType]]:
+) -> tuple[io.BytesIO, Optional[FileType]]:
     response = requests.get(url, headers=headers, verify=ssl_verify, timeout=request_timeout)
     file = io.BytesIO(response.content)
 
-    content_type = content_type or response.headers.get("Content-Type")
+    content_type = (
+        content_type or response.headers.get("Content-Type", "").split(";")[0].strip().lower()
+    )
     encoding = response.headers.get("Content-Encoding", "utf-8")
 
     filetype = detect_filetype(file=file, content_type=content_type, encoding=encoding)
@@ -522,18 +584,16 @@ def file_and_type_from_url(
 
 def decide_table_extraction(
     filetype: Optional[FileType],
-    skip_infer_table_types: List[str],
+    skip_infer_table_types: list[str],
     pdf_infer_table_structure: bool,
 ) -> bool:
     doc_type = filetype.name.lower() if filetype else None
 
     if doc_type == "pdf":
-        if doc_type in skip_infer_table_types and pdf_infer_table_structure:
-            logger.warning(
-                f"Conflict between variables skip_infer_table_types: {skip_infer_table_types} "
-                f"and pdf_infer_table_structure: {pdf_infer_table_structure}, "
-                "please reset skip_infer_table_types to turn on table extraction for PDFs.",
-            )
-        return doc_type not in skip_infer_table_types or pdf_infer_table_structure
+        # For backwards compatibility. Ultimately we want to remove pdf_infer_table_structure
+        # completely and rely exclusively on `skip_infer_table_types` for all file types.
+        # Until then for pdf files we first check pdf_infer_table_structure and then update
+        # based on skip_infer_tables.
+        return pdf_infer_table_structure or doc_type not in skip_infer_table_types
 
     return doc_type not in skip_infer_table_types
