@@ -6,19 +6,11 @@ import subprocess
 from datetime import datetime
 from io import BufferedReader, BytesIO, TextIOWrapper
 from tempfile import SpooledTemporaryFile
-from typing import (
-    IO,
-    TYPE_CHECKING,
-    Any,
-    BinaryIO,
-    Dict,
-    List,
-    Optional,
-    Tuple,
-    Union,
-)
+from time import sleep
+from typing import IO, TYPE_CHECKING, Any, Optional, TypeVar, cast
 
 import emoji
+import psutil
 from tabulate import tabulate
 
 from unstructured.documents.coordinates import CoordinateSystem, PixelSpace
@@ -39,11 +31,8 @@ from unstructured.nlp.patterns import ENUMERATED_BULLETS_RE, UNICODE_BULLETS_RE
 from unstructured.partition.utils.constants import SORT_MODE_DONT, SORT_MODE_XY_CUT
 from unstructured.utils import dependency_exists, first
 
-if dependency_exists("docx") and dependency_exists("docx.table"):
-    from docx.table import Table as docxtable
-
 if dependency_exists("pptx") and dependency_exists("pptx.table"):
-    from pptx.table import Table as pptxtable
+    from pptx.table import Table as PptxTable
 
 if dependency_exists("numpy") and dependency_exists("cv2"):
     from unstructured.partition.utils.sorting import sort_page_elements
@@ -52,7 +41,6 @@ if TYPE_CHECKING:
     from unstructured_inference.inference.layout import DocumentLayout, PageLayout
     from unstructured_inference.inference.layoutelement import LayoutElement
 
-
 HIERARCHY_RULE_SET = {
     "Title": [
         "Text",
@@ -80,35 +68,56 @@ HIERARCHY_RULE_SET = {
 }
 
 
-def get_last_modified_date(filename: str) -> Union[str, None]:
+def get_last_modified(
+    filename: str | None, file: IO[bytes] | None, date_from_file_object: bool
+) -> str | None:
+    """Determine best available last-modified date from file or filename."""
+    if filename is not None:
+        return get_last_modified_date(filename)
+
+    if file is not None:
+        return get_last_modified_date_from_file(file) if date_from_file_object else None
+
+    return None
+
+
+def get_last_modified_date(filename: str) -> Optional[str]:
+    """Modification time of file at path `filename`, if it exists.
+
+    Returns `None` when `filename` is not a path to a file on the local filesystem.
+
+    Otherwise returns date and time in ISO 8601 string format (YYYY-MM-DDTHH:MM:SS) like
+    "2024-03-05T17:02:53".
+    """
+    if not os.path.isfile(filename):
+        return None
+
     modify_date = datetime.fromtimestamp(os.path.getmtime(filename))
     return modify_date.strftime("%Y-%m-%dT%H:%M:%S%z")
 
 
-def get_last_modified_date_from_file(
-    file: Union[IO[bytes], SpooledTemporaryFile[bytes], BinaryIO, bytes],
-) -> Union[str, None]:
-    filename = None
-    if hasattr(file, "name"):
-        filename = file.name
+def get_last_modified_date_from_file(file: IO[bytes] | bytes) -> Optional[str]:
+    """Modified timestamp of `file` if it corresponds to a file on the local filesystem."""
+    # -- a file-like object will have a name attribute if created by `open()` or if a name is
+    # -- assigned to it for metadata purposes. Use "" as default because the empty string is never
+    # -- a path to an actual file.
+    filename = str(getattr(file, "name", ""))
 
-    if not filename:
+    # -- there's no guarantee the path corresponds to an actual file on the filesystem. In
+    # -- particular, a user can set the `.name` attribute of an e.g. `io.BytesIO` object to
+    # -- populate the `.metadata.filename` fields for a payload perhaps downloaded via HTTP.
+    if not os.path.isfile(filename):
         return None
 
-    modify_date = get_last_modified_date(filename)
-    return modify_date
+    return get_last_modified_date(filename)
 
 
 def normalize_layout_element(
-    layout_element: Union[
-        "LayoutElement",
-        Element,
-        Dict[str, Any],
-    ],
+    layout_element: LayoutElement | Element | dict[str, Any],
     coordinate_system: Optional[CoordinateSystem] = None,
     infer_list_items: bool = True,
     source_format: Optional[str] = "html",
-) -> Union[Element, List[Element]]:
+) -> Element | list[Element]:
     """Converts an unstructured_inference LayoutElement object to an unstructured Element."""
 
     if isinstance(layout_element, Element) and source_format == "html":
@@ -123,13 +132,13 @@ def normalize_layout_element(
     else:
         layout_dict = layout_element
 
-    text = layout_dict.get("text")
+    text = layout_dict.get("text", "")
     # Both `coordinates` and `coordinate_system` must be present
     # in order to add coordinates metadata to the element.
     coordinates = layout_dict.get("coordinates")
     element_type = layout_dict.get("type")
     prob = layout_dict.get("prob")
-    aux_origin = layout_dict["source"] if "source" in layout_dict else None
+    aux_origin = layout_dict.get("source", None)
     origin = None
     if aux_origin:
         origin = aux_origin.value
@@ -137,78 +146,74 @@ def normalize_layout_element(
         class_prob_metadata = ElementMetadata(detection_class_prob=float(prob))  # type: ignore
     else:
         class_prob_metadata = ElementMetadata()
+    common_kwargs = {
+        "coordinates": coordinates,
+        "coordinate_system": coordinate_system,
+        "metadata": class_prob_metadata,
+        "detection_origin": origin,
+    }
     if element_type == ElementType.LIST:
         if infer_list_items:
             return layout_list_to_list_items(
                 text,
-                coordinates=coordinates,
-                coordinate_system=coordinate_system,
-                metadata=class_prob_metadata,
-                detection_origin=origin,
+                **common_kwargs,
             )
         else:
             return ListItem(
-                text=text if text else "",
-                coordinates=coordinates,
-                coordinate_system=coordinate_system,
-                metadata=class_prob_metadata,
-                detection_origin=origin,
+                text=text,
+                **common_kwargs,
             )
 
     elif element_type in TYPE_TO_TEXT_ELEMENT_MAP:
+        assert isinstance(element_type, str)  # Added to resolve type-error
         _element_class = TYPE_TO_TEXT_ELEMENT_MAP[element_type]
         _element_class = _element_class(
             text=text,
-            coordinates=coordinates,
-            coordinate_system=coordinate_system,
-            metadata=class_prob_metadata,
-            detection_origin=origin,
+            **common_kwargs,
         )
         if element_type == ElementType.HEADLINE:
             _element_class.metadata.category_depth = 1
         elif element_type == ElementType.SUB_HEADLINE:
             _element_class.metadata.category_depth = 2
         return _element_class
-    elif element_type == ElementType.CHECKED:
+    elif element_type in [
+        ElementType.CHECK_BOX_CHECKED,
+        ElementType.CHECK_BOX_UNCHECKED,
+        ElementType.RADIO_BUTTON_CHECKED,
+        ElementType.RADIO_BUTTON_UNCHECKED,
+        ElementType.CHECKED,
+        ElementType.UNCHECKED,
+    ]:
+        checked = element_type in [
+            ElementType.CHECK_BOX_CHECKED,
+            ElementType.RADIO_BUTTON_CHECKED,
+            ElementType.CHECKED,
+        ]
         return CheckBox(
-            checked=True,
-            coordinates=coordinates,
-            coordinate_system=coordinate_system,
-            metadata=class_prob_metadata,
-            detection_origin=origin,
-        )
-    elif element_type == ElementType.UNCHECKED:
-        return CheckBox(
-            checked=False,
-            coordinates=coordinates,
-            coordinate_system=coordinate_system,
-            metadata=class_prob_metadata,
-            detection_origin=origin,
+            checked=checked,
+            **common_kwargs,
         )
     else:
         return Text(
-            text=text if text else "",
-            coordinates=coordinates,
-            coordinate_system=coordinate_system,
-            metadata=class_prob_metadata,
-            detection_origin=origin,
+            text=text,
+            **common_kwargs,
         )
 
 
 def layout_list_to_list_items(
     text: Optional[str],
-    coordinates: Optional[Tuple[Tuple[float, float], ...]],
+    coordinates: Optional[tuple[tuple[float, float], ...]],
     coordinate_system: Optional[CoordinateSystem],
-    metadata=Optional[ElementMetadata],
-    detection_origin=Optional[str],
-) -> List[Element]:
+    metadata: Optional[ElementMetadata],
+    detection_origin: Optional[str],
+) -> list[Element]:
     """Converts a list LayoutElement to a list of ListItem elements."""
     split_items = ENUMERATED_BULLETS_RE.split(text) if text else []
     # NOTE(robinson) - this means there wasn't a match for the enumerated bullets
     if len(split_items) == 1:
         split_items = UNICODE_BULLETS_RE.split(text) if text else []
 
-    list_items: List[Element] = []
+    list_items: list[Element] = []
     for text_segment in split_items:
         if len(text_segment.strip()) > 0:
             # Both `coordinates` and `coordinate_system` must be present
@@ -226,14 +231,13 @@ def layout_list_to_list_items(
 
 
 def set_element_hierarchy(
-    elements: List[Element],
-    ruleset: Dict[str, List[str]] = HIERARCHY_RULE_SET,
-) -> List[Element]:
+    elements: list[Element], ruleset: dict[str, list[str]] = HIERARCHY_RULE_SET
+) -> list[Element]:
     """Sets the parent_id for each element in the list of elements
     based on the element's category, depth and a ruleset
 
     """
-    stack: List[Element] = []
+    stack: list[Element] = []
     for element in elements:
         if element.metadata.parent_id is not None:
             continue
@@ -274,23 +278,24 @@ def set_element_hierarchy(
     return elements
 
 
-def _add_element_metadata(
+def add_element_metadata(
     element: Element,
     filename: Optional[str] = None,
     filetype: Optional[str] = None,
     page_number: Optional[int] = None,
     url: Optional[str] = None,
     text_as_html: Optional[str] = None,
-    coordinates: Optional[Tuple[Tuple[float, float], ...]] = None,
+    coordinates: Optional[tuple[tuple[float, float], ...]] = None,
     coordinate_system: Optional[CoordinateSystem] = None,
-    section: Optional[str] = None,
     image_path: Optional[str] = None,
     detection_origin: Optional[str] = None,
-    languages: Optional[List[str]] = None,
-    **kwargs,
+    languages: Optional[list[str]] = None,
+    **kwargs: Any,
 ) -> Element:
-    """Adds document metadata to the document element. Document metadata includes information
-    like the filename, source url, and page number."""
+    """Adds document metadata to the document element.
+
+    Document metadata includes information like the filename, source url, and page number.
+    """
 
     coordinates_metadata = (
         CoordinatesMetadata(
@@ -303,6 +308,7 @@ def _add_element_metadata(
     links = element.links if hasattr(element, "links") and len(element.links) > 0 else None
     link_urls = [link.get("url") for link in links] if links else None
     link_texts = [link.get("text") for link in links] if links else None
+    link_start_indexes = [link.get("start_index") for link in links] if links else None
     emphasized_texts = (
         element.emphasized_texts
         if hasattr(element, "emphasized_texts") and len(element.emphasized_texts) > 0
@@ -329,9 +335,9 @@ def _add_element_metadata(
         text_as_html=text_as_html,
         link_urls=link_urls,
         link_texts=link_texts,
+        link_start_indexes=link_start_indexes,
         emphasized_text_contents=emphasized_text_contents,
         emphasized_text_tags=emphasized_text_tags,
-        section=section,
         category_depth=depth,
         image_path=image_path,
         languages=languages,
@@ -342,13 +348,12 @@ def _add_element_metadata(
     return element
 
 
-def _remove_element_metadata(
-    layout_elements,
-) -> List[Element]:
-    """Removes document metadata from the document element. Document metadata includes information
-    like the filename, source url, and page number."""
-    # Init an empty list of elements to write to
-    elements: List[Element] = []
+def remove_element_metadata(layout_elements) -> list[Element]:
+    """Removes document metadata from the document element.
+
+    Document metadata includes information like the filename, source url, and page number.
+    """
+    elements: list[Element] = []
     metadata = ElementMetadata()
     for layout_element in layout_elements:
         element = normalize_layout_element(layout_element)
@@ -362,11 +367,22 @@ def _remove_element_metadata(
     return elements
 
 
+def _is_soffice_running():
+    for proc in psutil.process_iter():
+        try:
+            if "soffice" in proc.name().lower():
+                return True
+        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
+            pass
+    return False
+
+
 def convert_office_doc(
     input_filename: str,
     output_directory: str,
     target_format: str = "docx",
     target_filter: Optional[str] = None,
+    wait_for_soffice_ready_time_out: int = 10,
 ):
     """Converts a .doc file to a .docx file using the libreoffice CLI.
 
@@ -381,6 +397,8 @@ def convert_office_doc(
     target_filter: str
         The output filter name to use when converting. See references below
         for details.
+    wait_for_soffice_ready_time_out: int
+        The max wait time in seconds for soffice to become available to run
 
     References
     ----------
@@ -404,12 +422,21 @@ def convert_office_doc(
         input_filename,
     ]
     try:
-        process = subprocess.Popen(
-            command,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-        )
-        output, error = process.communicate()
+        # only one soffice process can be ran
+        wait_time = 0
+        sleep_time = 0.1
+        output = subprocess.run(command, capture_output=True)
+        message = output.stdout.decode().strip()
+        # we can't rely on returncode unfortunately because on macOS it would return 0 even when the
+        # command failed to run; instead we have to rely on the stdout being empty as a sign of the
+        # process failed
+        while (wait_time < wait_for_soffice_ready_time_out) and (message == ""):
+            wait_time += sleep_time
+            if _is_soffice_running():
+                sleep(sleep_time)
+            else:
+                output = subprocess.run(command, capture_output=True)
+                message = output.stdout.decode().strip()
     except FileNotFoundError:
         raise FileNotFoundError(
             """soffice command was not found. Please install libreoffice
@@ -420,9 +447,12 @@ on your system and try again.
 - Debian: https://wiki.debian.org/LibreOffice""",
         )
 
-    logger.info(output.decode().strip())
-    if error:
-        logger.error(error.decode().strip())
+    logger.info(message)
+    if output.returncode != 0 or message == "":
+        logger.error(
+            "soffice failed to convert to format %s with code %i", target_format, output.returncode
+        )
+        logger.error(output.stderr.decode().strip())
 
 
 def exactly_one(**kwargs: Any) -> None:
@@ -441,62 +471,70 @@ def exactly_one(**kwargs: Any) -> None:
         raise ValueError(message)
 
 
-def spooled_to_bytes_io_if_needed(
-    file_obj: Optional[Union[bytes, BinaryIO, SpooledTemporaryFile[bytes]]],
-) -> Optional[Union[bytes, BinaryIO]]:
-    if isinstance(file_obj, SpooledTemporaryFile):
-        file_obj.seek(0)
-        contents = file_obj.read()
-        return BytesIO(contents)
-    else:
-        # Return the original file object if it's not a SpooledTemporaryFile
-        return file_obj
+_T = TypeVar("_T")
 
 
-def convert_to_bytes(
-    file: Optional[Union[bytes, SpooledTemporaryFile, IO[bytes]]] = None,
-) -> bytes:
+def spooled_to_bytes_io_if_needed(file: _T | SpooledTemporaryFile[bytes]) -> _T | BytesIO:
+    """Convert `file` to `BytesIO` when it is a `SpooledTemporaryFile`.
+
+    Note that `file` does not need to be IO[bytes]. It can be `None` or `bytes` and this function
+    will not complain.
+
+    In Python <3.11, `SpooledTemporaryFile` does not implement `.readable()` or `.seekable()` which
+    triggers an exception when the file is loaded by certain packages. In particular, the stdlib
+    `zipfile.Zipfile` raises on opening a `SpooledTemporaryFile` as does `Pandas.read_csv()`.
+    """
+    if isinstance(file, SpooledTemporaryFile):
+        file.seek(0)
+        return BytesIO(cast(bytes, file.read()))
+
+    # -- return `file` unchanged otherwise --
+    return file
+
+
+def convert_to_bytes(file: bytes | IO[bytes]) -> bytes:
+    """Extract the bytes from `file` without preventing it from being read again later.
+
+    As a convenience to simplify client code, also returns `file` unchanged if it is already bytes.
+    """
     if isinstance(file, bytes):
-        f_bytes = file
-    elif isinstance(file, SpooledTemporaryFile):
+        return file
+
+    if isinstance(file, SpooledTemporaryFile):
         file.seek(0)
         f_bytes = file.read()
         file.seek(0)
-    elif isinstance(file, BytesIO):
-        f_bytes = file.getvalue()
-    elif isinstance(file, (TextIOWrapper, BufferedReader)):
+        return f_bytes
+
+    if isinstance(file, BytesIO):
+        return file.getvalue()
+
+    if isinstance(file, (TextIOWrapper, BufferedReader)):
         with open(file.name, "rb") as f:
-            f_bytes = f.read()
-    else:
-        raise ValueError("Invalid file-like object type")
+            return f.read()
 
-    return f_bytes
+    raise ValueError("Invalid file-like object type")
 
 
-def convert_ms_office_table_to_text(
-    table: Union["docxtable", "pptxtable"],
-    as_html: bool = True,
-) -> str:
-    """
-    Convert a table object from a Word document to an HTML table string using the tabulate library.
+def convert_ms_office_table_to_text(table: PptxTable, as_html: bool = True) -> str:
+    """Convert a PPTX table object to an HTML table string using the tabulate library.
 
     Args:
-        table (Table): A docx.table.Table object.
+        table (Table): A pptx.table.Table object.
         as_html (bool): Whether to return the table as an HTML string (True) or a
             plain text string (False)
 
     Returns:
         str: An table string representation of the input table.
     """
-    fmt = "html" if as_html else "plain"
     rows = list(table.rows)
-    if len(rows) > 0:
-        headers = [cell.text for cell in rows[0].cells]
-        data = [[cell.text for cell in row.cells] for row in rows[1:]]
-        table_text = tabulate(data, headers=headers, tablefmt=fmt)
-    else:
-        table_text = ""
-    return table_text
+
+    if not rows:
+        return ""
+
+    headers = [cell.text for cell in rows[0].cells]
+    data = [[cell.text for cell in row.cells] for row in rows[1:]]
+    return tabulate(data, headers=headers, tablefmt="html" if as_html else "plain")
 
 
 def contains_emoji(s: str) -> bool:
@@ -513,9 +551,7 @@ def contains_emoji(s: str) -> bool:
     return bool(emoji.emoji_count(s))
 
 
-def _get_page_image_metadata(
-    page: PageLayout,
-) -> dict:
+def _get_page_image_metadata(page: PageLayout) -> dict[str, Any]:
     """Retrieve image metadata and coordinate system from a page."""
 
     image = getattr(page, "image", None)
@@ -541,8 +577,13 @@ def _get_page_image_metadata(
     }
 
 
+# FIXME: document here can be either DocumentLayout or HTMLDocument; HTMLDocument is defined in
+# unstructured.documents.html, which imports this module so we can't import the class for type
+# hints. Moreover, those two types of documents have different lists of attributes
+# UPDATE(scanny): HTMLDocument no longer uses this function, so it can be optimized for use by
+# DocumentLayout only.
 def document_to_element_list(
-    document: "DocumentLayout",
+    document: DocumentLayout,
     sortable: bool = False,
     include_page_breaks: bool = False,
     last_modification_date: Optional[str] = None,
@@ -550,22 +591,23 @@ def document_to_element_list(
     source_format: Optional[str] = None,
     detection_origin: Optional[str] = None,
     sort_mode: str = SORT_MODE_XY_CUT,
-    languages: Optional[List[str]] = None,
-    **kwargs,
-) -> List[Element]:
+    languages: Optional[list[str]] = None,
+    starting_page_number: int = 1,
+    **kwargs: Any,
+) -> list[Element]:
     """Converts a DocumentLayout object to a list of unstructured elements."""
-    elements: List[Element] = []
+    elements: list[Element] = []
 
     num_pages = len(document.pages)
-    for i, page in enumerate(document.pages):
-        page_elements: List[Element] = []
+    for page_number, page in enumerate(document.pages, start=starting_page_number):
+        page_elements: list[Element] = []
 
         page_image_metadata = _get_page_image_metadata(page)
         image_format = page_image_metadata.get("format")
         image_width = page_image_metadata.get("width")
         image_height = page_image_metadata.get("height")
 
-        translation_mapping: List[Tuple["LayoutElement", Element]] = []
+        translation_mapping: list[tuple["LayoutElement", Element]] = []
         for layout_element in page.elements:
             if image_width and image_height and hasattr(layout_element.bbox, "coordinates"):
                 coordinate_system = PixelSpace(width=image_width, height=image_height)
@@ -578,27 +620,29 @@ def document_to_element_list(
                 infer_list_items=infer_list_items,
                 source_format=source_format if source_format else "html",
             )
-            if isinstance(element, List):
+            if isinstance(element, list):
                 for el in element:
                     if last_modification_date:
                         el.metadata.last_modified = last_modification_date
-                    el.metadata.page_number = i + 1
+                    el.metadata.page_number = page_number
                 page_elements.extend(element)
                 translation_mapping.extend([(layout_element, el) for el in element])
                 continue
             else:
                 if last_modification_date:
                     element.metadata.last_modified = last_modification_date
-                element.metadata.text_as_html = (
-                    layout_element.text_as_html if hasattr(layout_element, "text_as_html") else None
-                )
-                try:
-                    if (
-                        isinstance(element, Title) and element.metadata.category_depth is None
-                    ) and any(el.type in ["Headline", "Subheadline"] for el in page.elements):
-                        element.metadata.category_depth = 0
-                except AttributeError:
-                    logger.info("HTML element instance has no attribute type")
+                element.metadata.text_as_html = getattr(layout_element, "text_as_html", None)
+                element.metadata.table_as_cells = getattr(layout_element, "table_as_cells", None)
+                # FIXME: here the elements in a page can be either:
+                # 1. LayoutElement if the document is LayoutDocument (if the partition is on a
+                #   pdf/image);
+                # 2. Element if the document is HTMLDocument (if the partition is on an html file)
+                # this discrepency is due to Element class defined in unstructured and LayoutElement
+                # class defined in unstructured_inference do not have the same list of attributes
+                if (isinstance(element, Title) and element.metadata.category_depth is None) and any(
+                    getattr(el, "type", "") in ["Headline", "Subheadline"] for el in page.elements
+                ):
+                    element.metadata.category_depth = 0
 
                 page_elements.append(element)
                 translation_mapping.append((layout_element, element))
@@ -610,9 +654,9 @@ def document_to_element_list(
                 layout_element.image_path if hasattr(layout_element, "image_path") else None
             )
 
-            _add_element_metadata(
+            add_element_metadata(
                 element,
-                page_number=i + 1,
+                page_number=page_number,
                 filetype=image_format,
                 coordinates=coordinates,
                 coordinate_system=coordinate_system,
@@ -633,7 +677,7 @@ def document_to_element_list(
         if sortable and sort_mode != SORT_MODE_DONT:
             sorted_page_elements = sort_page_elements(page_elements, sort_mode)
 
-        if include_page_breaks and i < num_pages - 1:
+        if include_page_breaks and page_number < num_pages + starting_page_number:
             sorted_page_elements.append(PageBreak(text=""))
         elements.extend(sorted_page_elements)
 
@@ -641,17 +685,17 @@ def document_to_element_list(
 
 
 def ocr_data_to_elements(
-    ocr_data: List["LayoutElement"],
-    image_size: Tuple[Union[int, float], Union[int, float]],
+    ocr_data: list["LayoutElement"],
+    image_size: tuple[int | float, int | float],
     common_metadata: Optional[ElementMetadata] = None,
     infer_list_items: bool = True,
     source_format: Optional[str] = None,
-) -> List[Element]:
+) -> list[Element]:
     """Convert OCR layout data into `unstructured` elements with associated metadata."""
 
     image_width, image_height = image_size
     coordinate_system = PixelSpace(width=image_width, height=image_height)
-    elements = []
+    elements: list[Element] = []
     for layout_element in ocr_data:
         element = normalize_layout_element(
             layout_element,
