@@ -1,29 +1,31 @@
-from typing import IO, List, Optional
+from __future__ import annotations
 
-from unstructured.chunking.title import add_chunking_strategy
+from typing import IO, Any, Optional
+
+from unstructured.chunking import add_chunking_strategy
 from unstructured.documents.elements import Element, process_metadata
+from unstructured.file_utils.file_conversion import convert_file_to_html_text_using_pandoc
 from unstructured.file_utils.filetype import FileType, add_metadata_with_filetype
-from unstructured.partition.html import convert_and_partition_html
-from unstructured.partition.lang import apply_lang_metadata
+from unstructured.partition.common import exactly_one, get_last_modified
+from unstructured.partition.html import partition_html
 
 DETECTION_ORIGIN: str = "epub"
 
 
 @process_metadata()
 @add_metadata_with_filetype(FileType.EPUB)
-@add_chunking_strategy()
+@add_chunking_strategy
 def partition_epub(
     filename: Optional[str] = None,
+    *,
     file: Optional[IO[bytes]] = None,
-    include_page_breaks: bool = False,
-    include_metadata: bool = True,
     metadata_filename: Optional[str] = None,
     metadata_last_modified: Optional[str] = None,
-    chunking_strategy: Optional[str] = None,
-    languages: Optional[List[str]] = ["auto"],
+    languages: Optional[list[str]] = ["auto"],
     detect_language_per_element: bool = False,
-    **kwargs,
-) -> List[Element]:
+    date_from_file_object: bool = False,
+    **kwargs: Any,
+) -> list[Element]:
     """Partitions an EPUB document. The document is first converted to HTML and then
     partitioned using partition_html.
 
@@ -33,8 +35,6 @@ def partition_epub(
         A string defining the target filename path.
     file
         A file-like object using "rb" mode --> open(filename, "rb").
-    include_page_breaks
-        If True, the output will include page breaks if the filetype supports it
     metadata_last_modified
         The last modified date for the document.
     languages
@@ -44,24 +44,24 @@ def partition_epub(
         Additional Parameters:
             detect_language_per_element
                 Detect language per element instead of at the document level.
+    date_from_file_object
+        Applies only when providing file via `file` parameter. If this option is True, attempt
+        infer last_modified metadata from bytes, otherwise set it to None.
     """
+    exactly_one(filename=filename, file=file)
 
-    elements = convert_and_partition_html(
-        filename=filename,
-        file=file,
-        include_page_breaks=include_page_breaks,
+    html_text = convert_file_to_html_text_using_pandoc(
+        source_format="epub", filename=filename, file=file
+    )
+
+    return partition_html(
+        text=html_text,
+        encoding="unicode",
         metadata_filename=metadata_filename,
-        metadata_last_modified=metadata_last_modified,
-        source_format="epub",
+        metadata_last_modified=(
+            metadata_last_modified or get_last_modified(filename, file, date_from_file_object)
+        ),
+        languages=languages,
+        detect_language_per_element=detect_language_per_element,
         detection_origin=DETECTION_ORIGIN,
     )
-
-    elements = list(
-        apply_lang_metadata(
-            elements,
-            languages=languages,
-            detect_language_per_element=detect_language_per_element,
-        ),
-    )
-
-    return elements
