@@ -1,11 +1,12 @@
-from typing import IO, List, Optional, Union
+from __future__ import annotations
+
+from typing import IO, Any, Optional, Union
 
 import markdown
 import requests
 
-from unstructured.chunking.title import add_chunking_strategy
+from unstructured.chunking import add_chunking_strategy
 from unstructured.documents.elements import Element, process_metadata
-from unstructured.documents.xml import VALID_PARSERS
 from unstructured.file_utils.filetype import FileType, add_metadata_with_filetype
 from unstructured.partition.common import (
     exactly_one,
@@ -26,7 +27,7 @@ DETECTION_ORIGIN: str = "md"
 
 @process_metadata()
 @add_metadata_with_filetype(FileType.MD)
-@add_chunking_strategy()
+@add_chunking_strategy
 def partition_md(
     filename: Optional[str] = None,
     file: Optional[IO[bytes]] = None,
@@ -34,14 +35,14 @@ def partition_md(
     url: Optional[str] = None,
     include_page_breaks: bool = False,
     include_metadata: bool = True,
-    parser: VALID_PARSERS = None,
     metadata_filename: Optional[str] = None,
     metadata_last_modified: Optional[str] = None,
     chunking_strategy: Optional[str] = None,
-    languages: Optional[List[str]] = ["auto"],
+    languages: Optional[list[str]] = ["auto"],
     detect_language_per_element: bool = False,
-    **kwargs,
-) -> List[Element]:
+    date_from_file_object: bool = False,
+    **kwargs: Any,
+) -> list[Element]:
     """Partitions a markdown file into its constituent elements
 
     Parameters
@@ -69,6 +70,9 @@ def partition_md(
         Additional Parameters:
             detect_language_per_element
                 Detect language per element instead of at the document level.
+    date_from_file_object
+        Applies only when providing file via `file` parameter. If this option is True, attempt
+        infer last_modified metadata from bytes, otherwise set it to None.
     """
     # Verify that only one of the arguments was provided
     if text is None:
@@ -83,7 +87,9 @@ def partition_md(
             text = optional_decode(f.read())
 
     elif file is not None:
-        last_modification_date = get_last_modified_date_from_file(file)
+        last_modification_date = (
+            get_last_modified_date_from_file(file) if date_from_file_object else None
+        )
         text = optional_decode(file.read())
 
     elif url is not None:
@@ -105,7 +111,6 @@ def partition_md(
         text=html,
         include_page_breaks=include_page_breaks,
         include_metadata=include_metadata,
-        parser=parser,
         source_format="md",
         metadata_filename=metadata_filename,
         metadata_last_modified=metadata_last_modified or last_modification_date,
