@@ -1,9 +1,11 @@
+from __future__ import annotations
+
 import copy
 import re
 import textwrap
-from typing import IO, Any, Callable, List, Optional, Tuple
+from typing import IO, Any, Callable, Literal, Optional
 
-from unstructured.chunking.title import add_chunking_strategy
+from unstructured.chunking import add_chunking_strategy
 from unstructured.cleaners.core import (
     auto_paragraph_grouper,
     clean_bullets,
@@ -14,6 +16,8 @@ from unstructured.documents.elements import (
     Element,
     ElementMetadata,
     EmailAddress,
+    Footer,
+    Header,
     ListItem,
     NarrativeText,
     Text,
@@ -45,18 +49,19 @@ def partition_text(
     file: Optional[IO[bytes]] = None,
     text: Optional[str] = None,
     encoding: Optional[str] = None,
-    paragraph_grouper: Optional[Callable[[str], str]] = None,
+    paragraph_grouper: Optional[Callable[[str], str]] | Literal[False] = None,
     metadata_filename: Optional[str] = None,
     include_metadata: bool = True,
-    languages: Optional[List[str]] = ["auto"],
+    languages: Optional[list[str]] = ["auto"],
     max_partition: Optional[int] = 1500,
     min_partition: Optional[int] = 0,
     metadata_last_modified: Optional[str] = None,
     chunking_strategy: Optional[str] = None,
     detect_language_per_element: bool = False,
     detection_origin: Optional[str] = "text",
+    date_from_file_object: bool = False,
     **kwargs: Any,
-) -> List[Element]:
+) -> list[Element]:
     """Partitions an .txt documents into its constituent paragraph elements.
     If paragraphs are below "min_partition" or above "max_partition" boundaries,
     they are combined or split.
@@ -89,6 +94,9 @@ def partition_text(
         The minimum number of characters to include in a partition.
     metadata_last_modified
         The day of the last modification
+    date_from_file_object
+        Applies only when providing file via `file` parameter. If this option is True, attempt
+        infer last_modified metadata from bytes, otherwise set it to None.
     """
     return _partition_text(
         filename=filename,
@@ -105,30 +113,32 @@ def partition_text(
         chunking_strategy=chunking_strategy,
         detect_language_per_element=detect_language_per_element,
         detection_origin=detection_origin,
+        date_from_file_object=date_from_file_object,
         **kwargs,
     )
 
 
 @process_metadata()
 @add_metadata_with_filetype(FileType.TXT)
-@add_chunking_strategy()
+@add_chunking_strategy
 def _partition_text(
     filename: Optional[str] = None,
     file: Optional[IO[bytes]] = None,
     text: Optional[str] = None,
     encoding: Optional[str] = None,
-    paragraph_grouper: Optional[Callable[[str], str]] = None,
+    paragraph_grouper: Optional[Callable[[str], str]] | Literal[False] = None,
     metadata_filename: Optional[str] = None,
     include_metadata: bool = True,
-    languages: Optional[List[str]] = ["auto"],
+    languages: Optional[list[str]] = ["auto"],
     max_partition: Optional[int] = 1500,
     min_partition: Optional[int] = 0,
     metadata_last_modified: Optional[str] = None,
     chunking_strategy: Optional[str] = None,
     detect_language_per_element: bool = False,
     detection_origin: Optional[str] = "text",
+    date_from_file_object: bool = False,
     **kwargs: Any,
-) -> List[Element]:
+) -> list[Element]:
     """internal API for `partition_text`"""
     if text is not None and text.strip() == "" and not file and not filename:
         return []
@@ -151,8 +161,9 @@ def _partition_text(
 
     elif file is not None:
         encoding, file_text = read_txt_file(file=file, encoding=encoding)
-        last_modification_date = get_last_modified_date_from_file(file)
-
+        last_modification_date = (
+            get_last_modified_date_from_file(file) if date_from_file_object else None
+        )
     elif text is not None:
         file_text = str(text)
 
@@ -172,7 +183,7 @@ def _partition_text(
         max_partition=max_partition,
     )
 
-    elements: List[Element] = []
+    elements: list[Element] = []
     if include_metadata:
         metadata = ElementMetadata(
             filename=metadata_filename or filename,
@@ -206,12 +217,60 @@ def is_empty_bullet(text: str) -> bool:
     return UNICODE_BULLETS_RE.match(text) and len(text) == 1
 
 
+def _get_height_percentage(
+    coordinates: Optional[tuple[tuple[float, float], ...]] = None,
+    coordinate_system: Optional[CoordinateSystem] = None,
+) -> float:
+    avg_y = sum([coordinate[1] for coordinate in coordinates]) / len(coordinates)
+    return avg_y / coordinate_system.height
+
+
+def is_in_header_position(
+    coordinates: Optional[tuple[tuple[float, float], ...]] = None,
+    coordinate_system: Optional[CoordinateSystem] = None,
+    threshold: float = 0.07,
+) -> bool:
+    """Checks to see if the position of the text indicates that the text belongs
+    to a header."""
+    if coordinates is None or coordinate_system is None:
+        return False
+
+    height_percentage = _get_height_percentage(coordinates, coordinate_system)
+    return height_percentage < threshold
+
+
+def is_in_footer_position(
+    coordinates: Optional[tuple[tuple[float, float], ...]] = None,
+    coordinate_system: Optional[CoordinateSystem] = None,
+    threshold: float = 0.93,
+) -> bool:
+    """Checks to see if the position of the text indicates that the text belongs
+    to a footer."""
+    if coordinates is None or coordinate_system is None:
+        return False
+
+    height_percentage = _get_height_percentage(coordinates, coordinate_system)
+    return height_percentage > threshold
+
+
 def element_from_text(
     text: str,
-    coordinates: Optional[Tuple[Tuple[float, float], ...]] = None,
+    coordinates: Optional[tuple[tuple[float, float], ...]] = None,
     coordinate_system: Optional[CoordinateSystem] = None,
 ) -> Element:
-    if is_bulleted_text(text):
+    if is_in_header_position(coordinates, coordinate_system):
+        return Header(
+            text=text,
+            coordinates=coordinates,
+            coordinate_system=coordinate_system,
+        )
+    elif is_in_footer_position(coordinates, coordinate_system):
+        return Footer(
+            text=text,
+            coordinates=coordinates,
+            coordinate_system=coordinate_system,
+        )
+    elif is_bulleted_text(text):
         clean_text = clean_bullets(text)
         return ListItem(
             text=clean_text,
@@ -253,17 +312,17 @@ def element_from_text(
 
 
 def _combine_paragraphs_less_than_min(
-    split_paragraphs: List[str],
+    split_paragraphs: list[str],
     max_partition: Optional[int] = 1500,
     min_partition: Optional[int] = 0,
-) -> List[str]:
+) -> list[str]:
     """Combine paragraphs less than `min_partition` while not exceeding `max_partition`."""
     min_partition = min_partition or 0
     max_possible_partition = len(" ".join(split_paragraphs))
     max_partition = max_partition or max_possible_partition
 
-    combined_paras: List[str] = []
-    combined_idxs: List[int] = []
+    combined_paras: list[str] = []
+    combined_idxs: list[int] = []
     for i, para in enumerate(split_paragraphs):
         if i in combined_idxs:
             continue
@@ -291,11 +350,11 @@ def _split_by_paragraph(
     file_text: str,
     min_partition: Optional[int] = 0,
     max_partition: Optional[int] = 1500,
-) -> List[str]:
+) -> list[str]:
     """Split text into paragraphs that fit within the `min_` and `max_partition` window."""
     paragraphs = re.split(PARAGRAPH_PATTERN, file_text.strip())
 
-    split_paragraphs: List[str] = []
+    split_paragraphs: list[str] = []
     for paragraph in paragraphs:
         split_paragraphs.extend(
             _split_content_to_fit_max(
@@ -313,7 +372,7 @@ def _split_by_paragraph(
     return combined_paragraphs
 
 
-def _split_content_size_n(content: str, n: int) -> List[str]:
+def _split_content_size_n(content: str, n: int) -> list[str]:
     """Splits a section of content into chunks that are at most
     size n without breaking apart words."""
     segments = []
@@ -327,11 +386,11 @@ def _split_content_size_n(content: str, n: int) -> List[str]:
 def _split_content_to_fit_max(
     content: str,
     max_partition: Optional[int] = 1500,
-) -> List[str]:
+) -> list[str]:
     """Splits a paragraph or section of content so that all of the elements fit into the
     max partition window."""
     sentences = sent_tokenize(content)
-    chunks: List[str] = []
+    chunks: list[str] = []
     tmp_chunk = ""
     # Initialize an empty string to collect sentence segments (`tmp_chunk`).
     for sentence in sentences:
@@ -368,7 +427,7 @@ def _split_content_to_fit_max(
 def _split_in_half_at_breakpoint(
     content: str,
     breakpoint: str = " ",
-) -> List[str]:
+) -> list[str]:
     """Splits a segment of content at the breakpoint closest to the middle"""
     mid = len(content) // 2
     for i in range(len(content) // 2):
